#!/usr/bin/python
########################################################################
#
# File Name:            initDbDom.py
#
# Documentation:        http://docs.4suite.com/DbDom/initDbDom.py.html
#
"""
WWW: http://4suite.com/4DOM         e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""


import string

from Ft.DbDom import Document
from Ft.DbDom import Node
from Ft.DbDom import Document
from Ft.DbDom import Element
from Ft.DbDom import Attribute
from Ft.DbDom import Text
from Ft.DbDom import Comment
from Ft.DbDom import ProcessingInstruction
from Ft.DbDom import DocumentFragment
from Ft.Ods.Collections import LiteralCollection 


#ExceptionCode
#FIXME This will probably 
INDEX_SIZE_ERR                 = 1
DOMSTRING_SIZE_ERR             = 2
HIERARCHY_REQUEST_ERR          = 3
WRONG_DOCUMENT_ERR             = 4
INVALID_CHARACTER_ERR          = 5
NO_DATA_ALLOWED_ERR            = 6
NO_MODIFICATION_ALLOWED_ERR    = 7
NOT_FOUND_ERR                  = 8
NOT_SUPPORTED_ERR              = 9
INUSE_ATTRIBUTE_ERR            = 10
INVALID_STATE_ERR              = 11
SYNTAX_ERR                     = 12
INVALID_MODIFICATION_ERR       = 13
NAMESPACE_ERR                  = 14
INVALID_ACCESS_ERR             = 15

class DOMException(Exception):
    def __init__(self, *args):
	if len(args) >= 1:
	    self.code = args[0]
        self.args = args
        Exception.__init__(self, g_errorMessages[self.code] % args)

#FIXME Just borrow the error messages from 4DOM for now
import xml.dom
g_errorMessages = xml.dom.FtExceptionStrings





FEATURES_MAP = {'XML':2.0,
        }

class DOMImplementation:
    def __init__(self):
        pass

    def hasFeature(self, feature, version=''):
        import string
        featureVersion = FEATURES_MAP.get(string.upper(feature))
        if featureVersion:
            if version and float(version) != featureVersion:
                return 0
            return 1
        return 0

    def createDocumentType(self, qualifiedName, publicId, systemId):
        return None

    def createDocument(self, namespaceURI, qualifiedName, doctype):
        doc = DocumentImp()
        if qualifiedName:
            el = doc.createElementNS(namespaceURI, qualifiedName)
            doc.appendChild(el)
        return doc

implementation = DOMImplementation()


class NodeImp:
    def __init__(self, ownerDocument=None, namespaceURI='', prefix='', localName=''):
        if ownerDocument:
            self.ownerDocument = ownerDocument
        if namespaceURI:
            self.namespaceURI = namespaceURI
        if prefix:
            self.prefix = prefix
        if localName:
            self.localName = localName

    def appendChild(self,newChild):
        if (newChild.nodeType == Node.DOCUMENT_FRAGMENT_NODE):
            # Only for non-empty document fragments
            for node in newChild.childNodes.value:
                self.appendChild(node)
        else:
            if newChild.parentNode:
                newChild.parentNode.removeChild(newChild)
            if self.lastChild:
                self.lastChild.nextSibling = newChild
                newChild.previousSibling = self.lastChild

            if self.childNodes.cardinality() == 0:
                self.firstChild = newChild
            self.lastChild = newChild
            self.add_childNodes(newChild)
        return newChild

    def removeChild(self, childNode):
        self.drop_childNodes(childNode)
        
        if childNode.nextSibling:
            childNode.nextSibling.previousSibling = childNode.previousSibling
        else:
            self.lastChild = childNode.previousSibling

        if childNode.previousSibling:
            childNode.previousSibling.nextSibling = childNode.nextSibling
        else:
            self.firstChild = childNode.nextSibling

        childNode.previousSibling = None
        childNode.nextSibling = None
        return childNode

    def insertBefore(self,newChild,refChild):
        if refChild is None:
            return self.appendChild(newChild)


        #FIXME can be optimized for DFs
        if newChild.nodeType == Node.DOCUMENT_FRAGMENT_NODE:
            nodes = newChild.childNodes[:]
        else:
            nodes = [newChild]
        for node in nodes:
            if node.parentNode:
                node.parentNode.removeChild(node)

            index = self.childNodes.index(refChild)
            tNodes = []
            for ctr in range(len(self.childNodes)):
                c = self.childNodes[0]
                tNodes.append(c)
                self.remove_childNodes(c)

            tNodes.insert(index,node)
            for c in tNodes:
                self.add_childNodes(c)

            if refChild.previousSibling:
                refChild.previousSibling.nextSibling = node
            else:
                self.firstChild = node
                
            node.previousSibling = refChild.previousSibling
            refChild.previousSibling = node
            node.nextSibling = refChild
        return newChild

    def replaceChild(self, newChild, oldChild):
        sibling = oldChild.nextSibling
        self.removeChild(oldChild)
        if sibling:
            self.insertBefore(newChild, sibling)
        else:
            self.appendChild(newChild)
        return oldChild


    def normalize(self):
        #FIXME this should be implemented
        pass

    def cloneNode(self,deep):

        newNode = self.copy(deep)

        if not deep and self.nodeType == Node.ELEMENT_NODE:
            #Still clone the attributes
            for attr in self.attributes.values():
                newNode.setAttributeNodeNS(attr.copy())
        return newNode


class NamedNodeMapImp(LiteralCollection.LiteralListOfObjects):

    def values(self):
        return self


from Ft.Ods.PersistentObject import TupleDefinitions
Element.Element_stub._tupleDefinitions['attributes'][TupleDefinitions.COLLECTION_CLASS] = NamedNodeMapImp
class ElementImp(NodeImp, Element.Element_stub):
    nodeType = Node.ELEMENT_NODE

    def __init__(self, data=None, ownerDocument=None, tagName='', namespaceURI='', prefix='', localName=''):
        Element.Element_stub.__init__(self, data)
        if data is None:
            NodeImp.__init__(self, ownerDocument, namespaceURI, prefix, localName)
            if self.localName:
                self.nodeName = self.prefix and (self.prefix + ':' + self.localName) or self.localName
                self.tagName = self.nodeName
            else:
                self.nodeName = self.tagName = tagName
        else:
            self.nodeName = self.tagName
        
    def setAttributeNS(self,namespaceURI,qName,value):
        attr = self.ownerDocument.createAttributeNS(namespaceURI,qName)
        attr.value = value
        self.setAttributeNodeNS(attr)

    def setAttributeNodeNS(self,node):
        if node.ownerElement:
            node.drop_ownerElement(node.ownerElement)
        self.add_attributes(node)

    def setAttribute(self,name,value):
        attr = self.ownerDocument.createAttribute(name)
        attr.value = value
        self.setAttributeNode(attr)

    def setAttributeNode(self,node):
        if node.ownerElement:
            node.drop_ownerElement(node.ownerElement)
        self.add_attributes(node)

    def getAttributeNodeNS(self,namespaceURI,localName):
        for attr in self.attributes:
            if attr.namespaceURI == namespaceURI and attr.localName == localName:
                return attr
        return None

    def getAttributeNS(self,namespaceURI,localName):
        attr = self.getAttributeNodeNS(namespaceURI,localName)
        return attr and attr.value or ""

    def getAttributeNode(self,name):
        for attr in self.attributes:
            if attr.name == name:
                return attr
        return None

    def getAttribute(self,name):
        attr = self.getAttributeNode(name)
        return attr and attr.value or ""

    def hasAttributeNS(self,namespaceURI,localName):
        return self.getAttributeNodeNS(namespaceURI,localName) != None

    def hasAttribute(self,name):
        return self.getAttributeNode(name) != None

    def __repr__(self):
        return "<DbDom Element Node at %x: name='%s' with %d attributes and %d children>" % (
            id(self),
            self.nodeName,
            len(self.attributes),
            len(self.childNodes)
            )
    
class AttributeImp(NodeImp,Attribute.Attribute_stub):
    nodeType = Node.ATTRIBUTE_NODE
    attributes = None

    def __init__(self, data=None, ownerDocument=None, name='', namespaceURI='', prefix='', localName=''):
        Attribute.Attribute_stub.__init__(self, data)
        if data is None:
            NodeImp.__init__(self, ownerDocument, namespaceURI, prefix, localName)
            if self.localName:
                self.nodeName = self.prefix and (self.prefix + ':' + self.localName) or self.localName
            else:
                self.nodeName = name
            self.name = self.nodeName
        else:
            self.nodeName = self.name


class TextImp(NodeImp,Text.Text_stub):
    nodeType = Node.TEXT_NODE
    attributes = None
    nodeName = '#text'
    
    def __init__(self, data=None, ownerDocument=None, cdata=''):
        Text.Text_stub.__init__(self, data)
        if data is None:
            NodeImp.__init__(self,ownerDocument,'','','')
            self.data = cdata
        
class CommentImp(NodeImp,Comment.Comment_stub):
    nodeType = Node.COMMENT_NODE
    attributes = None
    nodeName = '#comment'

    def __init__(self, data=None, ownerDocument=None, cdata=''):
        Comment.Comment_stub.__init__(self, data)
        if data is None:
            NodeImp.__init__(self,ownerDocument,'','','')
            self.data = cdata
        
class ProcessingInstructionImp(NodeImp,ProcessingInstruction.ProcessingInstruction_stub):
    nodeType = Node.PROCESSING_INSTRUCTION_NODE
    attributes = None
    def __init__(self, initData=None, ownerDocument=None, target='',data=''):
        ProcessingInstruction.ProcessingInstruction_stub.__init__(self, initData)
        if initData is None:
            NodeImp.__init__(self,ownerDocument,'','','')
            if data:
                self.data = data
            if target:
                self.target = target

        self.nodeName = self.target


class DocumentImp(NodeImp,Document.Document_stub):
    nodeType = Node.DOCUMENT_NODE
    attributes = None
    nodeName = '#document'
    doctype = None

    def __init__(self, data=None):
        Document.Document_stub.__init__(self, data)
        if data is None:
            NodeImp.__init__(self)
            if not self.ownerDocument:
                self.ownerDocument = self
            self.doctype = None

    def createElementNS(self, namespaceURI, qName):
        names = string.split(qName,':')
        if len(names) > 2:
            raise "Too manyt :s in %s" % qName
        elif len(names) == 2:
            prefix,localName = names[0],names[1]
        else:
            prefix,localName = '',names[0]
            
        e = ElementImp(None, self, qName, namespaceURI, prefix, localName)
        return e

    def createElement(self,name):
        e = ElementImp(None, self, name, '', '', '')
        return e

    def createAttribute(self, name):
        a = AttributeImp(None, self, name, '', '', '')
        return a

    def createAttributeNS(self, namespaceURI, qName):
        names = string.split(qName,':')
        if len(names) > 2:
            raise "Too manyt :s in %s" % qName
        elif len(names) == 2:
            prefix,localName = names[0],names[1]
        else:
            prefix,localName = '',names[0]
            
        e = AttributeImp(None, self, qName, namespaceURI, prefix, localName)
        return e

    def createTextNode(self, data):
        t = TextImp(None, self, data)
        return t

    def createProcessingInstruction(self, target, data):
        t = ProcessingInstructionImp(None, self, target,data)
        return t

    def createComment(self,data):
        c = CommentImp(None,self, data)
        return c

    def appendChild(self,node):
        if node.nodeType == Node.ELEMENT_NODE:
            if self.documentElement:
                raise DOMException(HIERARCHY_REQUEST_ERR)
            self.documentElement = node
        return NodeImp.appendChild(self,node)


    def createNodeIterator(self, root, whatToShow, filter, entityReferenceExpansion):
        from xml.dom import NodeIterator
        nodi = NodeIterator.NodeIterator(root, whatToShow, filter, entityReferenceExpansion)
        return nodi

    def createDocumentFragment(self):
        df = DocumentFragmentImp(None, self)
        return df


class DocumentFragmentImp(NodeImp, DocumentFragment.DocumentFragment_stub):
    nodeName = '#document-fragment'
    nodeType = Node.DOCUMENT_FRAGMENT_NODE
    attributes = None
    def __init__(self, data=None, ownerDocument=None):
        DocumentFragment.DocumentFragment_stub.__init__(self, data)
        if data is None:
            NodeImp.__init__(self, ownerDocument)
 
