import os, sys, string, shutil, glob
import distutils
from distutils.dist import Distribution
from distutils import version, util
from distutils.core import Command
from distutils.dep_util import newer
from distutils.command import install
from distutils.command import build, build_py
from distutils.command import install_lib
from distutils.command import sdist
from distutils.version import StrictVersion
from types import *
import tokenize
from install_data import Data_Files, install_Data_Files

if sys.hexversion >= 0x2000000:
    import pygettext, msgfmt


# Make sure that we are using the proper version of Distutils
# We make certain assumptions about the implementation

def EnsureVersion(version):
    """Checks Distutils version against specified version number"""
    dist_version = StrictVersion(distutils.__version__)
    expected = StrictVersion(version)
    if dist_version < expected:
        print
        print 'Requires Distutils v%s or newer.  Found version %s.' % (
            expected,
            dist_version,
            )
        print
        sys.exit(1)

EnsureVersion('1.0')

# Specification structure for generate_docs directive

class Document:
    def __init__(self, source, stylesheets, destFile,params=None):
        self.source = source
        self.stylesheets = stylesheets
        self.outfile = os.path.join(os.path.dirname(source), destFile)
        self.params = params or {}

# Specification structures for install_web directive

class WebFiles:
    """Installs relative to Apache ServerRoot"""
    def __init__(self, path, files):
        self.path = path
        self.files = files
        self._server = 1

    def serverRoot(self):
        return self._server

class CgiFiles(WebFiles):
    """Installs relative to Apache [ServerRoot]/cgi-bin"""
    def __init__(self, path,files):
        WebFiles.__init__(self, os.path.join('cgi-bin',path), files)

class WebDocuments(WebFiles):
    """Installs relative to DocumentRoot on Apache"""
    def __init__(self, path, files):
        WebFiles.__init__(self, path, files)
        self._server = 0


# Our new Distribution class

class Dist(Distribution):
    def __init__(self,attrs):
        self.py_files = None
        self.l10n = None
        self.gen_docs = None
        self.bgen_files = None
        self.executables = None
        self.idlFiles = None
        self.odlFiles = None
        self.generatedDirectory = None
        self.corbaNamePath = None
        self.webFiles = None
        Distribution.__init__(self,attrs)

    def has_docs(self):
        return self.gen_docs and len(self.gen_docs) > 0

    def has_idl(self):
        return self.idlFiles and len(self.idlFiles) > 0

    def has_odl(self):
        return self.odlFiles and len(self.odlFiles) > 0

    def has_namepath(self):
        return self.corbaNamePath

    def has_webfiles(self):
        return self.webFiles and len(self.webFiles) > 0

    def has_l10n(self):
        return self.l10n and len(self.l10n) > 0


class BuildPy(build_py.build_py):
    def initialize_options(self):
        build_py.build_py.initialize_options(self)
        self.py_files = None
        self.outfiles = []

    def finalize_options(self):
        build_py.build_py.finalize_options(self)
        self.py_files = self.distribution.py_files or []

    def run(self):
        for (package, filelist) in self.py_files:
            package = string.split(package, '.')
            package_dir = [self.build_lib] + list(package)
            package_dir = apply(os.path.join, package_dir)
            self.mkpath(package_dir)
            for file in filelist:
                (out, _) = self.copy_file(file, package_dir)
                self.outfiles.append(out)
        build_py.build_py.run(self)

    def get_outputs(self, include_bytecode=1):
        outputs = build_py.build_py.get_outputs(self, include_bytecode)
        outputs.extend(self.outfiles)
        return outputs

class BuildL10n(build_py.build_py):
    def initialize_options(self):
        build_py.build_py.initialize_options(self)
        self.l10n = None
        self.outfiles = []

    def finalize_options(self):
        build_py.build_py.finalize_options(self)
        self.l10n = self.distribution.l10n or []

    def run(self):
        for (package, po_dir, po_list, filelist) in self.l10n:
            package = string.split(package, '.')
            package_dir = [self.build_lib] + list(package)
            package_dir = apply(os.path.join, package_dir)
            for po_file in po_list:
                self.mkpath(package_dir)
                (out, _) = self.copy_file(po_file, package_dir)
                self.outfiles.append(out)
                locale = os.path.split(po_file)[1][:-3]
                mo_dir = os.path.join(package_dir, locale, 'LC_MESSAGES')
                mo_filename = os.path.join(mo_dir, '4Suite.mo')
                po_filename = os.path.join(package_dir, locale + '.po')
                if not self.dry_run:
                    self.mkpath(mo_dir)
                    msgfmt.MESSAGES = {}
                    msgfmt.make(po_filename)
                    tmp_mo_filename = po_filename[:-2] + 'mo'
                    shutil.copy(tmp_mo_filename, mo_filename)
                    os.remove(tmp_mo_filename)
                self.outfiles.append(mo_filename)
                

    def get_outputs(self, include_bytecode=1):
        outputs = build_py.build_py.get_outputs(self, include_bytecode)
        outputs.extend(self.outfiles)
        return outputs

class Stage(Command):

    description = "shortcut for generate/build/install"

    user_options = [
        ('force', 'f',
         'forcibly do everything (ignore file timestamps)'),
        ]

    boolean_options = ['force']

    def initialize_options(self):
        # Keep stage quiet
        self.distribution.verbose = 0
        self.force = 0

    def finalize_options(self):
        return
    
    def run(self):
        for cmd_name in ['build' ,'install', 'generate']:
            self.distribution.get_option_dict(cmd_name)['force'] = ('stage',
                                                                    self.force)
            self.run_command(cmd_name)

        cmd = self.get_finalized_command('generate')
        if cmd.get_outputs():
            # Now re-run install (quietly) to copy any changed docs
            self.announce('Copying generated files')
            self.distribution.verbose = 0
            del self.distribution.get_option_dict('install')['force']
            self.distribution.reinitialize_command('install')
            self.run_command('install')

class Generate(Command):

    description = "generate additional files needed to install"

    user_options = [
        ('force', 'f',
         'forcibly generate everything (ignore file timestamps)'),
        ]

    boolean_options = ['force']

    def initialize_options(self):
        self.force = 0

    def finalize_options(self):
        return

    def run(self):
        for cmd_name in self.get_sub_commands():
            self.run_command(cmd_name)

    def get_outputs(self):
        outputs = []
        for cmd_name in self.get_sub_commands():
            cmd = self.get_finalized_command(cmd_name)
            outputs.extend(cmd.get_outputs())
        return outputs

    def has_docs(self):
        return self.distribution.has_docs()

    def has_bisongen(self):
        bgen_files = self.distribution.bgen_files
        return bgen_files and len(bgen_files) > 0

    def has_executables(self):
        executables = self.distribution.executables
        return executables and len(executables) > 0

    def has_odlFiles(self):
        odlFiles = self.distribution.odlFiles
        return odlFiles and len(odlFiles) > 0

    def has_l10n(self):
        l10n = self.distribution.l10n
        return l10n and len(l10n) > 0

    sub_commands = [('generate_docs', has_docs),
                    ('generate_odl', has_odlFiles),
                    ('generate_idl', has_odlFiles),
                    ('generate_bgen', has_bisongen),
                    ('generate_exec', has_executables),
                    ]

    if sys.hexversion >= 0x2000000:
        sub_commands.append(('generate_l10n', has_l10n),)

class GenerateDocs(Command):

    description = "generate documentation files"

    user_options = [
        ('force', 'f',
         'force document generatation (overwrite existing files)'),
        ]

    boolean_options = ['force']

    def initialize_options(self):
        self.outfiles = []
        self.gen_docs = None
        self.force = 0


    def finalize_options(self):
        self.set_undefined_options('generate', ('force', 'force'))
        self.gen_docs = self.distribution.gen_docs

    def get_outputs(self):
        return self.outfiles

    def run(self):
        if not self.gen_docs:
            return

        from xml.xslt.Processor import Processor

        for doc in self.gen_docs:
            if not self.force:
                update = 0
                for file in doc.stylesheets:
                    if os.path.exists(file):
                        update = newer(file, doc.outfile)
                        if update:
                            break
                if not (update or newer(doc.source, doc.outfile)):
                    continue
            
            self.announce("Generating '%s' from %s" % (doc.outfile, doc.source))
            self.outfiles.append(doc.outfile)

            processor = Processor()
            params = {'version' : self.distribution.get_version()}
            params.update(doc.params)
            try:
                ignorePis = 1
                for file in doc.stylesheets:
                    if os.path.exists(file):
                        processor.appendStylesheetFile(file)
                    else:
                        ignorePis = 0
                result = processor.runUri('file:' + doc.source, ignorePis, params)
                stream = open(doc.outfile, 'w')
                stream.write(result + '\n')
                stream.close()
            except Exception, e:
                import traceback, cStringIO
                stream = cStringIO.StringIO()
                traceback.print_exc(None, stream)
                stream.write(str(e) + '\n')
                self.announce('Unable to generate %s' % doc.outfile)
                self.announce(stream.getvalue())
                stream.close()


class GenerateL10n(Command):

    description = "generate localizations"

    user_options = [
        ('force', 'f',
         'force locale generatation (overwrite existing files)'),
        ]

    boolean_options = ['force']

    def initialize_options(self):
        self.outfiles = []
        self.l10n = None
        self.force = 0

    def finalize_options(self):
        self.set_undefined_options('generate', ('force', 'force'))
        self.l10n = self.distribution.l10n

    def get_outputs(self):
        return self.outfiles

    def run(self):
        if not self.l10n:
            return

        class Options:
            # constants
            GNU = 1
            SOLARIS = 2
            # defaults
            extractall = 0 # FIXME: currently this option has no effect at all.
            escape = 0
            keywords = []
            outpath = ''
            outfile = 'en_US.po'
            writelocations = 1
            locationstyle = GNU
            verbose = 0
            width = 78
            excludefilename = ''

        for (package, po_dir, po_list, filelist) in self.l10n:
            for source_file in filelist:
                fp = open(source_file)
                pygettext_options = Options()
                pygettext_options.keywords = ['_']
                pygettext_options.toexclude = []
                pygettext.make_escapes(pygettext_options.escape)
                pygettext_eater = pygettext.TokenEater(pygettext_options)
                pygettext_eater.set_filename(source_file)
                tokenize.tokenize(fp.readline, pygettext_eater)
                fp.close()
            po_filename = os.path.join(po_dir, 'en_US' + ".po")
            fp = open(po_filename, 'w')
            pygettext_eater.write(fp)
            self.announce("Generating " + po_filename)
            fp.close()


class GenerateBisonGen(Command):

    description = "generate documentation files"

    user_options = [
        ('force', 'f',
         'force document generatation (overwrite existing files)'),
        ]

    boolean_options = ['force']

    def initialize_options(self):
        self.outfiles = []
        self.gen_docs = None
        self.force = 0


    def finalize_options(self):
        self.set_undefined_options('generate', ('force', 'force'))
        self.bgen_files = self.distribution.bgen_files

    def get_outputs(self):
        outputs = []
        #for fileName in self.bgen_files:
        #    path,fName = os.path.split(fileName)
        #    baseName = string.split(fName,'.')[0]
        #    outputs.append(os.path.join(path,baseName+'.y'))
        #    outputs.append(flexInFileName = os.path.join(path,baseName+'.l'))
        #    outputs.append(swigInFileName = os.path.join(path,baseName+'.i'))
        return outputs


    def run(self):
        if not self.bgen_files:
            return

        for fileName in self.bgen_files:
            path,fName = os.path.split(fileName)
            baseName = string.split(fName,'.')[0]

            testFileName = os.path.join(path,baseName+'.h')
            if self.force or newer(fileName,testFileName):
                #Run Bisongen
                os.system("cd %s && BisonGen %s" % (path,fName))
                #Run Bison
                os.system("cd %s && bison --name-prefix=%s -d %s.y"%(path,baseName,baseName))
                #Run Flex
                os.system("cd %s && flex -f -P%s %s.l"%(path,baseName,baseName))
                #Run Swig
                os.system("cd %s && swig -python -shadow %s.i"%(path,baseName))

                toDelete = ['%s.i'%baseName,
                            '%s.l'%baseName,
                            '%s.y'%baseName,
                            '%s_wrap.doc'%baseName,
                            ]
                for f in toDelete:
		    filename = os.path.join(path,f)
		    if os.path.exists(filename):
                        os.unlink(filename)


class GenerateIdl(Command):

    description = "Generate omniORB IDL files in the generated directory"

    user_options = [
        ('force', 'f', "forcibly build everything (ignore file timestamps)"),
        ]

    def initialize_options (self):
        self.force = None
        self.idlFiles = None

    def finalize_options (self):
        self.set_undefined_options('build',
                                   ('force', 'force'))

        
        self.idlFiles = self.distribution.idlFiles

        return

    def get_outputs (self):
        #FIXME
        raise Exception('Exception trying to use get_outputs')

    def run(self):
        #Should be safe to import xslt here
        if not self.has_idl():
            return

        destDir = 'generated'

        #Make sure the dir is there
        self.mkpath(destDir)
        #Make sure the init file is there
        initFile = os.path.join(destDir,'__init__.py')
        if not os.path.exists(initFile):
            f = open(initFile,"w")
            f.write("\n")
            f.close()
                
        for idlFile in self.idlFiles:

            idlFilePath,idlFileName = os.path.split(idlFile)
            
            destFileName = '%s_idl.py' % os.path.splitext(idlFileName)[0]
            destFileName = os.path.join(destDir,destFileName)
            destIdlFile = os.path.join(destDir,idlFileName)
            if not newer(idlFile, destIdlFile):
                continue
            self.announce ("Generating '%s' from %s" % (destFileName,idlFile))

            idl_cmd = ['omniidl','-bpython','-C%s'%destDir,'-I%s'%destDir,idlFile]

            self.spawn(idl_cmd)            

            #Copy the IDL (for includes)
            self.copy_file(idlFile,destIdlFile)

        return

    def has_idl(self):
        return self.distribution.has_idl()


class GenerateOdl(Command):

    description = "Build 4ODS ODL files into the generated"

    user_options = [
        ('force', 'f', "forcibly build everything (ignore file timestamps)"),
        ]

    def initialize_options (self):
        self.force = None
        self.odlFiles = None

    def finalize_options (self):
        self.set_undefined_options('build',
                                   ('force', 'force'))

        
        self.odlFiles = self.distribution.odlFiles

        return

    def get_outputs (self):
        #FIXME
        raise Exception('Exception trying to use get_outputs')

    def run(self):

        if not self.has_odl():
            return

        destDir = 'generated'


        #Make sure the dir is there
        self.mkpath(destDir)
        #Make sure the init file is there
        initFile = os.path.join(destDir,'__init__.py')
        if not os.path.exists(initFile):
            f = open(initFile,"w")
            f.write("\n")
            f.close()


        for odlFile in self.odlFiles:

            odlFilePath,odlFileName = os.path.split(odlFile)
            destOdlFile = os.path.join(destDir,odlFileName)

            new = newer(odlFile,destOdlFile)

            if not new and not self.force:
                continue
            self.copy_file(odlFile,destOdlFile)

            odl_cmd = ["cd %s"%destDir,"&&","OdlParse"]
            self.announce ("Generating ODL Files from %s" % (odlFile))
            odl_cmd.append("-p")
            odl_cmd.append("foo")
            odl_cmd.append(odlFileName)
            os.system(string.join(odl_cmd,' '))            

        return




    def has_odl(self):
        return self.distribution.has_odl()


class InstallIdl(Command):

    description = "Install IDL generated files"

    user_options = [
        ]

    def initialize_options (self):
        self.idlFiles = None

    def finalize_options (self):
        self.idlFiles = self.distribution.idlFiles


    def get_outputs (self):
        return self.outfiles

    def run(self):

        destDir = 'generated'

        for idlFile in self.idlFiles:

            idlFilePath,idlFileName = os.path.split(idlFile)
            
            destFileName = '%s_idl.py' % os.path.splitext(idlFileName)[0]
            destFileName = os.path.join(destDir,destFileName)
            destIdlFile = os.path.join(destDir,idlFileName)
            if newer(idlFile, destIdlFile):
                #Print a big old message
                sys.stderr.write("%s is newer then %s, you need to regenerate the files\n"% (idlFile,destIdlFile))
                raise "Please run 'setup.py generate'"


        if not self.has_idl():
            return


    def has_idl(self):
        return self.distribution.has_idl()



class InstallOdl(Command):
    description = "Just check to make sure the ODL files are up to date"

    user_options = [
        ]

    def initialize_options (self):
        pass

    def finalize_options (self):
        self.odlFiles = self.distribution.odlFiles
        pass

    def get_outputs (self):
        return self.outfiles

    def run(self):
        if not self.has_odl():
            return


        destDir = 'generated'


        for odlFile in self.odlFiles:

            odlFilePath,odlFileName = os.path.split(odlFile)
            destOdlFile = os.path.join(destDir,odlFileName)

            new = newer(odlFile,destOdlFile)

            if new:
                #Print a big old message
                sys.stderr.write("%s is newer then %s, you need to regenerate the files\n"% (odlFile,destOdlFile))
                raise "Please run 'setup.py generate'"


    def has_odl(self):
        return self.distribution.has_odl()


class InstallWebFiles(Command):
    description = "install data files for Apache web server"

    user_options = [
        ('server-root=', 's', "Apache server root directory"),
        ('document-root=', 'd', "Apache document root directory"),
        ('force', 'f', "force installation (overwrite existing files)"),
        ]

    def initialize_options (self):
        self.outfiles = []
        self.server_root = None
        self.document_root = None
        self.build_dir = None
        self.force = 0
        
    def finalize_options (self):
      self.set_undefined_options('install',
                                 ('force', 'force'),
                                 )
      if not self.server_root:
          self.server_root = '/usr/local/apache'
      if not self.document_root:
          self.document_root = '/usr/local/apache/htdocs'

    def get_outputs (self):
        return self.outfiles
        
    def run(self):
        webFiles = self.distribution.webFiles or []
        for file_info in webFiles:
            if file_info.serverRoot():
                root = self.server_root
            else:
                root = self.document_root
            dest = os.path.join(root, file_info.path)
            self.mkpath(dest)
            for file in file_info.files:
                (out, _) = self.copy_file(file, dest)
                self.outfiles.append(out)


class InstallNamePath(Command):
    description = "Install an omniORB Nameservice path"

    user_options = [
                    ]

    def initialize_options (self):
        self.corbaNamePath = None
        
    def finalize_options (self):
        self.corbaNamePath = self.distribution.corbaNamePath

    def get_outputs (self):
        return []
        
    def run(self):
        if not self.corbaNamePath:
            return
        names = string.split(self.corbaNamePath,'/')
        fullName = ''
        for n in names:
            if fullName:
                fullName = fullName + '/' + n
            else:
                fullName = n
            if not os.popen('nameclt resolve %s' % fullName).read():
                self.announce('nameclt bind_new_context %s' % fullName)
                os.system('nameclt bind_new_context %s' % fullName)
            else:
                self.announce('not nameclt bind_new_context %s (already there)' % fullName)

        return


class InstallXmlPlusInit(install_lib.install_lib):
    '''
    In Python 2.0, PyXML is not strictly required.  In this case, we need to
    add our own __init__.py to _xmlplus
    '''
    def run(self):
        xmlplus_dir = self.install_dir + '/_xmlplus/__init__.py'
        if sys.version[0] == '2' and not os.access(xmlplus_dir, os.F_OK):
            f = open(xmlplus_dir, 'a')
            f.write('version_info = (0,6,2)\n__version__ = "0.6.2"')
            f.close()


class SDist(sdist.sdist):

    def add_defaults(self):
        sdist.sdist.add_defaults(self)
        if self.distribution.has_data_files():
            data_files = self.distribution.data_files
            source_files = []
            for f in data_files:
                if type(f) == StringType:
                    source_files.append(f)
                elif type(f) == TupleType:
                    source_files.extend(f[1])
                else:
                    # it's a tuple with path to install to and a list of files
                    for data in f.files:
                        source_files.append(data)

            self.filelist.extend (source_files)


class Validate(Command):
    
    description = "ensure all files would be installed"

    user_options = [
                    ]

    def initialize_options (self):
        self.ignored = ['CVS', '.cvsignore', 'dist', 'build', 'MANIFEST']
        return
    
    def finalize_options (self):
        return

    def run(self):
        self.distribution.dry_run = 1
        self.distribution.verbose = 0
        self.distribution.get_option_dict('sdist')['manifest_only'] = ('validate', 1)
        self.run_command('sdist')
        sdist = self.get_finalized_command('sdist')
        # prepend './' to files
        dist_files = map(lambda file:
                         (file[:2] != './' and './' + file or file),
                         sdist.filelist.files)

        # Get the list of files missing from the dist
        src_files = self.find_files('.')
        for file in src_files:
            if file not in dist_files and file[-3:] != 'pyc':
                self.warn('Not in distribution, %s' % file)
        return

    def find_files(self, file):
        rt = []
        if os.path.isdir(file) and not os.path.islink(file):
            names = os.listdir(file)
            for name in names:
                fullname = os.path.join(file, name)
                if not (name in self.ignored
                        or os.path.islink(fullname)):
                    rt.extend(self.find_files(fullname))
            return rt
        rt.append(file)
        return rt


class Uninstall(Command):

    description = "uninstall the package"

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass
        
    def run(self):

        # Execute build
        self.announce('determining installation files')
        orig_dry_run = self.distribution.dry_run
        orig_verbose = self.distribution.verbose
        self.distribution.dry_run = 0
        self.distribution.verbose = 0
        self.run_command('build')

        # Execute install in dry-run mode
        self.distribution.dry_run = 1
        self.run_command('install')
        self.distribution.dry_run = orig_dry_run
        self.distribution.verbose = orig_verbose
        build = self.get_finalized_command('build')
        install = self.get_finalized_command('install')

        # Directories that should not be removed
        # install_headers is skipped because is uses $dist_name
        root_dirs = [install.install_purelib,
                     install.install_platlib,
                     install.install_scripts,
                     install.install_data,
                     ]

        # Remove all installed files
        self.announce("removing files")
        dirs = {}
        filenames = install.get_outputs()
        for filename in filenames:
            if not os.path.isabs(filename):
                raise DistutilsError,\
                      'filename "%s" from .get_output() not absolute' % \
                      filename

            if os.path.isfile(filename):
                self.announce("removing '%s'" % filename)
                if not self.dry_run:
                    try:
                        os.remove(filename)
                    except OSError, details:
                        self.warn("Could not remove file: %s" % details)
                    dir = os.path.split(filename)[0]
                    if not dirs.has_key(dir):
                        dirs[dir] = 1
                    if os.path.splitext(filename)[1] == '.py':
                        # Try and remove the .pyc if not already in the list
                        if filename+'c' not in filenames:
                            try:
                                os.remove(filename + 'c')
                            except OSError:
                                pass

                        # Try and remove the .pyo if not already in the list
                        if filename+'o' not in filenames:
                            try:
                                os.remove(filename + 'o')
                            except OSError:
                                pass

            elif os.path.isdir(filename):
                if not dirs.has_key(dir):
                    dirs[filename] = 1

            else:
                self.announce("skipping removal of '%s' (not found)" %
                              filename)

        # Remove the installation directories
        self.announce("removing directories")
        dirs = dirs.keys()
        dirs.sort(); dirs.reverse() # sort descending
        for dir in dirs:
            if dir in root_dirs:
                # A base directory that shouldn't be removed
                continue
            self.announce("removing directory '%s'" % dir)
            if not self.dry_run:
                if os.listdir(dir):
                    self.warn("skipping removal of '%s' (not empty)" % dir)
                else:
                    try:
                        os.rmdir(dir)
                    except OSError, details:
                        self.warn("could not remove directory: %s" % details)



# Setup functions

def EnsureScripts(*linuxScripts):
    """Creates the proper script names required for each platform"""
    scripts = linuxScripts
    if util.get_platform()[:3] == 'win':
        scripts = map(lambda s: s + '.bat', scripts)
    return scripts

            
def InstallSubCommands():
    """Adds our own sub-commands to build and install"""
    has_idl = lambda self: self.distribution.has_idl()
    has_odl = lambda self: self.distribution.has_odl()
    has_namepath = lambda self: self.distribution.has_namepath()
    has_webfiles = lambda self: self.distribution.has_webfiles()
    is_py2 = lambda self: sys.version[0] == '2'
    has_l10n = lambda self: sys.hexversion >= 0x2000000 and self.distribution.has_l10n()

    installCmds = [('install_idl', has_idl),
                   ('install_odl', has_odl),
                   ('install_namepath', has_namepath),
                   ('install_web', has_webfiles),
                   ('install_xmlplus_init', is_py2),
                   ]

    if sys.hexversion >= 0x2000000:
        buildCmds = [('build_l10n', has_l10n),
                     ]
    else:
        buildCmds = []
    
    install.install.sub_commands.extend(installCmds)
    build.build.sub_commands.extend(buildCmds)

# Command function overrides
Commands = {
    'build_py' : BuildPy,
    'build_l10n' : BuildL10n,

    'stage' : Stage,

    'validate' : Validate,

    'uninstall' : Uninstall,

    'generate' : Generate,
    'generate_docs' : GenerateDocs,
    'generate_odl' : GenerateOdl,
    'generate_bgen' : GenerateBisonGen,
    'generate_idl' : GenerateIdl,
    'generate_l10n' : GenerateL10n,

    'install_data' : install_Data_Files,
    'install_web' : InstallWebFiles,
    'install_namepath' : InstallNamePath,
    'install_idl' : InstallIdl,
    'install_odl' : InstallOdl,
    'install_xmlplus_init': InstallXmlPlusInit,

    'sdist' : SDist,
    }
