############################################################################
#
# File Name: 		CollectionBase.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/CollectionBase.py.html
#
"""
Base collection class.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.Collections import Iterator
from Ft.Ods.Collections import BidirectionalIterator
from Ft.Ods import Collection

class CollectionBase(Collection.Collection):
    #Exception classes
    class InvalidCollectionType:
        pass

    class ElementNotFound:
        def __init__(self, element):
            self.element = element

    def __init__(self,cid=None,subType=None,values = None):

        Collection.Collection.__init__(self,cid)

        self.__dict__['_subType'] = subType
        self.__dict__['_values'] = values or []

    #The pythonic interface (defined first 
    def __cmp__(self, other):
        if isinstance(other, CollectionBase):
            if self._4ods_getId() and other._4ods_getId():
                return cmp(self._4ods_getId(), other._4ods_getId())
            return cmp(self[:], other[:])
        else:
            return cmp(self[:], other)

    def __len__(self): return len(self._values)

    def __getitem__(self, i):
        raise NotImplementedError('Subclass must override')

    def __setitem__(self, i, item):
        raise NotImplementedError('Subclass must override')

    def __delitem__(self, i):
        raise NotImplementedError('Subclass must override')

    def __getslice__(self, i, j):
        i = max(i, 0); j = min(j, self.cardinality())
        res = []
        for ctr in range(i,j):
            res.append(self[ctr])
        return res

    def __setslice__(self, i, j, other):
        if isinstance(other, CollectionBase):
            other = other[:]
        elif not type(other) == type([]):
            raise "Not a list"
        i = max(i, 0); j = min(j, self.cardinality())

        #Delete from i to j
        #insert at i
        for ctr in range(i,j):
            del self[i]
        for ctr in range(len(other)):
            self.insert(i+ctr,other[ctr])


    def __delslice__(self, i, j):
        i = max(i, 0); j = min(j, self.cardinality())
        for ctr in range(i,j):
            del self[i]


    def __add__(self, other):
        if isinstance(other, CollectionBase):
            other = other[:]
        elif not type(other) == type([]):
            raise "Not a list"
        return self[:] + other

    __radd__ = __add__

    def __mul__(self, n):
        return self[:] * n
    __rmul__ = __mul__

    def append(self, item):
        raise NotImplementedError('Subclass must override')

    def insert(self, i, item):
        raise NotImplementedError('Subclass must override')

    def pop(self, i=-1):
        rt = self[i]
        del self[i]
        return rt

    def remove(self, item):
        raise NotImplementedError('Subclass must override')

    def count(self, item):
        raise NotImplementedError('Subclass must override')

    def index(self, item):
        raise NotImplementedError('Subclass must override')

    def reverse(self):
        data = self[:]
        for ctr in range(len(self)):
            del self[0]
        data.reverse()
        for item in data:
            self.append(item)

    def sort(self, *args): 
        data = self[:]
        for ctr in range(len(self)):
            del self[0]
        apply(data.sort,args)
        for item in data:
            self.append(item)

    def extend(self, other):
        for o in other:
            self.append(o)

    #The collection interface
    cardinality = __len__

    def is_empty(self):
        return len(self) == 0

    def is_ordered(self):
        return 0

    def allows_duplicates(self):
        return 0

    def contains_element(self, element):
        return self.count(element) > 0

    def insert_element(self, element):
        self.append(element)

    def remove_element(self, element):
        if self.contains_element(element):
            self.remove(element)
        else:
            raise ElementNotFound(element)

    def copy(self,deep=1):
        other = apply(self.__class__)
        if deep:
            for v in self:
               other.insert_element(v)
        return other

    def create_iterator(self, stable):
        return Iterator.Iterator(self, stable)

    def create_bidirectional_iterator(self, stable):
        return BidirectionalIterator.BidirectionalIterator(self, stable)

    def select_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def select(self,OQL_predicate):
        raise FtodsNotSupportedException("select")

    def query(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def exists_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def _pseudo_del(self):
        self.__dict__['_values'] = []
    

