############################################################################
#
# File Name: 		Iterator.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Iterator.py.html
#
"""
Iterator class.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

#Exception classes
class InvalidCollectionType:
    pass

class NoMoreElements:
    pass

class Iterator:
    def __init__(self, collection, stable=0):

        from Ft.Ods.Collections import CollectionBase
        from Ft.Ods.Collections import DictionaryOfPersistentObjects
        from Ft.Ods.Collections import LiteralDictionaryOfValues

        if not isinstance(collection,CollectionBase.CollectionBase) and not isinstance(collection,DictionaryOfPersistentObjects.DictionaryOfPersistentObjects) and not isinstance(collection,LiteralDictionaryOfValues.LiteralDictionaryOfValues):
            raise InvalidCollectionType()

        if stable:
            self._collection = collection.copy(deep=1)
        else:
            self._collection = collection
        self._index = 0
        self._stable = stable

    def is_stable(self):
        return self._stable

    def at_end(self):
        return len(self._collection) == 0 or (self._collection.cardinality() -1 == self._index)

    def reset(self):
        self._index = 0

    def get_element(self):
        if self._index < self._collection.cardinality():
            return self._collection._4ods_retrieveElementByIndex(self._index)
        raise NoMoreElements()

    def next_position(self):
        if not self.at_end():
            self._index = self._index + 1
        else:
            raise NoMoreElements()

    def replace_element(self, e):
        #How does this work?
        raise "Not Supported"

