############################################################################
#
# File Name: 		ListBase.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/List.py.html
#
"""
Lists: ordered collections.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.Collections import CollectionBase

class ListBase:

    _collectionType = Constants.CollectionTypes.LIST

    #Exception classes
    class InvalidIndex:
        def __init__(self,index):
            self.index = index

    def is_ordered(self):
        return 1

    def allows_duplicates(self):
        return 1

    def remove_element_at(self, index):
        if index < 0:
            raise ListBase.InvalidIndex(index)
        if index >= self.cardinality():
            raise ListBase.InvalidIndex(index)
        try:
            del self[index]
        except IndexError:
            raise ListBase.InvalidIndex(index)

    def retrieve_element_at(self, index):
        if index < 0:
            raise ListBase.InvalidIndex(index)
        if index >= self.cardinality():
            raise ListBase.InvalidIndex(index)
        try:
            return self[index]
        except IndexError:
            raise ListBase.InvalidIndex(index)

    def replace_element_at(self, element, index):
        if index < 0:
            raise ListBase.InvalidIndex(index)
        if index >= self.cardinality():
            raise ListBase.InvalidIndex(index)
        try:
            self[index] = element
        except IndexError:
            raise ListBase.InvalidIndex(index)

    def insert_element_after(self, element, index):
        if index < 0:
            raise ListBase.InvalidIndex(index)
        if index >= self.cardinality():
            raise ListBase.InvalidIndex(index)
            
        try:
            self.insert(index+1,element)
        except IndexError:
            raise ListBase.InvalidIndex(index)

    def insert_element_before(self, element, index):
        if index < 0:
            raise ListBase.InvalidIndex(index)
        if index >= self.cardinality():
            raise ListBase.InvalidIndex(index)
        try:
            self.insert(index,element)
        except IndexError:
            raise ListBase.InvalidIndex(index)

    def insert_element_first(self, obj):
        self.insert(0,obj)
        return
    
    def insert_element_last(self, obj):
        self.append(obj)

    def remove_first_element(self):
        try:
            del self[0]
        except:
            raise CollectionBase.CollectionBase.ElementNotFound(None)

    def remove_last_element(self):
        try:
            del self[-1]
        except IndexError:
            raise CollectionBase.CollectionBase.ElementNotFound(None)

    def retrieve_first_element(self):
        try:
            return self[0]
        except IndexError:
            raise CollectionBase.CollectionBase.ElementNotFound(None)

    def retrieve_last_element(self):
        try:
            return self[-1]
        except IndexError:
            raise CollectionBase.CollectionBase.ElementNotFound(None)

    def concat(self, otherList):
        result = self.copy(deep=0)
        result.append(self)
        result.append(otherList)
        return result

    def append(self, otherList):
        if not isinstance(otherList,ListBase):
            otherList = [otherList]

        for o in otherList:
            self._operationalBase.append(self,o)

    insert_element = insert_element_last
