########################################################################
#
# File Name: 	       Enumeration.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Enumeration.py.html
#
"""
Implements the Enumeration meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.MetaData import ScopedType
from Ft.Ods.Collections import LiteralCollection
from Ft.Ods.MetaData import MetaKind
from Ft.Ods.MetaData import Constant
from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods import PersistentObject
td = PersistentObject.TupleDefinitions

class Enumeration(ScopedType.ScopedType):
    def __init__(self,data=None):
        ScopedType.ScopedType.__init__(self,data,MetaKind.MetaKind.mk_enumeration)


    #These three interfaces are for any scopedtype
    #We must support them
    def _4ods_getBoundChildren(self):
        return self.elements

    def _4ods_boundChild(self,obj):
        self.add_elements(obj)

    def _4ods_unboundChild(self,obj):
        self.remove_elements(obj)

    def add_elements(self,target,inverse=1):
        self._4ods_addRelationship('elements',Constant.Constant,'enumeration','form',target,inverse)
    form_elements = add_elements

    def remove_elements(self,target,inverse = 1):
        self._4ods_removeRelationship('elements','enumeration','drop',target,inverse)
    drop_elements = remove_elements

    def _4ods_getOdl(self):
        st = 'enum %s { ' % self.name
        for e in self.elements:
            st = st + ' %s,' % e.name
        st = st[:-1] + '};'
        return st

    def _4ods_checkRemove(self):
        return 1

    def _4ods_getOdmgType(self):
        return Constants.Types.ENUMERATION

    _tupleNames =  ('elements',)

    _tupleDefinitions = {'elements':{td.TYPE:Constants.Types.OBJECT_LIST,
                                     td.READONLY:1,
                                     td.RELATIONSHIP:1,
                                     td.COLLECTION_CLASS:LiteralCollection.LiteralListOfRepositoryObjects,
                                     }
                         }

    _localExtents = ()
