#!/usr/bin/python
########################################################################
#
# File Name:            GenerateType
#
# Documentation:        http://docs.4suite.com/ODS/GenerateType.py.html
#

#Desc.
#WWW: http://4suite.com/4ODS        e-mail: support@4suite.com

#Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
#See  http://4suite.com/COPYRIGHT  for license and copyright information

#Contains the definitions for _all_ types in ODS, plus a base class
import string

from Ft.Ods.MetaData.MetaKind import MetaKind
from Ft.Ods.MetaData.PrimitiveKind import PrimitiveKind
from Ft.Ods.MetaData.CollectionKind import CollectionKind

def WriteImports(typ,params):
    SplitByType(typ,params,('Write','Imports'))

def WriteCommit(typ,params):
    SplitByType(typ,params,('Write','Commit'))

def WriteInit(typ,params):
    SplitByType(typ,params,('Write','Init'))

def WriteRead(typ,params):
    SplitByType(typ,params,('Write','Read'))

def WriteWrite(typ,params):
    SplitByType(typ,params,('Write','Write'))

def WriteDelete(typ,params):
    SplitByType(typ,params,('Write','Delete'))

def WriteFtodsDelete(typ,params):
    SplitByType(typ,params,('Write','FtodsDelete'))

def SplitByType(typ,params,cmd):
    from Ft.Ods.Parsers.Odl import GenerateType
    if typ.meta_kind == MetaKind.mk_primitive_type:
        t = 'PrimitiveKind'
    elif typ.meta_kind == MetaKind.mk_collection:
        t = 'Collection'
    elif typ.meta_kind == MetaKind.mk_class:
        t = 'Class'
    elif typ.meta_kind == MetaKind.mk_enumeration:
        t = 'Enumeration'
    else:
        raise "Unkown Type %s" % typ
    cmd = cmd[0] + t + cmd[1]
    func = getattr(GenerateType,cmd)
    func(typ,params)




#Functions for each type
#####################
#The Primitives
####################

g_pyDefaultFromOdlType = {
    'float': "0.0",
    'double': "0.0",
    'unsigned short': "0",
    'unsigned long': "0L",
    'short': "0",
    'long': "0L",
    'char': "''",
    'boolean': "0",
    'octet': "0",
    'date': "'epoch'",
    'time': "'00:00:00'",
    'interval': "'@ 0 mins'",
    'timestamp': "'epoch'",
    'string': "''",
    'long long': "0L",
    'file':'""',
    }


def WritePrimitiveKindImports(typ,params):
    pass
def WritePrimitiveKindInit(typ,params):
    default_val = g_pyDefaultFromOdlType[typ._4ods_getOdl()]
    params.outFile.write('%s'%(default_val))        




#####################
#Structures
#####################

def WriteStructureImports(typ,params):
    path = string.split(typ.absolute_name(),'::')
    if params.basePackage:
        p = params.basePackage
        for curPath in path:
            if curPath == path[-1]:
                continue
            else:
                p = p + '.' + curPath
            
        outFile.write('from %s import %s\n' % (p,path[-1]))
    else:
        params.outFile.write('import %s\n' % string.join(path,'.'))



#####################
#Classes
#####################

def WriteClassImports(typ,params):
    params.outFile.write(params.indent)
    path = string.split(typ.absolute_name(),'::')
    if path[0] == 'ODLMetaObjects':
        path = path[1:]
    if params.basePackage:
        p = params.basePackage
        for curPath in path:
            if curPath == path[-1]:
                continue
            else:
                p = p + '.' + curPath
            
        params.outFile.write('from %s import %s\n' % (p,path[-1]))
    else:
        params.outFile.write('import %s\n' % (string.join(path,'.')))

def WriteClassInit(typ,params):
    params.outFile.write("None")



####################
#Collections
####################

from Ft.Ods.MetaData import Collection
def WriteCollectionImports(typ,params):
    params.outFile.write(params.indent)
    fName = CollectionToName(typ)
    params.outFile.write("from Ft.Ods.Collections.LiteralCollection import %s\n" % fName)
    WriteImports(typ.subtype,params)

def WriteCollectionInit(typ,params):
    fName = CollectionToName(typ)
    if fName[0] == 'L':
        #Create a new object
        if typ.subtype.meta_kind == MetaKind.mk_class:
            absName = typ.subtype.absolute_name()
            absPath = string.split(absName,'::')
            absName = string.join(absPath,'.')
            init = "_class=%s.%s_stub" % (absName,absPath[-1])
        #FIXME: need to handle Python 2.0 unicode
        elif typ.subtype.meta_kind == MetaKind.mk_primitive_type:
            if typ.subtype.primitive_kind == PrimitiveKind.pk_boolean:
                init = "type(0)"
            elif typ.subtype.primitive_kind == PrimitiveKind.pk_short:
                init = "type(0)"
            elif typ.subtype.primitive_kind == PrimitiveKind.pk_string:
                init = "type('')"
            elif typ.subtype.primitive_kind == PrimitiveKind.pk_ushort:
                init = "type(0)"
            elif typ.subtype.primitive_kind == PrimitiveKind.pk_long:
                init = "type(0L)"
            elif typ.subtype.primitive_kind == PrimitiveKind.pk_ulong:
                init = "type(0L)"
            elif typ.subtype.primitive_kind == PrimitiveKind.pk_long_long:
                init = "type(0L)"
            elif typ.subtype.primitive_kind == PrimitiveKind.pk_float:
                init = "type(0.0)"
            elif typ.subtype.primitive_kind == PrimitiveKind.pk_double:
                init = "type(0.0)"
            else:
                raise "Unsupported subtype for collection %s" % typ.subtype.primitive_kind
        else:
            raise "Unsupported subtype for collection %s" % typ.subtype.meta_kind
        if typ.collection_kind == CollectionKind.ck_dictionary:
            if typ.key_type.meta_kind == MetaKind.mk_primitive_type:
                if typ.key_type.primitive_kind == PrimitiveKind.pk_boolean:
                    init = "type(0)," + init
                elif typ.key_type.primitive_kind == PrimitiveKind.pk_short:
                    init = "type(0)," + init
                elif typ.key_type.primitive_kind == PrimitiveKind.pk_string:
                    init = "type('')," + init
                elif typ.key_type.primitive_kind == PrimitiveKind.pk_ushort:
                    init = "type(0)," + init
                elif typ.key_type.primitive_kind == PrimitiveKind.pk_long:
                    init = "type(0L)," + init
                elif typ.key_type.primitive_kind == PrimitiveKind.pk_ulong:
                    init = "type(0L)," + init
                elif typ.key_type.primitive_kind == PrimitiveKind.pk_long_long:
                    init = "type(0L)," + init
                elif typ.key_type.primitive_kind == PrimitiveKind.pk_float:
                    init = "type(0.0)," + init
                elif typ.key_type.primitive_kind == PrimitiveKind.pk_double:
                    init = "type(0.0)," + init
                else:
                    raise "Unsupported key_type for collection %s" % typ.key_type.primitive_kind
            else:
                raise "Unsupported key_type for collection %s" % typ.key_type.meta_kind
            
        params.outFile.write('%s.%s(%s)'%(fName,fName,init))
    else:
        #Where do we really use Object lists?
        params.outFile.write('None')

def CollectionToName(coll):
    #A collection is references by a property, either attribute or relationship
    #If we are referenced by an attribute we are an object, if we are referenced by a
    #relationship, then we are a literal

    #We should only be referenced by one!!!
    prop = coll.properties
    if len(prop) != 1:
        raise "Too many properties"
    prop = prop[0]
    if prop.meta_kind == MetaKind.mk_attribute:
        ref = 'Literal'
        
    else:
        ref = 'Literal'

    #Now determine the kind
    if coll.collection_kind == CollectionKind.ck_list:
        kind = 'List'
    elif coll.collection_kind == CollectionKind.ck_set:
        kind = 'Set'
    elif coll.collection_kind == CollectionKind.ck_bag:
        kind = 'Bag'
    elif coll.collection_kind == CollectionKind.ck_dictionary:
        kind = 'Dictionary'
    else:
        raise "Unknown Kind %s" % coll.collection_kind


    #Lastly determine what we are storing
    if coll.subtype.meta_kind == MetaKind.mk_class:
        subType = 'Objects'
    elif coll.subtype.meta_kind == MetaKind.mk_primitive_type:
        subType = 'Values'
    else:
        raise "Unknow subtype??"
    

    fName = ref + kind + 'Of' + subType
    return fName


############################################
#Enumerations
############################################
def WriteEnumerationImports(typ,params):
    params.outFile.write(params.indent)
    path = string.split(typ.absolute_name(),'::')
    if path[0] == 'ODLMetaObjects':
        path = path[1:]
    if params.basePackage:
        p = params.basePackage
        for curPath in path:
            if curPath == path[-1]:
                continue
            else:
                p = p + '.' + curPath
            
        params.outFile.write('from %s import %s\n' % (p,path[-1]))
    else:
        if len(path) > 1:
            params.outFile.write('from %s import %s\n' % (string.join(path[:-1],'.'),path[-1]))
        else:
            params.outFile.write('import %s\n' % path[0])
    
def WriteEnumerationInit(typ,params):
    path = typ.absolute_name()
    path = string.split(path,'::')
    if path[0] == 'ODLMetaObjects':
        path = path[1:]
    cPath = string.join(path,'.')
    params.outFile.write("%s.%s.elements[0]" % (path[-1],path[-1]))
    

def EnumerationToName(enum):
    path = enum.absolute_name()
    path = string.split(path, '::')
    if path[0] == 'ODLMetaObjects':
        del path[0]
    module = string.join(path, '.')
    return '%s.%s' % (path[-1], path[-1])
