import Odl


try:
    import os, gettext
    locale_dir = os.path.split(__file__)[0]
    gettext.install('4Suite', locale_dir)
except (ImportError, AttributeError, IOError):
    def _(msg):
        return msg

SYNTAX_ERR_MSG = _("'%.0sSyntax error at or near '%s' Line: %d, Production Number: %s")
INTERNAL_ERR_MSG = _("%.0sInternal error in processing at or near '%s', Line: %d, Production Number: %s, Exception: %s")

class SyntaxException(Exception):
    def __init__(self, source, lineNum, location, prodNum):
        Exception.__init__(self, SYNTAX_ERR_MSG%(source, location, lineNum, prodNum))
        self.source = source
        self.lineNum = lineNum
        self.loc = location
        self.prodNum = prodNum

class InternalException(Exception):
    def __init__(self, source, lineNum, location, prodNum, exc, val, tb):
        Exception.__init__(self, INTERNAL_ERR_MSG%(source, location, lineNum, prodNum, exc))
        self.source = source
        self.lineNum = lineNum
        self.loc = location
        self.prodNum = prodNum
        self.errorType = exc
        self.errorValue = val
        self.errorTraceback = tb

import threading
g_parseLock = threading.RLock()

class OdlParserBase:
    def __init__(self):
        self.initialize()

    def initialize(self):
        self.results = None
        self.__stack = []
        Odl.cvar.g_prodNum = "-1"
        Odl.cvar.g_errorOccured = 0

    def parse(self,st):
        g_parseLock.acquire()
        try:
            self.initialize()
            Odl.my_Odlparse(self,st)
            if Odl.cvar.g_errorOccured == 1:
                raise SyntaxException(
                    st,
                    Odl.cvar.lineNum,
                    Odl.cvar.g_errorLocation,
                    Odl.cvar.g_prodNum)
            if Odl.cvar.g_errorOccured == 2:
                raise InternalException(
                    st,
                    Odl.cvar.lineNum,
                    Odl.cvar.g_errorLocation,
                    Odl.cvar.g_prodNum,
                    Odl.cvar.g_errorType,
                    Odl.cvar.g_errorValue,
                    Odl.cvar.g_errorTraceback)
            return self.__stack
        finally:
            g_parseLock.release()

    def pop(self):
        if len(self.__stack):
            rt = self.__stack[-1]
            del self.__stack[-1]
            return rt
        self.raiseException("Pop with 0 stack length")

    def push(self,item):
        self.__stack.append(item)

    def empty(self):
        return len(self.__stack) == 0

    def size(self):
        return len(self.__stack)

    def raiseException(self, message):
        raise Exception(message + "\n" +
                        "EBNF ProductionNumber: " +
                        str(Odl.cvar.g_prodNum)
                        )

    ### Callback methods ###

    def start(self, arg0 = None, arg1 = None):
        print 'start called with %s %s ' % (str(arg0), str(arg1))
    def specification(self, arg0 = None, arg1 = None):
        print 'specification called with %s %s ' % (str(arg0), str(arg1))
    def definition(self, arg0 = None, arg1 = None):
        print 'definition called with %s %s ' % (str(arg0), str(arg1))
    def _class(self, arg0 = None):
        print 'class called with %s ' % (str(arg0))
    def class_dcl(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'class_dcl called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def class_forward_dcl(self, arg0 = None, arg1 = None):
        print 'class_forward_dcl called with %s %s ' % (str(arg0), str(arg1))
    def class_header(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None, arg5 = None):
        print 'class_header called with %s %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4), str(arg5))
    def type_property_list(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'type_property_list called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def extent_spec(self, arg0 = None, arg1 = None):
        print 'extent_spec called with %s %s ' % (str(arg0), str(arg1))
    def key_spec(self, arg0 = None, arg1 = None):
        print 'key_spec called with %s %s ' % (str(arg0), str(arg1))
    def key_list(self, arg0 = None, arg1 = None, arg2 = None):
        print 'key_list called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def key(self, arg0 = None, arg1 = None, arg2 = None):
        print 'key called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def property_list(self, arg0 = None, arg1 = None, arg2 = None):
        print 'property_list called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def property_name(self, arg0 = None):
        print 'property_name called with %s ' % (str(arg0))
    def module(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None):
        print 'module called with %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4))
    def interface(self, arg0 = None):
        print 'interface called with %s ' % (str(arg0))
    def interface_dcl(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'interface_dcl called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def interface_forward_dcl(self, arg0 = None, arg1 = None):
        print 'interface_forward_dcl called with %s %s ' % (str(arg0), str(arg1))
    def interface_header(self, arg0 = None, arg1 = None, arg2 = None):
        print 'interface_header called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def interface_body(self, arg0 = None, arg1 = None):
        print 'interface_body called with %s %s ' % (str(arg0), str(arg1))
    def export(self, arg0 = None, arg1 = None):
        print 'export called with %s %s ' % (str(arg0), str(arg1))
    def inheritance_spec(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'inheritance_spec called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def scoped_name(self, arg0 = None, arg1 = None, arg2 = None):
        print 'scoped_name called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def const_dcl(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None):
        print 'const_dcl called with %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4))
    def const_type(self, arg0 = None):
        print 'const_type called with %s ' % (str(arg0))
    def const_exp(self, arg0 = None):
        print 'const_exp called with %s ' % (str(arg0))
    def or_expr(self, arg0 = None, arg1 = None, arg2 = None):
        print 'or_expr called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def xor_expr(self, arg0 = None, arg1 = None, arg2 = None):
        print 'xor_expr called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def and_expr(self, arg0 = None, arg1 = None, arg2 = None):
        print 'and_expr called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def shift_expr(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'shift_expr called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def add_expr(self, arg0 = None, arg1 = None, arg2 = None):
        print 'add_expr called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def mult_expr(self, arg0 = None, arg1 = None, arg2 = None):
        print 'mult_expr called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def unary_expr(self, arg0 = None, arg1 = None):
        print 'unary_expr called with %s %s ' % (str(arg0), str(arg1))
    def unary_operator(self, arg0 = None):
        print 'unary_operator called with %s ' % (str(arg0))
    def primary_expr(self, arg0 = None, arg1 = None, arg2 = None):
        print 'primary_expr called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def literal(self, arg0 = None):
        print 'literal called with %s ' % (str(arg0))
    def boolean_literal(self, arg0 = None):
        print 'boolean_literal called with %s ' % (str(arg0))
    def positive_int_const(self, arg0 = None):
        print 'positive_int_const called with %s ' % (str(arg0))
    def type_dcl(self, arg0 = None, arg1 = None):
        print 'type_dcl called with %s %s ' % (str(arg0), str(arg1))
    def type_declarator(self, arg0 = None, arg1 = None):
        print 'type_declarator called with %s %s ' % (str(arg0), str(arg1))
    def type_spec(self, arg0 = None):
        print 'type_spec called with %s ' % (str(arg0))
    def simple_type_spec(self, arg0 = None):
        print 'simple_type_spec called with %s ' % (str(arg0))
    def base_type_spec(self, arg0 = None):
        print 'base_type_spec called with %s ' % (str(arg0))
    def date_type(self, arg0 = None):
        print 'date_type called with %s ' % (str(arg0))
    def time_type(self, arg0 = None):
        print 'time_type called with %s ' % (str(arg0))
    def blob_type(self, arg0 = None):
        print 'blob_type called with %s ' % (str(arg0))
    def interval_type(self, arg0 = None):
        print 'interval_type called with %s ' % (str(arg0))
    def timestamp_type(self, arg0 = None):
        print 'timestamp_type called with %s ' % (str(arg0))
    def template_type_spec(self, arg0 = None):
        print 'template_type_spec called with %s ' % (str(arg0))
    def coll_type(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None, arg5 = None):
        print 'coll_type called with %s %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4), str(arg5))
    def coll_spec(self, arg0 = None):
        print 'coll_spec called with %s ' % (str(arg0))
    def constr_type_spec(self, arg0 = None):
        print 'constr_type_spec called with %s ' % (str(arg0))
    def declarators(self, arg0 = None, arg1 = None, arg2 = None):
        print 'declarators called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def declarator(self, arg0 = None):
        print 'declarator called with %s ' % (str(arg0))
    def simple_declarator(self, arg0 = None):
        print 'simple_declarator called with %s ' % (str(arg0))
    def complex_declarator(self, arg0 = None):
        print 'complex_declarator called with %s ' % (str(arg0))
    def floating_pt_type(self, arg0 = None):
        print 'floating_pt_type called with %s ' % (str(arg0))
    def integer_type(self, arg0 = None):
        print 'integer_type called with %s ' % (str(arg0))
    def signed_int(self, arg0 = None):
        print 'signed_int called with %s ' % (str(arg0))
    def signed_long_int(self, arg0 = None):
        print 'signed_long_int called with %s ' % (str(arg0))
    def signed_long_long_int(self, arg0 = None, arg1 = None):
        print 'signed_long_long_int called with %s %s ' % (str(arg0), str(arg1))
    def signed_short_int(self, arg0 = None):
        print 'signed_short_int called with %s ' % (str(arg0))
    def unsigned_int(self, arg0 = None):
        print 'unsigned_int called with %s ' % (str(arg0))
    def unsigned_long_int(self, arg0 = None, arg1 = None):
        print 'unsigned_long_int called with %s %s ' % (str(arg0), str(arg1))
    def unsigned_short_int(self, arg0 = None, arg1 = None):
        print 'unsigned_short_int called with %s %s ' % (str(arg0), str(arg1))
    def char_type(self, arg0 = None):
        print 'char_type called with %s ' % (str(arg0))
    def boolean_type(self, arg0 = None):
        print 'boolean_type called with %s ' % (str(arg0))
    def octet_type(self, arg0 = None):
        print 'octet_type called with %s ' % (str(arg0))
    def struct_type(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None):
        print 'struct_type called with %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4))
    def member_list(self, arg0 = None, arg1 = None):
        print 'member_list called with %s %s ' % (str(arg0), str(arg1))
    def member(self, arg0 = None, arg1 = None, arg2 = None):
        print 'member called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def union_type(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None, arg5 = None, arg6 = None, arg7 = None, arg8 = None):
        print 'union_type called with %s %s %s %s %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4), str(arg5), str(arg6), str(arg7), str(arg8))
    def switch_type_spec(self, arg0 = None):
        print 'switch_type_spec called with %s ' % (str(arg0))
    def switch_body(self, arg0 = None, arg1 = None):
        print 'switch_body called with %s %s ' % (str(arg0), str(arg1))
    def case(self, arg0 = None, arg1 = None, arg2 = None):
        print 'case called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def case_label_list(self, arg0 = None, arg1 = None):
        print 'case_label_list called with %s %s ' % (str(arg0), str(arg1))
    def case_label(self, arg0 = None, arg1 = None, arg2 = None):
        print 'case_label called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def element_spec(self, arg0 = None, arg1 = None):
        print 'element_spec called with %s %s ' % (str(arg0), str(arg1))
    def enum_type(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None):
        print 'enum_type called with %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4))
    def enumerator_list(self, arg0 = None, arg1 = None, arg2 = None):
        print 'enumerator_list called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def enumerator(self, arg0 = None):
        print 'enumerator called with %s ' % (str(arg0))
    def array_type(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None, arg5 = None):
        print 'array_type called with %s %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4), str(arg5))
    def array_spec(self, arg0 = None):
        print 'array_spec called with %s ' % (str(arg0))
    def string_type(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'string_type called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def array_declarator(self, arg0 = None, arg1 = None):
        print 'array_declarator called with %s %s ' % (str(arg0), str(arg1))
    def array_size_list(self, arg0 = None, arg1 = None):
        print 'array_size_list called with %s %s ' % (str(arg0), str(arg1))
    def fixed_array_size(self, arg0 = None, arg1 = None, arg2 = None):
        print 'fixed_array_size called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def attr_dcl(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'attr_dcl called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def attr_list(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'attr_list called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def attribute_name(self, arg0 = None):
        print 'attribute_name called with %s ' % (str(arg0))
    def domain_type(self, arg0 = None):
        print 'domain_type called with %s ' % (str(arg0))
    def rel_dcl(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None):
        print 'rel_dcl called with %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4))
    def target_of_path(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'target_of_path called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def inverse_traversal_path(self, arg0 = None, arg1 = None, arg2 = None):
        print 'inverse_traversal_path called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def except_dcl(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None):
        print 'except_dcl called with %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4))
    def op_dcl(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None, arg4 = None, arg5 = None):
        print 'op_dcl called with %s %s %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3), str(arg4), str(arg5))
    def op_attribute(self, arg0 = None):
        print 'op_attribute called with %s ' % (str(arg0))
    def op_type_spec(self, arg0 = None):
        print 'op_type_spec called with %s ' % (str(arg0))
    def parameter_dcls(self, arg0 = None, arg1 = None, arg2 = None):
        print 'parameter_dcls called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def param_dcl_list(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'param_dcl_list called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def param_dcl(self, arg0 = None, arg1 = None, arg2 = None):
        print 'param_dcl called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def param_attribute(self, arg0 = None):
        print 'param_attribute called with %s ' % (str(arg0))
    def raises_expr(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'raises_expr called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def scoped_name_list(self, arg0 = None, arg1 = None, arg2 = None):
        print 'scoped_name_list called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def context_expr(self, arg0 = None, arg1 = None, arg2 = None, arg3 = None):
        print 'context_expr called with %s %s %s %s ' % (str(arg0), str(arg1), str(arg2), str(arg3))
    def string_literal_list(self, arg0 = None, arg1 = None, arg2 = None):
        print 'string_literal_list called with %s %s %s ' % (str(arg0), str(arg1), str(arg2))
    def string_literal(self, arg0 = None):
        print 'string_literal called with %s ' % (str(arg0))
    def integer_literal(self, arg0 = None):
        print 'integer_literal called with %s ' % (str(arg0))
    def floating_pt_literal(self, arg0 = None):
        print 'floating_pt_literal called with %s ' % (str(arg0))
    def character_literal(self, arg0 = None):
        print 'character_literal called with %s ' % (str(arg0))
def PrintSyntaxException(e):
    print "********** Syntax Exception **********"
    print "Exception at or near '%s'" % e.loc
    print "  Line: %d, Production Number: %s" % (e.lineNum, str(e.prodNum))

def PrintInternalException(e):
    print "********** Internal Exception **********"
    print "Exception at or near '%s'" % e.loc
    print "  Line: %d, Production Number: %s" % (e.lineNum, e.prodNum)
    print "    Exception: %s" % e.errorType
    print "Original traceback:"
    import traceback
    traceback.print_tb(e.errorTraceback)

if __name__ == "__main__":
    import sys
    p = OdlParserBase()
    if len(sys.argv) == 2:
        l = open(sys.argv[1],"r").read()
    else:
        l = raw_input(">>>")
    try:
        p.parse(l)
    except InternalException, e:
        PrintInternalException(e)
    except SyntaxException, e:
        PrintSyntaxException(e)
