########################################################################
#
# File Name: 	        PostProcessor.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Drivers/PostProcessor.py.html
#
"""
Post process and validate the raw parse tree
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

#There are 3 steps in this file
#1. Validation
#2. Translation
#3. Compression
import string

from Ft.Ods import FtodsParseError
from Ft.Ods import FtodsUnsupportedError

from Ft.Ods.MetaData import Module
from Ft.Ods.MetaData import Class
from Ft.Ods.MetaData import Attribute
from Ft.Ods.MetaData import Relationship
from Ft.Ods.MetaData import Collection
from Ft.Ods.MetaData import Repository
from Ft.Ods.MetaData import Literal
from Ft.Ods.MetaData import ConstOperand
from Ft.Ods.MetaData import Enumeration
from Ft.Ods.MetaData import Constant
#from Ft.Ods.MetaData import Member
#from Ft.Ods.MetaData import Structure
from Ft.Ods.MetaData.PrimitiveKind import PrimitiveKind
from Ft.Ods.MetaData.CollectionKind import CollectionKind

from Ft.Ods.Collections import LiteralCollection

#Need so we can complete both sides
g_relationships = {}
g_forwards = {}
g_collectionNameCtr = 0
g_currentDefScope = None


def PostProcess(ptree):

    if len(ptree) != 1:
        raise FtodsParseError(FtodsParseError.TOO_MANY_TOP_LEVEL)
    ptree = ptree[0]

    if ptree['TYPE'] != 'START':
        raise FtodsParseError(FtodsParseError.WRONG_TOP_LEVEL)

    #NOTE: We try to validate as much as we can in the first pass (Process stuff)
    #Some things, ie relationships can only be validated in a second pass


    global g_relationships
    global g_forwards
    global g_collectionNameCtr
    global g_currentDefScope
    g_relationships = {}
    g_forwards = {}
    g_collectionNameCtr = 0
    g_currentDefScope = None


    if ptree[0] == '~':
        #Simple parse of an expression
        return ProcessConstExp(ptree[1],None)
    spec = ptree[0]
    #Parse a sepcification

    from Ft.Ods import Database
    db = Database.Database()

    global g_repository
    g_repository = db.schema()
    topMod = g_repository.resolve('ODLMetaObjects')

    ProcessSpecification(spec,topMod)

    global g_relationships
    if len(g_relationships.keys()):
        for n in g_relationships.keys():
            print "No inverse to relationship %s.%s" % (str(n[0].name),n[1])
        raise "Inverse Relationship Error"

    return

#Rule 1
def ProcessSpecification(spec,parent):

    #A Specification is really a list of definitions
    #Process this definition and then see if we need to process a specification

    ProcessDefinition(spec[0],parent)
    if spec[1]:
        ProcessSpecification(spec[1],parent)
    return

#Rule 2
def ProcessDefinition(d,parent):

    #A definition is anything top level
    #Split based on type
    if d[0]['TYPE'] == 'TYPE_DCL':
        ProcessTypeDcl(d[0],parent)
    elif d[0]['TYPE'] == 'CONST_DCL':
        ProcessConstDcl(d[0],parent)
    elif d[0]['TYPE'] == 'EXCEPT_DCL':
        ProcessExceptDcl(d[0],parent)
    elif d[0]['TYPE'] == 'INTERFACE':
        ProcessInterface(d[0],parent)
    elif d[0]['TYPE'] == 'MODULE':
        ProcessModule(d[0],parent)
    elif d[0]['TYPE'] == 'CLASS':
        ProcessClass(d[0],parent)
    else:
        raise "How did Bison let me get here?"

#2a
def ProcessClass(klass,parent):

    if klass[0]['TYPE'] == 'CLASS_FORWARD_DCL':
        ProcessClassForwardDcl(klass[0],parent)
    elif klass[0]['TYPE'] == 'CLASS_DCL':
        ProcessClassDcl(klass[0],parent)
    else:
        raise "How did Bison let me get here?"


#2b
def ProcessClassDcl(klass, parent):

    #A class is pretty simple, it can have a header and a body.  That's it
    name, extender, inherits, propList = ProcessClassHeader(klass[0], parent)

    #See if this was forward declared
    absoluteName = '%s::%s' % (parent.absolute_name(), name)
    
    c = g_forwards.get(absoluteName)
    if c:
        #This has been declared before, use it and fill in the new information
        if inherits:
            for i in inherits:
                c.add_inherits(i)
        if extender:
            c.form_extender(extender)
    else:
        c = parent.add_class(name, inherits, extender)

    # Allows for multiple forward declarations
    g_forwards[absoluteName] = None

    (extent, keys) = propList
        
    if extent:
        c.extents.insert_element(extent)

    for k in keys:
        c.keys.insert_element(k)

    ProcessInterfaceBody(klass[2],c)

    return

#2c
def ProcessClassForwardDcl(forward, parent):

    #Create a new instance of a class so it can be used later
    name = forward[1]
    absoluteName = '%s::%s' % (parent.absolute_name(), name)
    if not g_forwards.has_key(absoluteName):
        klass = parent.add_class(name, None, None)
        g_forwards[absoluteName] = klass


#2d
def ProcessClassHeader(header,parent):
    #This is one of the most complex definitions in the system
    #a class header in logical form is (extends ext_name)? (inheritance_spec)? (type_property_list)?

    #FIXME we do not support extends, inheritance, or type lists
    extends = None
    inherits = None
    propList = (None,[])
    curCtr = 2
    if header[2] == 'extends':
        extends = ProcessScopedName(header[3],parent)
        curCtr = curCtr + 2
    if header[curCtr] and header[curCtr]['TYPE'] == 'INHERITANCE_SPEC':
         inherits = ProcessInheritanceSpec(header[curCtr],parent)
    if header[curCtr] and header[curCtr]['TYPE'] == 'TYPE_PROPERTY_LIST':
        propList = ProcessTypePropertyList(header[curCtr],parent)

    name = header[1]
    return name, extends, inherits, propList


#2e
def ProcessTypePropertyList(tpl,parent):

    #NOTE the spec says there can be 2 of these, we only support 1
    extent = None
    keys = []

    curCtr = 1
    
    if tpl[1] != ')':
        if tpl[1]['TYPE'] == 'EXTENT_SPEC':
            extent = ProcessExtentSpec(tpl[1], parent)
            curCtr = curCtr + 1
        if tpl[curCtr] != ')' and tpl[curCtr]['TYPE'] == 'KEY_SPEC':
            keys = ProcessKeySpec(tpl[curCtr],parent)
    return extent, keys

#2f
def ProcessExtentSpec(ext,parent):
    return ext[1]

#2g
def ProcessKeySpec(ks,parent):
    raise FtodsUnsupportedError("Key Specification")

#2h
def ProcessKeyList(kl,parent):
    raise FtodsUnsupportedError("Key List")

#2i
def ProcessKey(k,parent):
    raise FtodsUnsupportedError("Key")

#2j
def ProcessPropertyList(pl,parent):
    raise FtodsUnsupportedError("Property List")

#2k
def ProcessPropertyName(pn,parent):
    raise FtodsUnsupportedError("Property Name")

#3
def ProcessModule(mod,parent):

    #Simple wrapper around a specification list
    name = mod[1]
    m = parent.add_module(name)
    ProcessSpecification(mod[3],m)
    return m




#8
def ProcessInterfaceBody(ib,parent):

    ProcessExport(ib[0],parent)
    if ib[1]:
        ProcessInterfaceBody(ib[1],parent)

#9
def ProcessExport(export,parent):
    #An export is a union of attr_dcl, rel_dcl,type_dcl,const_dcl,execpt_dcl
    if export[0]['TYPE'] == 'TYPE_DCL':
        return ProcessTypeDcl(export[0], parent)
    elif export[0]['TYPE'] == 'CONST_DCL':
        return ProcessConstDcl(export[0], parent)
    elif export[0]['TYPE'] == 'EXCEPT_DCL':
        return ProcessExceptDcl(export[0], parent)
    elif export[0]['TYPE'] == 'ATTR_DCL':
        return ProcessAttrDcl(export[0], parent)
    elif export[0]['TYPE'] == 'REL_DCL':
        return ProcessRelDcl(export[0], parent)
    elif export[0]['TYPE'] == 'OP_DCL':
        return ProcessOpDcl(export[0], parent)
    else:
        raise "Huh?"


#11
def ProcessScopedName(sn,parent):
    name = RebuildScopedName(sn)
    rt = ResolveName(name,parent)
    if rt == None:
        raise "Unresolved Scoped Name %s" % name
    return rt


def RebuildScopedName(sn):
    name = ''
    if not sn[1]:
        #Simple Name
        name =  sn[0]
    elif not sn[2]:
        #Global scoped name
        name =  sn[1]
    else:
        #A name in a chain
        first = RebuildScopedName(sn[0])
        name =  first + '::' + sn[2]

    return name


#12
def ProcessConstDcl(cd,parent):
    #0: const
    #1: const_type
    #2: IDENT
    #3: =
    #4: const_exp

    typ = ProcessConstType(cd[1],parent)
    name = cd[2]
    op = ProcessConstExp(cd[4],parent)
    op.name = name

    #Create the const on the parent
    con = parent.add_constant(name,typ,op)
    return con


#13
def ProcessConstType(ct,parent):
    #A Union of types allowed in constants
    if ct[0]['TYPE'] == 'INTEGER_TYPE':
        return ProcessIntegerType(ct[0],parent)
    elif ct[0]['TYPE'] == 'CHAR_TYPE':
        return ProcessCharType(ct[0],parent)
    elif ct[0]['TYPE'] == 'BOOLEAN_TYPE':
        return ProcessBooleanType(ct[0],parent)
    elif ct[0]['TYPE'] == 'FLOATING_PT_TYPE':
        return ProcessFloatingPtType(ct[0],parent)
    elif ct[0]['TYPE'] == 'STRING_TYPE':
        return ProcessStringType(ct[0],parent)
    elif ct[0]['TYPE'] == 'SCOPED_NAME':
        return ProcessScopedName(ct[0],parent)
    else:
        raise "Huh?"

#14
def ProcessConstExp(ce,parent):
    #Parent could be none
    #Is this bad???
    return ProcessOrExpr(ce[0],parent)

#15
def ProcessOrExpr(oe,parent):
    if oe[1]:
        raise FtodsUnsupportedError('|')
    else:
        return ProcessXorExpr(oe[0],parent)
#16
def ProcessXorExpr(xe,parent):
    if xe[1]:
        raise FtodsUnsupportedError('^')
    else:
        return ProcessAndExpr(xe[0],parent)

#17
def ProcessAndExpr(ae,parent):
    if ae[1]:
        raise FtodsUnsupportedError('&')
    else:
        return ProcessShiftExpr(ae[0],parent)

#18
def ProcessShiftExpr(se,parent):
    if se[1]:
        raise FtodsUnsupportedError('<< | >>')
    else:
        return ProcessAddExpr(se[0],parent)

#19
def ProcessAddExpr(ae,parent):
    if ae[1]:
        raise FtodsUnsupportedError('+ | -')
    else:
        return ProcessMultExpr(ae[0],parent)

#20
def ProcessMultExpr(me,parent):
    if me[1]:
        raise FtodsUnsupportedError('* | / | %')
    else:
        return ProcessUnaryExpr(me[0],parent)
#21
def ProcessUnaryExpr(ue,parent):
    if ue[1]:
        raise FtodsUnsupportedError('+ | ~')
    else:
        return ProcessPrimaryExpr(ue[0],parent)

#23
def ProcessPrimaryExpr(pe,parent):
    if pe[0]['TYPE'] == 'LITERAL':
        return ProcessLiteral(pe[0],parent)
    elif pe[0]['TYPE'] == 'SCOPED_NAME':
        #This needs to be a ConstOperand!!!!
        const = ProcessScopedName(pe[0],parent)
        co = ConstOperand.ConstOperand()
        co.form_references(const)
        return co
    else:
        raise FtodsUnsupportedError(pe[0]['TYPE'])
    
#24
def ProcessLiteral(lit,parent):
    if lit[0]['TYPE'] == 'INTEGER_LITERAL':
        #An integer Literal
        l = Literal.Literal()
        l.literal_value = int(lit[0][0])
        return l
    elif lit[0]['TYPE'] == 'FLOATING_PT_LITERAL':
        l = Literal.Literal()
        l.literal_value = float(lit[0][0])
        return l
    elif lit[0]['TYPE'] == 'CHARACTER_LITERAL':
        l = Literal.Literal()
        l.literal_value = str(lit[0][0])[1]
        return l
    elif lit[0]['TYPE'] == 'STRING_LITERAL':
        l = Literal.Literal()
        l.literal_value = str(lit[0][0][1:-1])
        return l
    elif lit[0]['TYPE'] == 'BOOLEAN_LITERAL':
        l = Literal.Literal()
        l.literal_value = lit[0][0] == 'true'
        return l
    else:
        raise FtodsUnsupportedError(lit[0]['TYPE'])
        


#27
def ProcessTypeDcl(td,parent):

    if td[0]['TYPE'] == 'ENUM_TYPE':
        tdcl = ProcessEnumType(td[0],parent)
    elif td[0]['TYPE'] == 'STRUCT_TYPE':
        tdcl = ProcessStructType(td[0],parent)
    else:
        #FIXME don't support rest of type dcls
        raise FtodsUnsupportedError(td[0]['TYPE'])

    #Create a type for this td
    return tdcl


#29
def ProcessTypeSpec(ts,parent):

    #A union of simple_type and constr_type
    if ts[0]['TYPE'] == 'SIMPLE_TYPE_SPEC':
        return ProcessSimpleTypeSpec(ts[0],parent)
    elif ts[0]['TYPE'] == 'CONSTR_TYPE_SPEC':
        return ProcessConstrTypeSpec(ts[0],parent)
    else:
        raise "Huh?"



#30
def ProcessSimpleTypeSpec(st,parent):
    #A union for base_type,template_type or a scoped name
    if st[0]['TYPE'] == 'BASE_TYPE_SPEC':
        return ProcessBaseTypeSpec(st[0],parent)
    elif st[0]['TYPE'] == 'TEMPLATE_TYPE_SPEC':
        return ProcessTemplateTypeSpec(st[0],parent)
    elif st[0]['TYPE'] == 'SCOPED_NAME':
        sn = ProcessScopedName(st[0],parent)
        return sn
    else:
        raise FtodsUnsupportedError(st[0]['TYPE'])        

#31
def ProcessBaseTypeSpec(bt,parent):
    #A unions of all the base types
    #FIXME only int is supported
    if bt[0]['TYPE'] == 'FLOATING_PT_TYPE':
        return ProcessFloatingPtType(bt[0],parent)
    elif bt[0]['TYPE'] == 'INTEGER_TYPE':
        return ProcessIntegerType(bt[0],parent)
    elif bt[0]['TYPE'] == 'CHAR_TYPE':
        return ProcessCharType(bt[0],parent)
    elif bt[0]['TYPE'] == 'BOOLEAN_TYPE':
        return ProcessBooleanType(bt[0],parent)
    elif bt[0]['TYPE'] == 'OCTET_TYPE':
        raise "Not Supported"
        pk = PrimitiveKind.pk_octet
    elif bt[0]['TYPE'] == 'DATE_TYPE':
        raise "Not Supported"
        return ProcessDateType(bt[0],parent)
    elif bt[0]['TYPE'] == 'TIME_TYPE':
        raise "Not Supported"
        return ProcessTimeType(bt[0],parent)
    elif bt[0]['TYPE'] == 'INTERVAL_TYPE':
        raise "Not Supported"
        return ProcessIntervalType(INTERVAL_TYPE)
    elif bt[0]['TYPE'] == 'TIMESTAMP_TYPE':
        raise "Not Supported"
        return ProcessTimestampType(bt[0],parent)
    elif bt[0]['TYPE'] == 'BLOB_TYPE':
        return Database.Database().schema().resolve(PrimitiveKind.pk_blob._n)
    else:
        raise "Huh?"



#32
def ProcessTemplateTypeSpec(tt,parent):
    #A union of array_type, string_type, or coll_type
    #FIXME no support for array and coll
    if tt[0]['TYPE'] == 'ARRAY_TYPE':
        return ProcessArrayType(tt[0],parent)
    elif tt[0]['TYPE'] == 'STRING_TYPE':
        return ProcessStringType(tt[0],parent)
    elif tt[0]['TYPE'] == 'COLL_TYPE':
        return ProcessCollType(tt[0],parent)
    else:
        raise "Huh?"

#32a
def ProcessCollType(ct,parent):
    if ct[5] != None:
        raise NotImplementedError("Dictionaries are not supported")
        #
        #0 dictionary
        #1 <
        #2 simple_type_spec
        #3 ,
        #4 simple_type_spec
        #5 >

        keyType = ProcessSimpleTypeSpec(ct[2],parent)
        valueType = ProcessSimpleTypeSpec(ct[4],parent)
        coll = parent.create_dictionary_type(keyType,valueType)
    else:
        #
        #0 coll_spec
        #1 <
        #2 simple_type_spec
        #3 >
        coll = ProcessCollSpec(ct[0],parent)
        st = ProcessSimpleTypeSpec(ct[2],parent)
        coll.form_subtype(st)
    return coll

#32b
g_collectionCtr = 0
def ProcessCollSpec(cs,parent):
    if cs[0] == 'list':
        t = Collection.Collection(collection_kind = CollectionKind.ck_list)
    elif cs[0] == 'bag':
        t = Collection.Collection(collection_kind = CollectionKind.ck_bag)
    elif cs[0] == 'set':
        t = Collection.Collection(collection_kind = CollectionKind.ck_set)
    t.name = "Collection%d" % g_collectionCtr
    global g_collectionCtr
    g_collectionCtr = g_collectionCtr + 1
    return t

#33
def ProcessConstrTypeSpec(cts,parent):
    #A simple union of struct_type,enum_type, and union_type
    if cts[0]['TYPE'] == 'STRUCT_TYPE':
        return ProcessStructType(cts[0],parent)
    elif cts[0]['TYPE'] == 'ENUM_TYPE':
        return ProcessEnumType(cts[0],parent)
    else:
        raise FtodsUnsupportedError(cts[0]['TYPE'])        


#34
def ProcessDeclarators(decs,parent):
    ds = [ProcessDeclarator(decs[0],parent)]
    if decs[2]:
        ds = ds + ProcessDeclarators(decs[2],parent)
    return ds

#35
def ProcessDeclarator(dec,parent):
    # A union of simple declarator and complex declarator
    if dec[0]['TYPE'] == 'SIMPLE_DECLARATOR':
        return dec[0][0]
    else:
        raise FtodsUnsupportedError(dec[0]['TYPE'])        
    


#38
def ProcessFloatingPtType(ft,parent):
    typ = ft[0]
    if ft[0] == 'float':
        t = PrimitiveKind.pk_float
    elif ft[0] == 'double':
        t = PrimitiveKind.pk_double
    else:
        raise "Unknown Floating Point type: %s" % ft[0]

    return Database.Database().schema().resolve(t._n)


#39
def ProcessIntegerType(it,parent):
    typ = it[0][0]['TYPE']
    name = it[0][0][0]
    if it[0][0].has_key(1):
        name = name + ' ' + it[0][0][1]
    if typ == 'UNSIGNED_SHORT_INT':
        t = PrimitiveKind.pk_ushort
    elif typ == 'SIGNED_SHORT_INT':
        t = PrimitiveKind.pk_short
    elif typ == 'UNSIGNED_LONG_INT':
        t = PrimitiveKind.pk_ulong
    elif typ == 'SIGNED_LONG_INT':
        t = PrimitiveKind.pk_long
    elif typ == 'SIGNED_LONG_LONG_INT':
        t = PrimitiveKind.pk_long_long
    else:
        raise "Unknown Integer type %s" % typ
    return Database.Database().schema().resolve(t._n)


#46
def ProcessCharType(ct,parent):
    pk = PrimitiveKind.pk_char
    return Database.Database().schema().resolve(pk._n)

#47
def ProcessBooleanType(ct,parent):
    pk = PrimitiveKind.pk_boolean
    return Database.Database().schema().resolve(pk._n)



#50
def ProcessStructType(struct,parent):

    #0:struct
    #1:name
    #2:{
    #3:member_list
    #4:}
    
    name = struct[1]

    if name == 'PhoneNum':
        print parent
        raise "Boo"

    members = ProcessMemberList(struct[3], parent)

    s = parent.add_structure(name,members)
    return s

#51
def ProcessMemberList(memberList,parent):

    list = ProcessMember(memberList[0],parent)
    if memberList[1]:
        list = list + ProcessMemberList(memberList[1],parent)
    return list

#52
def ProcessMember(member,parent):
    typ = ProcessTypeSpec(member[0],parent)
    decls = ProcessDeclarators(member[1],parent)
    members = []
    for d in decls:
        member = parent.create_member(d,typ)
    return members
        




#59
def ProcessEnumType(enum,parent):
    #0: enum
    #1: name
    #2: {
    #3: enum_list
    #4: }

    name = enum[1]
    enumList = ProcessEnumeratorList(enum[3],parent)
    enumList.reverse()
    #Turn this into a ODS List
    li = LiteralCollection.LiteralListOfValues()
    for el in enumList:
        li.insert_element(el)
    e = parent.add_enumeration(name,li)
    e.name = name
    return e

#59a
def ProcessEnumeratorList(el,parent):

    #0:enum
    #1: ,?
    #2  enumList?

    enums = []
    if el[1]:
        enums = enums + ProcessEnumeratorList(el[2],parent)
    enums.append(el[0][0])
    return enums

#60
#Not needed








#62
def ProcessStringType(st,path):
    # a string is fixed length or variable length
    #FIXME no support for fixedlength strings
    if st[1] != None:
        raise FtodsUnsupportedError("Fixed Length Strings")
    return Database.Database().schema().resolve('pk_string')
        



#65
def ProcessAttrDcl(attr,parent):
    #A simple attribute definition, could be readonly or an array

    curCtr = 0
    readOnly = 0
    #FIXME No support for readonly or arrays
    if attr[3] != None:
        readOnly = 1
        curCtr = 1

    dt = ProcessDomainType(attr[curCtr + 1],parent)
    names = ProcessAttrList(attr[curCtr + 2],parent)

    for name in names:
        a = parent.add_attribute(name,dt)
        if readOnly:
            a.is_read_only = 1

    return

#65a
def ProcessAttrList(attrList,parent):

    #FIXME What does it mean to have an array here???

    if attrList[2] != None:
        raise FtodsUnsupportedError("Arrays")
    if attrList[1] != None and attrList[1]['TYPE'] == 'FIXED_ARRAY_SIZE':
        raise FtodsUnsupportedError("Arrays")

    names = [ProcessAttributeName(attrList[0],parent)]

    if attrList[1]:
        names = names + ProcessAttrList(attrList[1],parent)

    return names
    
#65b
def ProcessAttributeName(aName,parent):
    return aName[0]







#65c
def ProcessDomainType(dt,parent):
    #A uion type for simple_typ, struct_type, and enum_type
    if dt[0]['TYPE'] == 'SIMPLE_TYPE_SPEC':
        return ProcessSimpleTypeSpec(dt[0],parent)
    elif dt[0]['TYPE'] == 'STRUCT_TYPE':
        return ProcessStructType(dt[0],parent)
    elif dt[0]['TYPE'] == 'ENUM_TYPE':
        return ProcessStructType(dt[0],parent)
    else:
        raise FtodsUnsupportedError(dt[0]['TYPE'])        
        

#65d
def ProcessRelDcl(rel,parent):

    #A combination
    #0 relationship
    #1 target
    #2 ident
    #3 invers
    #4 inverse target


    #2 cases?
    #1 this is the first side of the relationship: inwhich we store in the list
    #2 this is the second side, inwhich we get the inverse from the list

    #Get the target (it is a type)
    rel_name = rel[2]
    rel_type = ProcessTargetOfPath(rel[1],parent)

    target,inverse_name = ProcessInverseTraversalPath(rel[4],parent)
    if rel_type == None:
        raise "Unknown type for relationship %s" % rel_name
    if target == None:
        raise "No target for relationship %s" % rel_name

    #See if the inverse rel exists
    global g_relationships
    if g_relationships.has_key((target,inverse_name)):
        #Case 2 the other side was already defined
        inverse_type = g_relationships[(target,inverse_name)]
        parent.add_relationship(rel_name,rel_type,target,inverse_name,inverse_type)
        del g_relationships[(target,inverse_name)]
    else:
        #Case1 the other side was not declared.
        #Set up to be called later
        g_relationships[(parent,rel_name)] = rel_type

#65e
def ProcessTargetOfPath(top,parent):
    
    #Either a simepl type of a collection
    if top[1]:
        #This is a collection
        typ = ProcessCollSpec(top[0],parent)
        subtype = ProcessScopedName(top[2],parent)
        typ.form_subtype(subtype)
        return typ
    else:
        #This is a simple
        typ = ProcessScopedName(top[0],parent)
        return typ


#65f
def ProcessInverseTraversalPath(itp,path):

    #A simple container
    #0 scoped name
    #1 ::
    #2 ident
    target = ProcessScopedName(itp[0],path)
    return target,itp[2]





















from Ft.Ods.MetaData import Scope
from Ft.Ods import Database
def ResolveName(name,scope):


    #A name can reference something in 2 places:
    #1 the scope of  scope's parent
    #2 the root module ODLMetaObjects
    noGlob = 0
    if name[0] == ':':
        name = name[2:]
        noGlob = 1

    if not noGlob:
        #First try parent's current scope
        p = scope.parent()
        if p:
            try:
                n = p.resolve(name)
                return n
            except Scope.Scope.NameNotFound, e:
                pass
        #try at current scope
        try:
            n = scope.resolve(name)
            return n
        except Scope.Scope.NameNotFound, e:
            pass

    #Now try at the ODLMetaObject module.
    try:
        n = Database.Database().schema().resolve("ODLMetaObjects").resolve(name)
        return n
    except Scope.Scope.NameNotFound, e:
        pass

    #Now try at the Repository module.
    try:
        n = Database.Database().schema().resolve(name)
        return n
    except Scope.Scope.NameNotFound, e:
        pass

    return None
