import cStringIO, string

from Ft.Ods.PersistentObject import TupleDefinitions
from Ft.Ods.MetaData.MetaKind import MetaKind
from Ft.Ods.MetaData.PrimitiveKind import PrimitiveKind
import GenerateType
TD = TupleDefinitions

def GetBases(name, objects, params):
    bases = []
    for object in objects:
        names = string.split(object.absolute_name(), '::')
        if names[0] == 'ODLMetaObjects':
            del names[0]
        module = string.join(names, '.')
        bases.append('%s.%s_stub' % (module, names[-1]))
        GenerateType.WriteImports(object, params)
    return bases

def GlobalFactory(write, name):
    write('class %sFactory:\n\n' % name)
    write('    def new(self):\n')
    write('        return %s_stub()\n\n' % name)

    write('g_%sFactory = %sFactory()\n\n' % (name, name))

    write('def new():\n')
    write('    return g_%sFactory.new()\n\n' % name)
    return

def ClassHeader(write, name, bases):
    write('class %s_stub' % name)
    if bases:
        write('(%s' % bases[0])
        for base in bases[1:]:
            write(', %s' % base)
        write(')')
    write(':\n\n')

    write('    def __init__(self, data=None):\n')
    for base in bases:
        write('        %s.__init__(self, data)\n' % base)
    return

def AttributeInit(attributes, params):
    if attributes:
        params.outFile.write('        if data is None:\n')
        params.outFile.write('            pass\n')
        for attr in attributes:
            if attr.type.meta_kind not in [MetaKind.mk_class,
                                           MetaKind.mk_interface,
                                           MetaKind.mk_collection]:
                if (attr.type.meta_kind == MetaKind.mk_primitive_type and
                    attr.type.primitive_kind == PrimitiveKind.pk_blob):
                    continue
                params.outFile.write('            self.__dict__["%s"] = ' % attr.name)
                GenerateType.WriteInit(attr.type, params)
            params.outFile.write('\n')
    params.outFile.write('\n')
    return

def RelationshipInit(relationship, params):
    write = params.outFile.write
    traversal = relationship.traversal
    absoluteName = string.split(traversal.definedIn.absolute_name(), '::')
    if absoluteName[0] == 'ODLMetaObjects':
        del absoluteName[0]
    absoluteName.append(absoluteName[-1])
    className = string.join(absoluteName, '.') + '_stub'
    name = relationship.name

    nSided = traversal._4ods_getCardinality() == 'N'
    params.indent = '        '
    if relationship._4ods_getCardinality() == '1':
        write('    def form_%s(self, target, inverse=1):\n' % name)
        GenerateType.WriteImports(relationship.type, params)
        FormRelationship(write, name, className, traversal.name, nSided)
        write('    def drop_%s(self, target, inverse=1):\n' % name)
        GenerateType.WriteImports(relationship.type, params)
        DropRelationship(write, name, traversal.name, nSided)
    else:
        write('    def add_%s(self, target, inverse=1):\n' % name)
        GenerateType.WriteImports(relationship.type, params)
        AddRelationship(write, name, className, traversal.name, nSided)

        write('    def remove_%s(self, target, inverse=1):\n' % name)
        GenerateType.WriteImports(relationship.type, params)
        RemoveRelationship(write, name, traversal.name, nSided)
    params.indent = ''
    return    
    
def FormRelationship(write, name, className, inverseOf, nSided):
    write('        if not isinstance(target, %s):\n' % className)
    write('            from Ft.Ods import IntegrityException\n')
    write('            raise IntegrityException("%s")\n' % name)
    write('        if getattr(self, "%s"):\n' % name)
    write('            from Ft.Ods import IntegrityException\n')
    write('            raise IntegrityException("%s")\n' % name)
    write('        self.__dict__["%s"] = target\n' % name)
    write('        if inverse:\n')
    if nSided:
        write('            target.add_%s(self, 0)\n' % inverseOf)
    else:
        write('            target.form_%s(self, 0)\n' % inverseOf)
    write('        self.__dict__["_modifiedAttrs"]["%s"] = 1\n' % name)
    write('\n')
    return

def AddRelationship(write, name, className, inverseOf, nSided):
    write('        if not isinstance(target, %s):\n' % className)
    write('            from Ft.Ods import IntegrityException\n')
    write('            raise IntegrityException("%s")\n' % name)
    write('        collection = getattr(self, "%s")\n' % name)
    write('        if not collection.allows_duplicates() and collection.contains_element(target):\n')
    write('            from Ft.Ods import IntegrityException\n')
    write('            raise IntegrityException("%s")\n' % name)
    write('        collection.insert_element(target)\n')
    write('        if inverse:\n')
    if nSided:
        write('            target.add_%s(self, 0)\n' % inverseOf)
    else:
        write('            target.form_%s(self, 0)\n' % inverseOf)
    write('\n    form_%s = add_%s\n\n' % (name, name))
    return        

def DropRelationship(write, name, inverseOf, nSided):
    write('        if self.__dict__.has_key("%s"):\n' % name)
    write('            if self.__dict__["%s"] != target:\n' % name)
    write('                from Ft.Ods import IntegrityException\n')
    write('                raise IntegrityException("%s")\n' % name)
    write('        elif not self.__dict__["_objectIds"].has_key("%s") and self.__dict__["_objectIds"]["%s"] != target._4ods_getId():\n' % (name, name))
    write('            from Ft.Ods import IntegrityException\n')
    write('            raise IntegrityException("%s")\n' % name)
    write('        if self.__dict__.has_key("%s"):\n' % name)
    write('            del self.__dict__["%s"]\n' % name)
    write('        if self.__dict__["_objectIds"].has_key("%s"):\n' % name)
    write('            del self.__dict__["_objectIds"]["%s"]\n' % name)
    write('        if inverse:\n')
    if nSided:
        write('            target.remove_%s(self, 0)\n' % inverseOf)
    else:
        write('            target.drop_%s(self, 0)\n' % inverseOf)
    write('        self.__dict__["_modifiedAttrs"]["%s"] = 1\n\n' % name)


def RemoveRelationship(write, name, inverseOf, nSided):
    write('        collection = getattr(self, "%s")\n' % name)
    write('        if not collection.contains_element(target):\n')
    write('            from Ft.Ods import IntegrityException\n')
    write('            raise IntegrityException("%s")\n' % name)
    write('        collection.remove_element(target)\n')
    write('        if inverse:\n')
    if nSided:
        write('            target.remove_%s(self, 0)\n' % inverseOf)
    else:
        write('            target.drop_%s(self, 0)\n' % inverseOf)
    write('\n    drop_%s = remove_%s\n\n' % (name, name))


def TupleDefinitions(defines, bases, params):
    write = params.outFile.write
    stream = cStringIO.StringIO()
    output = stream.write

    names = []
    types = []
    ids = []
    collections = {}
    enumerations = []
    
    output('    _tupleDefinitions = %s._tupleDefinitions.copy()\n' % bases[0])
    for base in bases[1:]:
        output('    _tupleDefinitions.update(%s._tupleDefinitions)\n' % base)

    tupleDefs = '    _tupleDefinitions.update({'
    name_padding = ' '*len(tupleDefs)
    output(tupleDefs)
    for define in defines:
        if define.meta_kind in [MetaKind.mk_attribute, MetaKind.mk_relationship]:
            name = define.name
            odmgType = define._4ods_getOdmgType()
            names.append(name)
            types.append(odmgType)
            ids.append(define._4ods_getId())

            output(repr(name) + ' : {')
            defs_padding = ' '*(len(name_padding) + len(repr(name)) + 4)
            output('%d : %d,\n' % (TD.TYPE, odmgType))
            output(defs_padding)
            if define.meta_kind == MetaKind.mk_relationship or define.is_read_only:
                output('%d : 1,\n' % TD.READONLY)
                output(defs_padding)
            if define.type.meta_kind == MetaKind.mk_collection:
                collectionName = GenerateType.CollectionToName(define.type)
                if collectionName[:7] == 'Literal':
                    collections['LiteralCollection'] = 1
                    collectionName = 'LiteralCollection.%s' % collectionName
                else:
                    raise NotImplementedError('No ObjectCollections yet!')
                output('%d : %s,\n' % (TD.COLLECTION_CLASS,
                                       collectionName))
                output(defs_padding)
            if define.type.meta_kind == MetaKind.mk_enumeration:
                enumerationName = GenerateType.EnumerationToName(define.type)
                enumerations.append(define.type)
                output('%d : %s,\n' % (TD.ENUM_CLASS, enumerationName))
                output(defs_padding)
            if define.meta_kind == MetaKind.mk_relationship:
                output('%d : 1,\n' % (TD.RELATIONSHIP))
                output(defs_padding)
            output('},\n%s' % name_padding)
    output('})\n')

    for collection in collections.keys():
        write('    from Ft.Ods.Collections import %s\n' % collection)
    params.indent = '    '
    #for enumeration in enumerations:
    #    GenerateType.WriteEnumerationImports(enumeration, params)
    params.indent = ''
    write(stream.getvalue())
    stream.close()

    InheritedTuples(write, '_tupleNames', names, bases)
    InheritedTuples(write, '_tupleTypes', types, bases)
    InheritedTuples(write, '_tupleIds', ids, bases)

    return

def InheritedTuples(write, name, value, bases):
    write('    %s = %s.%s + ' % (name, bases[0], name))
    for base in bases[1:]:
        write('    %s.%s + ' % (base, name))
    write('%s\n' % str(tuple(value)))
    return
