########################################################################
#
# File Name: 	    DbmManagement.py
#
# Documentation:	http://docs.4suite.org/4ODS/StorageManager/Adapters/DbmManagement.py.html
#
"""
The DBM Database Management functions
WWW: http://4suite.org/             e-mail: support@4suite.org

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.StorageManager.Adapters import Util, Constants, Manager

from Ft.Lib import DbmDatabase

### Management Functions ###

class DbManager(Manager.DbManager):

    def connect(self,connectString):
        dbName = Util.SplitDbConnectString(connectString)[0]
        return DbmDatabase.Database('ftods__' + dbName)

    def create(self,connectString):
        dbName = 'ftods__' + Util.SplitDbConnectString(connectString)[0]
        DbmDatabase.CreateDatabase(dbName)

    def exists(self,connectString):
        dbName = 'ftods__' + Util.SplitDbConnectString(connectString)[0]
        try:
            return dbName in DbmDatabase.GetAllDatabaseNames()
        except:
            return 0
    def destroy(self,connectString):
        dbName = 'ftods__' + Util.SplitDbConnectString(connectString)[0]
        return DbmDatabase.DropDatabase(dbName)

    def close(self,conn):
        conn.rollback()

    def init(self,connString):

        db = self.connect(connString)


        #The tables
        for name in self.tables:
            db.createTable(name)

        #The sequences
        t = db['sequences']
        for s in self.sequences:
            t[s] = 2


        db.commit()


    def initRepo(self,connString,adapter):

        #Create the initial structure
        db = self.connect(connString)
        adapter.begin(db)

        ex = db['extents']

        #Add the extent Tuple
        ex['ftods_types'] = (Constants.ObjectTypes.REPO,[])

        #Create the repository, and two sub modules and the types
        from Ft.Ods.MetaData import Repository
        from Ft.Ods.MetaData import Module
        from Ft.Ods.MetaData import PrimitiveType
        from Ft.Ods.MetaData import PrimitiveKind
        from Ft.Ods.Collections import LiteralCollection

        adapter.newPythonClass(db,Repository.Repository)
        adapter.newPythonClass(db,Module.Module)
        adapter.newPythonClass(db,PrimitiveType.PrimitiveType)
        adapter.newPythonClass(db,LiteralCollection.LiteralListOfRepositoryObjects)

        repoId = adapter.newRepositoryObjectId(db,16,Repository.Repository,rid=1)
        moId = adapter.newRepositoryObjectId(db,11,Module.Module)
        typesId = adapter.newRepositoryObjectId(db,11,Module.Module)
        repoDefinesCid = adapter.newCollection(db,LiteralCollection.LiteralListOfRepositoryObjects,Constants.CollectionTypes.LIST,Constants.Types.OBJECT)



        #Create and write the types
        typeIds = []
        for pk in PrimitiveKind.PrimitiveKind.elements:
            tid = adapter.newRepositoryObjectId(db,14,PrimitiveType.PrimitiveType)
            adapter.addExtentEntry(db,['ftods_types'],tid)
            typeIds.append(tid)

            adapter.writeRepositoryObject(db,
                                          14,
                                          tid,
                                          PrimitiveType.PrimitiveType._tupleTypes,
                                          PrimitiveType.PrimitiveType._tupleNames,
                                          (tid,
                                           14,
                                           pk._n,
                                           'Created by Initialization of Repo',
                                           repoId,
                                           0,
                                           0,
                                           0,
                                           0,
                                           0,
                                           0,
                                           0,
                                           0,
                                           pk._v))


        adapter.writeRepositoryObject(db,
                                      11,
                                      moId,
                                      Module.Module._tupleTypes,
                                      Module.Module._tupleNames,
                                      (moId,
                                       11,
                                       'ODLMetaObjects',
                                       'Created by Initialization of Repo',
                                       repoId,
                                       0))

        adapter.writeRepositoryObject(db,
                                      11,
                                      typesId,
                                      Module.Module._tupleTypes,
                                      Module.Module._tupleNames,
                                      (typesId,
                                       11,
                                       'ODLTypes',
                                       'Created by Initialization of Repo',
                                       repoId,
                                       0))




        typeChanges = []
        ctr = 2
        for tid in typeIds:
            typeChanges.append((tid,ctr))
            ctr = ctr + 1
        changes =  [(Constants.CollectionChanges.APPEND,[(moId,0),(typesId,1)] + typeChanges)]
        adapter.writeCollection(db,repoDefinesCid,changes,Constants.CollectionTypes.LIST,Constants.Types.OBJECT)
        adapter.writeRepositoryObject(db,
                                      16,
                                      repoId,
                                      Repository.Repository._tupleTypes,
                                      Repository.Repository._tupleNames,
                                      (repoId,
                                       16,
                                       'Repository',
                                       'Created by Initialization of Repo',
                                       0,
                                       repoDefinesCid))



        adapter.commit(db)
        return

    def reinit(self,dbName,adapter):

        db = self.connect(dbName)
        adapter.begin(db)

        tableNames = db.getAllTableNames()
        for t in tableNames:
            if t not in self.tables:
                db.dropTable(t)


        for tableName in self.tables:
            for k in db[tableName].keys():
                del db[tableName][k]

        #Reset the sequences
        st = db['sequences']
        for s in self.sequences:
            st[s] = 2

        adapter.commit(db)

    def clear(self,dbName):

        db = self.connect(dbName)

        for tableName in self.tables:
            for k in db[tableName].keys():
                del db[tableName][k]


        self.close(db)
        del db



    def grant(self,connectString, username, accessLevel):
        # DbmDatabase has no concept of users (yet)
        return

    def clean(self,connString):
        pass





    sequences = ['objIds',
                 'repoIds',
                 'litIds',
                 'extentIds',
                 'classIds',
                 'collectionIds',
                 'blobIds',
                 ]


    tables = ['collections',
              'repos',
              'extents',
              'boundNames',
              'classes',
              'blobs',
              'sequences',
              'objects',
              ]
    
