########################################################################
#
# File Name: 	        PostgresManagement.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Drivers/PostgresManagement.py.html
#
"""
The Postgres Database Management functions
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought, Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import string,os

from Ft.Ods.StorageManager.Adapters import Util
from Ft.Ods.StorageManager.Adapters import Constants
import _pg

from Ft.Ods.StorageManager.Adapters import SQL

class DbManager(SQL.DbManager):

    def connect(self,connectString):
        #Split the connection string
        dbName,hostName,port,userName,passwd = Util.SplitDbConnectString(connectString)
        dbName = 'ftods__' + dbName
        return _pg.connect(dbName, host=hostName, port=port, user=userName, passwd=passwd)


    def create(self,connString,initRepo = 1):
        dbName,hostName,port,userName,passwd = Util.SplitDbConnectString(connString)
        tmpl_db = _pg.connect('template1',host=hostName, port=port, user=userName, passwd=passwd)
        tmpl_db.query('create database ftods__' + dbName)
        


    def exists(self,connString):
        dbName,hostName,port,userName,passwd = Util.SplitDbConnectString(connString)
        pg_db_name = "ftods__" + dbName
        tmpl_db = _pg.connect('template1',host=hostName, port=port, user=userName, passwd=passwd)
        rt = tmpl_db.query("SELECT datname FROM pg_database where datname='%s'" % pg_db_name).dictresult()
        return len(rt) != 0

    def destroy(self,dbName):
        dbName,hostName,port,userName,passwd = Util.SplitDbConnectString(dbName)
        dbName = 'ftods__' + dbName
        tmpl_db = _pg.connect('template1',hostName,port,userName,passwd)    
        tmpl_db.query('drop database ' + dbName)

    def close(self,conn):
        conn.close()
        

    def _createSequence(self,db,name):
        if name == 'ftods_repoid':
            start = " MINVALUE 2 START 2"
        else:
            start = ""
        db.query('CREATE SEQUENCE %s' % name  + start)
    def _resetSequence(self,db,name):
        db.query("SELECT SETVAL('%s',1)" % name)
        

    def _createTable(self,db,table_name,initData,inherits):
        st = "CREATE TABLE %s (" % table_name
        for data in initData:
            st = st + "%s %s %s," % (data[0],self.systemTypesToSql[data[1]],data[2])
        if len(initData):
            st = st[:-1] + ')'
        else:
            st = st + ')'

        if inherits:
            st = st + 'INHERITS ('
            for i in inherits:
                st = st + i
                if i != inherits[-1]:
                    st = st + ','
            st = st + ')'
            
        db.query(st)

    def _dropTable(self,db,tableName):
        db.query("DROP TABLE %s" % tableName)

    def _createIndex(self,db,index_name,columns,table_name):
        iString = 'CREATE INDEX %s  on %s (' % (index_name,table_name)
        for c in columns:
            iString = iString + c
            if c != columns[-1]:
                iString = iString + ','
        iString = iString + ')'
        db.query(iString)


    def _getAllGeneratedTables(self,db):
        table_names = []

        rt = db.query('SELECT _repoid,inherits,extender from ftods_class').getresult()

        allInterfaces = {}
        for rid,inherits,extender in rt:
            rid = int(rid)
            allInterfaces[rid] = []
            if inherits:
                res = db.query('SELECT value from ftods_long_collection where _cid=%s'%inherits).getresult()
                for r in res:
                    allInterfaces[rid].append(int(r[0]))
            if int(extender):
                allInterfaces[rid].append(int(extender))


        rt = db.query('SELECT _repoid,inherits from ftods_interface').getresult()
        for rid,inherits in rt:
            rid = int(rid)
            allInterfaces[rid] = []
            if inherits:
                res = db.query('SELECT value from ftods_long_collection where _cid=%s'%inherits).getresult()
                for r in res:
                    allInterfaces[rid].append(int(r[0]))

        tables = []
        for rid,derived in allInterfaces.items():
            curIndex = len(tables)
            for d in derived:
                if d in tables:
                    curIndex = tables.index(d)
            tables.insert(curIndex,rid)

        return map(lambda x:'interface__%s'%x,tables)


    def _deleteTable(self,db,tableName):
        db.query("DELETE from %s" % tableName)

    def _getAllExtentNames(self,db):
        return db.query('SELECT name,extentid from ftods_extents').getresult()

    def _deleteExtent(self,db,id):
        db.query("DELETE from ftods_extentmapping where extentid = '%s'"%id)
        db.query("DELETE from ftods_extents where extentid = '%s'"%id)

    def _dropAllBindings(self,db):
        db.query("DELETE from ftods_binding")
        
    def _deleteAllBlobs(self, db):
        boids = db.query('SELECT boid from ftods_blobs').getresult()
        for boid in boids:
            lo = db.getlo(boid)
            lo.unlink()
        db.query('DELETE from ftods_blobs')

    def _createInterfaceStorage(self,db,data):
        tableName,defs,bClasses = data 
        qstr = 'CREATE TABLE %s ( ' % tableName
        for (rid,type_, other) in defs:
            qstr = qstr + 'def__%i %s %s,' % (rid,type_,other)
        qstr = qstr[:-1] + ' ) INHERITS ( '
            
        for bc in bClasses:
            qstr = qstr + bc[0] + ','

        qstr = qstr[:-1] + ')'
        db.query(qstr)



    odmgToSqlTypes = {
        Constants.Types.BOOLEAN: 'BOOLEAN',
        Constants.Types.BOOLEAN_BAG: 'INT8',
        Constants.Types.BOOLEAN_SET: 'INT8',
        Constants.Types.BOOLEAN_LIST: 'INT8',
        Constants.Types.SIGNED_SHORT: 'INT4',
        Constants.Types.SIGNED_SHORT_BAG: 'INT8',
        Constants.Types.SIGNED_SHORT_LIST: 'INT8',
        Constants.Types.SIGNED_SHORT_SET: 'INT8',
        Constants.Types.STRING: 'text',
        Constants.Types.STRING_BAG: 'INT8',
        Constants.Types.STRING_SET: 'INT8',
        Constants.Types.STRING_LIST: 'INT8',
        Constants.Types.UNSIGNED_SHORT: 'INT4',
        Constants.Types.UNSIGNED_SHORT_BAG: 'INT8',
        Constants.Types.UNSIGNED_SHORT_SET: 'INT8',
        Constants.Types.UNSIGNED_SHORT_LIST: 'INT8',
        Constants.Types.SIGNED_LONG: 'INT8',
        Constants.Types.SIGNED_LONG_BAG: 'INT8',
        Constants.Types.SIGNED_LONG_SET: 'INT8',
        Constants.Types.SIGNED_LONG_LIST: 'INT8',
        Constants.Types.UNSIGNED_LONG: 'INT8',
        Constants.Types.UNSIGNED_LONG_BAG: 'INT8',
        Constants.Types.UNSIGNED_LONG_SET: 'INT8',
        Constants.Types.UNSIGNED_LONG_LIST: 'INT8',
        Constants.Types.SIGNED_LONG_LONG: 'NUMERIC(40,0)',
        Constants.Types.SIGNED_LONG_LONG_BAG: 'INT8',
        Constants.Types.SIGNED_LONG_LONG_SET: 'INT8',
        Constants.Types.SIGNED_LONG_LONG_LIST: 'INT8',
        Constants.Types.FLOAT: 'FLOAT4',
        Constants.Types.FLOAT_BAG: 'INT8',
        Constants.Types.FLOAT_SET: 'INT8',
        Constants.Types.FLOAT_LIST: 'INT8',
        Constants.Types.DOUBLE: 'FLOAT8',
        Constants.Types.DOUBLE_BAG: 'INT8',
        Constants.Types.DOUBLE_SET: 'INT8',
        Constants.Types.DOUBLE_LIST: 'INT8',
        Constants.Types.OBJECT: 'INT8',
        Constants.Types.OBJECT_BAG: 'INT8',
        Constants.Types.OBJECT_SET: 'INT8',
        Constants.Types.OBJECT_LIST: 'INT8',
        Constants.Types.ENUMERATION: 'INT4',
        Constants.Types.STRUCTURE: 'INT8',
        Constants.Types.DICTIONARY:'INT8',
        Constants.Types.BLOB:'INT8',
    }


    systemTypesToSql = {SQL.DbManager.SystemTypes.NAME:"VARCHAR(128)",
                        SQL.DbManager.SystemTypes.OBJECT:odmgToSqlTypes[Constants.Types.OBJECT],
                        SQL.DbManager.SystemTypes.ENUMERATION:odmgToSqlTypes[Constants.Types.ENUMERATION],
                        SQL.DbManager.SystemTypes.STRING:odmgToSqlTypes[Constants.Types.STRING],
                        SQL.DbManager.SystemTypes.BOOLEAN:odmgToSqlTypes[Constants.Types.BOOLEAN],
                        SQL.DbManager.SystemTypes.SHORT:odmgToSqlTypes[Constants.Types.UNSIGNED_SHORT],
                        SQL.DbManager.SystemTypes.STRING:odmgToSqlTypes[Constants.Types.STRING],
                        SQL.DbManager.SystemTypes.DOUBLE:odmgToSqlTypes[Constants.Types.DOUBLE],
                        SQL.DbManager.SystemTypes.TIMESTAMP:odmgToSqlTypes[Constants.Types.DOUBLE],
                        SQL.DbManager.SystemTypes.ANY:odmgToSqlTypes[Constants.Types.STRING],
                        }

    tables = SQL.DbManager.tables.copy()
    tables['ftods_object'] = {SQL.DbManager.TableData.TABLE_INIT:[('dummy',SQL.DbManager.SystemTypes.SHORT,''),
                                                                             ],
                              SQL.DbManager.TableData.TABLE_ACCESS:SQL.DbManager.AccessTypes.OBJECT_ACCESS,
                              SQL.DbManager.TableData.TABLE_INHERITS:[]
                              }
    

    tables['ftods_blobs'] = {SQL.DbManager.TableData.TABLE_INIT:[('bid',SQL.DbManager.SystemTypes.OBJECT,'PRIMARY KEY'),
                                                                 ('boid',SQL.DbManager.SystemTypes.OBJECT,''),
                                                                 ],
                              SQL.DbManager.TableData.TABLE_ACCESS:SQL.DbManager.AccessTypes.OBJECT_ACCESS,
                              SQL.DbManager.TableData.TABLE_INHERITS:[]
                              }
    SQL.Manager.g_sortedTableNames.append('ftods_blobs')


