########################################################################
#
# File Name:            Memory.py
#
# Documentation:        http://docs.4suite.com/4Rdf/Drivers/Memory.py.html
#
"""
Non-persistent RDF model back-end
WWW: http://4suite.com/4RDF         e-mail: support@4suite.com

Copyright (c) 1999,2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""


import string, re

from Ft.Rdf.Drivers import DataBaseExceptions
from Ft.Rdf.Statement import Statement


def CreateDb(dbName):
    return DbAdapter(dbName)

#Identical function for Memory driver
GetDb = CreateDb

def DestroyDb(dbName):
    pass


class DbAdapter:
    def __init__(self, name):
        self._db = []
        self._bound = {}

    def begin(self):
        return None

    def commit(self):
        pass

    def rollback(self):
        pass

    ### Operations

    def add(self, stmts):
        for (subject, predicate, object, uri) in stmts:
            self._db.append((subject, predicate, object, uri))
        return

    def remove(self, stmts):
        for (subject, predicate, object, uri) in stmts:
            try:
                index = self._db.index((subject, predicate, object, uri))
                del self._db[index]
            except:
                pass

    def removeAll(self, uri=None):
        if not uri:
            self._db = []
        else:
            self._db = filter(lambda x, u=uri: x[3] != u, self._db)

    ### Queries

    def complete(self, subject, predicate, object):
        if subject and predicate and object:
            retval = filter(lambda x, s=subject, p=predicate, o=object:
                            x[0] == s and x[1] == p and x[2] == o,
                            self._db)
        elif subject and predicate:
            retval = filter(lambda x, s=subject, p=predicate:
                            x[0] == s and x[1] == p,
                            self._db)
        elif subject and object:
            retval = filter(lambda x, s=subject, o=object:
                            x[0] == s and x[2] == o,
                            self._db)
        elif predicate and object:
            retval = filter(lambda x, p=predicate, o=object:
                            x[1] == p and x[2] == o,
                            self._db)
        elif subject:
            retval = filter(lambda x, s=subject:
                            x[0] == s,
                            self._db)
        elif predicate:
            retval = filter(lambda x, p=predicate:
                            x[1] == p,
                            self._db)
        elif object:
            retval = filter(lambda x, o=object:
                            x[2] == o,
                            self._db)
        else:
            retval = self._db
        return retval

    def completeRegex(self, subject, predicate, object,ignoreCase):
        sReg = None
        pReg = None
        oReg = None
        flags = 0
        if ignoreCase:
            flags = re.IGNORECASE
        if subject:
            sReg = re.compile(subject,flags)
        if predicate:
            pReg = re.compile(predicate,flags)
        if object:
            oReg = re.compile(object,flags)


        if sReg and pReg and oReg:
            retval = filter(lambda x, s=sReg, p=pReg, o=oReg:
                            s.match(x[0]) and p.match(x[1]) and o.match(x[2]),
                            self._db)
        elif sReg and pReg:
            retval = filter(lambda x, s=sReg, p=pReg:
                            s.match(x[0]) and p.match(x[1]),
                            self._db)
        elif sReg and oReg:
            retval = filter(lambda x, s=sReg, o=oReg:
                            s.match(x[0]) and o.match(x[2]),
                            self._db)
        elif pReg and oReg:
            retval = filter(lambda x, p=pReg, o=oReg:
                            p.match(x[1]) and o.match(x[2]),
                            self._db)
        elif sReg:
            retval = filter(lambda x, s=sReg:
                            s.match(x[0]),
                            self._db)
        elif pReg:
            retval = filter(lambda x, p=pReg:
                            p.match(x[0]),
                            self._db)
        elif oReg:
            retval = filter(lambda x, o=oReg:
                            o.match(x[0]),
                            self._db)
        else:
            retval = self._db
        return retval

    def exclude(self, subject, predicate, object):
        """
        The most basic form of inference.  Any combination of s, p and o
        can be None, and all None items are treated as a wildcard that
        matches any item in the model.  A list is returned comprising
        all the tuples that match the exclusion arguments.
        Warning: this will typically be a large subset of the model
        """
        if subject and predicate and object:
            retval = filter(lambda x, s=subject, p=predicate, o=object:
                            x[0] != s and x[1] != p and x[2] != o,
                            self._db)
        elif subject and predicate:
            retval = filter(lambda x, s=subject, p=predicate:
                            x[0] != s and x[1] != p,
                            self._db)
        elif subject and object:
            retval = filter(lambda x, s=subject, o=object:
                            x[0] != s and x[2] != o,
                            self._db)
        elif predicate and object:
            retval = filter(lambda x, p=predicate, o=object:
                            x[1] != p and x[2] != o,
                            self._db)
        elif subject:
            retval = filter(lambda x, s=subject:
                            x[0] != s,
                            self._db)
        elif predicate:
            retval = filter(lambda x, p=predicate:
                            x[1] != p,
                            self._db)
        elif object:
            retval = filter(lambda x, o=object:
                            x[2] != o,
                            self._db)
        else:
            retval = self._db
        return retval

    def size(self):
        return len(self._db)

    def contains(self, subject, predicate, object, uri):
        try:
            index = self._db.index((subject, predicate, object, uri))
            return 1
        except:
            pass
        return 0

    def bind(self, object, name):
        self._bound[name] = object

    def unbind(self, name):
        del self._bound[name]

    def lookup(self, name):
        return self._bound.get(name,None)

    def has_key(self,name):
        return self._bound.has_hey(name)
