########################################################################
#
# File Name:            Postgres.py
#
# Documentation:        http://docs.4suite.com/4RDF/Drivers/Postgres.py.html
#
"""
Interface to Postgres for 4RDF
WWW: http://4suite.com/RDF        e-mail: support@4suite.com

Copyright (c) 2000 FourThought, Inc., USA.  All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import sys, string, cPickle, cStringIO
from Ft.Lib.PgDatabase import Database, EscapeQuotes
from Ft.Rdf.Drivers import DataBaseExceptions
from Ft.Rdf import Statement


CREATE_TABLE = """
CREATE %s TABLE _ftrdf_%s
(
    subject   text,
    predicate text,
    object    text,
    uri       text
)
"""
STATEMENT_TABLE = CREATE_TABLE % ('', 'statement')
TEMPORARY_TABLE = CREATE_TABLE % ('TEMPORARY', 'temporary')

CREATE_INDEX = """
CREATE INDEX statement_index ON _ftrdf_statement
(
    subject,
    predicate,
    object
)
"""

DROP_INDEX = "DROP INDEX statement_index"

INSERT_CMD = """
INSERT INTO _ftrdf_statement (
    subject,
    predicate,
    object,
    uri
) VALUES ('%s', '%s', '%s', '%s')"""

DELETE_CMD = """
DELETE FROM _ftrdf_statement
    WHERE subject='%s'
    AND predicate='%s'
    AND object='%s'
    AND uri='%s'
"""

DELETE_ALL_CMD = "DELETE FROM _ftrdf_statement"

# Emacs "
def GetDb(dbName):
    return DbAdapter(dbName)


def CreateDb(dbName):
    pg_db_name = "ftrdf__" + dbName
    #tmpl_db = Database('template1')
    #tmpl_db.query('create database ' + pg_db_name)
    #del tmpl_db
    #FIXME: Why does the above not work all of a sudden?
    import os; os.system('createdb ' + pg_db_name)
    pg_db = Database(pg_db_name)
    pg_db.query(STATEMENT_TABLE)
    pg_db.query(CREATE_INDEX)
    pg_db.query("""
                CREATE TABLE _ftrdf_bound
                (
                    name varchar primary key,
                    object varchar
                )
                """)
    return DbAdapter(dbName)


COMPLETE_BASE = "SELECT subject, predicate, object, uri FROM _ftrdf_statement"
COMPLETE_ALL = COMPLETE_BASE
COMPLETE_SUBJECT = COMPLETE_BASE + " where subject='%s'"
COMPLETE_PREDICATE = COMPLETE_BASE + " where predicate='%s'"
COMPLETE_OBJECT = COMPLETE_BASE + " where object='%s'"
COMPLETE_SUBJECT_PREDICATE = COMPLETE_BASE + " where subject='%s' and predicate='%s'"
COMPLETE_SUBJECT_OBJECT = COMPLETE_BASE + " where subject='%s' and object='%s'"
COMPLETE_PREDICATE_OBJECT = COMPLETE_BASE + " where predicate='%s' and object='%s'"
COMPLETE_SUBJECT_PREDICATE_OBJECT = COMPLETE_BASE + " where subject='%s' and predicate='%s' and object='%s'"


COMPLETE_SUBJECT_REGEX = COMPLETE_BASE + " where subject%s'%s'"
COMPLETE_PREDICATE_REGEX = COMPLETE_BASE + " where predicate%s'%s'"
COMPLETE_OBJECT_REGEX = COMPLETE_BASE + " where object%s'%s'"
COMPLETE_SUBJECT_PREDICATE_REGEX = COMPLETE_BASE + " where subject%s'%s' and predicate%s'%s'"
COMPLETE_SUBJECT_OBJECT_REGEX = COMPLETE_BASE + " where subject%s'%s' and object%s'%s'"
COMPLETE_PREDICATE_OBJECT_REGEX = COMPLETE_BASE + " where predicate%s'%s' and object%s'%s'"
COMPLETE_SUBJECT_PREDICATE_OBJECT_REGEX = COMPLETE_BASE + " where subject%s'%s' and predicate%s'%s' and object%s'%s'"


def DestroyDb(dbName):
    pg_db_name = "ftrdf__" + dbName

    import pg
    db=pg.connect('template1')
    db.query('DROP DATABASE %s' % pg_db_name)
    #try:
    #    db.query('DROP DATABASE %s' % pg_db_name)
    #except pg.error:
    #    pass


def ExistsDb(dbName):
    pg_db_name = "ftrdf__" + dbName
    db=Database('template1')
    rt = db.query("SELECT datname FROM pg_database where datname='%s'" % pg_db_name)
    return len(rt) != 0

class DbAdapter:
    def __init__(self,name):
        self._name = 'ftrdf__' + name
        self._connection = None
        self._bindings = {}
        return

    def begin(self):
        self._connection = Database(self._name)
        self._connection.execute('BEGIN')
        #self._connection.execute(TEMPORARY_TABLE)
        return

    def commit(self):
        #self._connection.execute(DROP_INDEX)
        #self._connection.execute("INSERT INTO _ftrdf_statement SELECT * FROM _ftrdf_temporary")
        #self._connection.execute(CREATE_INDEX)
        self._connection.execute('COMMIT')
        self._connection = None
        #self._tempCreated = 0
        return

    def rollback(self):
        self._connection = None
        self._tempCreated = 0
        return

    def add(self, statements):
        # Takes a list of tuples [(s, p, o, u), ...]
        if not self._connection:
            raise DataBaseExceptions.NoTransaction

        for stmt in statements:
            self._connection.execute(INSERT_CMD % self._escape(stmt))
        
    def addStatement(self, subject, predicate, object, uri):
        if not self._connection:
            raise DataBaseExceptions.NoTransaction
        s = EscapeQuotes(subject)
        p = EscapeQuotes(predicate)
        o = EscapeQuotes(object)
        u = EscapeQuotes(uri)
        self._connection.execute("INSERT INTO _ftrdf_statement (subject, predicate, object, uri) VALUES ('%s', '%s', '%s', '%s')" % (s, p, o, u))
        return

    def remove(self, statements):
        if not self._connection:
            raise DataBaseException.Notransaction
        #self._connection.execute(DROP_INDEX)
        try:
            for stmt in statements:
                self._connection.execute(DELETE_CMD % self._escape(stmt))
        finally:
            #self._connection.execute(CREATE_INDEX)
            pass

    def removeAll(self, uri=None):
        if not self._connection:
            raise DataBaseException.Notransaction
        #self._connection.execute(DROP_INDEX)
        try:
            delete_cmd = DELETE_ALL_CMD
            if uri:
                delete_cmd = delete_cmd + " WHERE uri='" + uri + "'"
            self._connection.execute(delete_cmd)
        finally:
            #self._connection.execute(CREATE_INDEX)
            pass

    def removeStatement(self, subject, predicate, object, uri):
        if not self._connection:
            raise DataBaseExceptions.NoTransaction
        s = EscapeQuotes(subject)
        p = EscapeQuotes(predicate)
        o = EscapeQuotes(object)
        u = EscapeQuotes(uri)
        self._connection.execute("DELETE FROM _ftrdf_statement WHERE subject = '%s' AND predicate = '%s' AND object = '%s' AND uri = '%s'" % (s, p, o, u))
        return

    def complete(self, subject, predicate, object):
        s = EscapeQuotes(subject)
        p = EscapeQuotes(predicate)
        o = EscapeQuotes(object)
        if not self._connection:
            raise DataBaseExceptions.NoTransaction
        if s:
            if p:
                if o:
                    cmd = COMPLETE_SUBJECT_PREDICATE_OBJECT % (s,p,o)
                else:
                    cmd = COMPLETE_SUBJECT_PREDICATE % (s,p)
            elif o:
                cmd = COMPLETE_SUBJECT_OBJECT % (s,o)
            else:
                cmd = COMPLETE_SUBJECT % s
        elif p:
            if o:
                cmd = COMPLETE_PREDICATE_OBJECT % (p,o)
            else:
                cmd = COMPLETE_PREDICATE % p
        elif o:
            cmd = COMPLETE_OBJECT % o
        else:
            cmd = COMPLETE_ALL

        result_set = self._connection.queryAllTuple(cmd)
        triples = map(lambda x: (x[0], x[1], x[2], x[3]),result_set)
        return triples

    def completeRegex(self, subject, predicate, object,ignoreCase):
        s = EscapeQuotes(subject)
        p = EscapeQuotes(predicate)
        o = EscapeQuotes(object)

        c = '~'
        if ignoreCase:
            c = '~*'

        if not self._connection:
            raise DataBaseExceptions.NoTransaction
        if s:
            if p:
                if o:
                    cmd = COMPLETE_SUBJECT_PREDICATE_OBJECT_REGEX % (c,s,c,p,c,o)
                else:
                    cmd = COMPLETE_SUBJECT_PREDICATE_REGEX % (c,s,c,p)
            elif o:
                cmd = COMPLETE_SUBJECT_OBJECT_REGEX % (c,s,c,o)
            else:
                cmd = COMPLETE_SUBJECT_REGEX % (c,s)
        elif p:
            if o:
                cmd = COMPLETE_PREDICATE_OBJECT_REGEX % (c,p,c,o)
            else:
                cmd = COMPLETE_PREDICATE_REGEX % (c,p)
        elif o:
            cmd = COMPLETE_OBJECT_REGEX % (c,o)
        else:
            cmd = COMPLETE_ALL

        result_set = self._connection.queryAllTuple(cmd)
        triples = map(lambda x: (x[0], x[1], x[2], x[3]),result_set)
        return triples

    def size(self):
        if not self._connection:
            raise DataBaseExceptions.NoTransaction
        result_set = self._connection.queryAllTuple("SELECT COUNT(subject) from _ftrdf_statement")
        return result_set[0][0]

    def contains(self, subject, predicate, object, uri):
        #FIXME We can do this faster with a optimized SQL query
        return len(self.complete(subject, predicate, object)) != 0

    def bind(self, object, name):
        if not self._connection:
            raise DataBaseExceptions.NoTransaction
        safeObject = self._makeSafe(object)
        self._connection.execute(
            "INSERT INTO _ftrdf_bound (name, object) VALUES ('%s', '%s')" % (
                name,
                safeObject
            )
        )
        self._bindings[name] = object

    def unbind(self, name):
        if not self._connection:
            raise DataBaseExceptions.NoTransaction
        self._connection.execute("DELETE FROM _ftrdf_bound WHERE name = '%s'" % name)
        if self._bindings.has_key(name):
            del self._bindings[name]

    def lookup(self, name):
        if self._bindings.has_key(name):
            return self._bindings[name]
        if not self._connection:
            raise DataBaseExceptions.NoTransaction
        results = self._connection.queryAllTuple(
            "SELECT object FROM _ftrdf_bound WHERE name = '%s'" % name
        )
        if not results:
            return None
        return self._restoreSafe(results[0][0])

    def keys(self):
        rt = self._connection.queryAllTuple("SELECT name from _ftrdf_bound")
        return map(lambda x:x[0], rt)

    def has_key(self,name):
        rt = self._connection.queryAllTuple("SELECT object from _ftrdf_bound where name = '%s'" % name)
        return len(rt) != 0

    def _makeSafe(self, object):
        #return EscapeQuotes(object.expression)
        stream = cStringIO.StringIO()
        cPickle.Pickler(stream).dump(object)
        safeString = EscapeQuotes(stream.getvalue())
        return safeString

    def _restoreSafe(self, pickled):
        #from Ft.Rdf.Parsers.Ril import RilParserImp
        #p = RilParserImp.RilParserImp()
        #return p.parse(pickled)
        stream = cStringIO.StringIO(pickled)
        object = cPickle.Unpickler(stream).load()
        return object

    def _escape(self, (subject, predicate, object, uri)):
        s = EscapeQuotes(subject)
        p = EscapeQuotes(predicate)
        o = EscapeQuotes(object)
        u = EscapeQuotes(uri)
        return (s, p, o, u)
