########################################################################
#
# File Name:            Shelve.py
#
# Documentation:        http://docs.4suite.com/4Rdf/Drivers/Shelve.py.html
#
"""
A simple persistent back end
WWW: http://4suite.com/4RDF         e-mail: support@4suite.com

Copyright (c) 1999,2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""


import string, re, os

from Ft.Rdf.Drivers import DataBaseExceptions
from Ft.Rdf.Drivers import Memory
from Ft.Rdf.Statement import Statement
import shelve
from Ft.Lib import DbmDatabase


DATABASE_DIR = os.environ.get('FT_DATABASE_DIR', None)

if DATABASE_DIR is None:    
    raise DbmDatabase.DbmError('BOOTSTRAP', 'Please define FT_DATABASE_DIR in the environment')

def ExistsDb(dbName):
    dbName = _FullName(dbName)
    return os.path.exists(dbName)

def CreateDb(dbName):
    dbName = _FullName(dbName)
    return DbAdapter(dbName)

#Identical function in this driver
GetDb = CreateDb

def DestroyDb(dbName):
    dbName = _FullName(dbName)
    if os.path.exists(dbName):
        os.unlink(dbName)


def _FullName(dbName):
    return os.path.join(DATABASE_DIR,dbName)

class DbAdapter(Memory.DbAdapter):
    def __init__(self, name):
        name = _FullName(name)
        self._shelve = shelve.open(name)

    def begin(self):
        if self._shelve.has_key('triples'):
            self._db = self._shelve['triples']
        else:
            self._db = []
        if self._shelve.has_key('bindings'):
            self._bound = self._shelve['bindings']
        else:
            self._bound = {}

    def commit(self):
        self._shelve['triples'] = self._db
        self._shelve['bindings'] = self._bound
        self._shelve.close()

    def rollback(self):
        self._shelve.close()

