########################################################################
#
# File Name:            Processor.py
#
# Documentation:        http://docs.4suite.com/4XLink/Processor.py.html
#
"""
Implement the XLink processor engine
WWW: http://4suite.com/4Link        e-mail: support@4suite.com

Copyright (c) 2000 Fourthought, Inc., USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import string, os
import cStringIO
from Ft.Lib import pDomlette
from xml import xpath
from Ft.XLink import XLINK_NAMESPACE
from Ft.XLink import XLinkElements

class Processor:

    def runString(self, source, reader=None):
        if reader == None:
            reader = pDomlette.PyExpatReader()
        st = cStringIO.StringIO(source)
        doc = reader.fromStream(st)
        return self._run(doc, reader)

    def runUri(self, uri, reader=None):
        if reader == None:
            reader = pDomlette.PyExpatReader()
        doc = reader.fromUri(uri)
        return self._run(doc, reader)

    def _run(self, node, reader=None):
        """
        Warning: The document will be modified in place
        The reader class MUST be able to support reading from href values.
        (i.e., URIs, but could be anything).
        """
        reader = reader or DomReader.UriReader()
        context = xpath.CreateContext(node.ownerDocument)
        context.processorNss.update({'xlink':XLINK_NAMESPACE})
        xlinks = xpath.Evaluate('/descendant-or-self::*[@xlink:type]', context=context)
        for link in xlinks:
            xlink = XLinkElements.Create(link)
            xlink.process(reader)
        return node.ownerDocument
