########################################################################
#
# File Name:   ParsedXPointer.py
#
# Docs:        http://docs.4suite.com/XPointer/ParsedXPointer.py.html
#
"""
A Parsed Token that represents a list of XPointers
WWW: http://4suite.com/XPointer        e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import types, string
from Ft.XPointer import XPtrContext, XPtrException
from xml.dom.ext import GetAllNs
from xml.xpath import ParsedToken

class ParsedXPointer(ParsedToken.ParsedToken):
    def __init__(self, xptrs=None):
        ParsedToken.ParsedToken.__init__(self, 'XPOINTER')
        if xptrs is None:
            self._xpointers = []
            self._length = 0
        else:
            self._xpointers = list(xptrs)
            self._length = len(xptrs)

    def append(self,xptr):
        self._xpointers.append(xptr)
        self._length = self._length + 1

    def select(self, document, contextNode, nss = None):
        nss = nss or {}
        
        nss.update(GetAllNs(document))
        context = XPtrContext.XPtrContext(document, 1, 1, contextNode, nss)
        for xptr in self._xpointers:
            set = xptr.evaluate(context)
            if len(set) >= 1:
                if len(set) == 1:
                    return set[0]
                raise XPtrException(XPtrException.SUB_RESOURCE_ERROR)
            

    def __getitem__(self, index):
        return self._xpointers[index]

    def __len__(self):
        return self._length

    def pprint(self, indent=''):
        print indent + str(self)
        for xptr in self._xpointers:
            xptr.pprint(indent + '  ')

    def __str__(self):
        result = '<XPointer at %x' % id(self)
        if self._xpointers:
            result = result + ': ' + repr(self)
        return result + '>'

    def __repr__(self):
        return reduce(lambda result, xptr:
                      result + ' ' + repr(xptr),
                      self._xpointers,
                      ''
                      )


class ParsedXPtrExpr(ParsedToken.ParsedToken):
    def __init__(self, expr):
        ParsedToken.ParsedToken.__init__(self, 'XPTREXPR')
        self.expr = expr


    def evaluate(self, context):
        return self.expr.evaluate(context)
            

    def pprint(self, indent=''):
        print indent + str(self)
        self.expr.pprint(indent+'  ')

    def __str__(self):
        return '<XPtrExpr at %x %s>' % (id(self),repr(self.expr))

    def __repr__(self):
        return 'xpointer(%s)' % repr(self.expr)


class ParsedXmlns(ParsedToken.ParsedToken):
    def __init__(self, decl):
        ParsedToken.ParsedToken.__init__(self, 'XMLNS')

        decl = decl[6:-1]
        fields = string.split(decl,'=')
        self.prefix = string.strip(fields[0])
        self.nss = string.lstrip(fields[1])


    def evaluate(self, context):
        if self.prefix != 'xml':
            context.processorNss[self.prefix] = self.nss
        return []
            

    def pprint(self, indent=''):
        print indent + str(self)

    def __str__(self):
        return '<XMLNS at %x %s=%s>' % (id(self),self.prefix,self.nss)


    def __repr__(self):
        return 'xmlns(%s=%s)' % (self.prefix,self.nss)
