########################################################################
#
# File Name:   ParsedPredicateList.py
#
# Docs:        http://docs.4suite.com/XPATH/ParsedPredicateList.py.html
#
"""
A Parsed Token that represents a predicate list.
WWW: http://4suite.com/XPATH        e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from xml.xpath import Conversions, ParsedToken
from xml.xpath.Boolean import BooleanEvaluate


class ParsedPredicateList(ParsedToken.ParsedToken):
    def __init__(self, preds):
        ParsedToken.ParsedToken.__init__(self, 'PREDICATE_LIST')
        if type(preds) == type(()):
            preds = list(preds)
        elif not type(preds) == type([]):
            raise "Invalid Predicates: ",str(preds)

        self._predicates = preds
        self._length = len(preds)

    def append(self,pred):
        self._predicates.append(pred)
        self._length = self._length + 1

    def filter(self, nodeList, context, reverse):
        if self._length:
            state = context.copyNodePosSize()
            for pred in self._predicates:
                l = len(nodeList)
                ctr = 0
                current = nodeList
                nodeList = []
                for node in current:
                    context.setNodePosSize((node,(reverse and l - ctr) or (ctr + 1),l))
                    if BooleanEvaluate(pred, context):
                        nodeList.append(node)
                    ctr = ctr + 1
            context.setNodePosSize(state)
        return nodeList

    def __getitem__(self, index):
        return self._predicates[index]

    def __len__(self):
        return self._length

    def pprint(self, indent=''):
        print indent + str(self)
        for pred in self._predicates:
            pred.pprint(indent + '  ')

    def __str__(self):
        return '<PredicateList at %x: %s>' % (
            id(self),
            repr(self) or '(empty)',
            )

    def __repr__(self):
        return reduce(lambda result, pred:
                      result + '[%s]' % repr(pred),
                      self._predicates,
                      ''
                      )
