########################################################################
#
# File Name:            ApplyImportsElement.py
#
# Documentation:        http://docs.4suite.com/4XSLT/ApplyImportsElement.py.html
#
"""
Implementation of the XSLT Spec apply-imports stylesheet element.
WWW: http://4suite.com/4XSLT        e-mail: support@4suite.com

Copyright (c) 1999-2000 FourThought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import xml.dom.Element
import xml.dom.ext
import xml.xslt
from xml.xslt import XsltElement, XSL_NAMESPACE, XsltException, Error, XPathParser
from xml.xpath import Util, g_xpathRecognizedNodes
from xml.xslt.FtElements import FT_EXT_NAMESPACE

class ApplyImportsElement(XsltElement):
    legalAttrs = ['select', 'mode']

    def __init__(self, doc, uri=xml.xslt.XSL_NAMESPACE,
                 localName='apply-imports', prefix='xsl', baseUri=''):

        XsltElement.__init__(self, doc, uri, localName, prefix, baseUri)

    def setup(self):
        self.__dict__['_nss'] = xml.dom.ext.GetAllNs(self)
        mode_attr = self.getAttributeNS('', 'mode')
        if mode_attr == '':
            self.__dict__['_mode'] = mode_attr
        else:
            split_name = Util.ExpandQName(
                mode_attr,
                namespaces=self._nss
                )
            self.__dict__['_mode'] = split_name
        if not self._mode:
            self.__dict__['_mode'] = None

        self.__dict__['_select'] = set.getAttributeNS('','select')
        if self._select:
            parser = XPathParser.XPathParser()
            self.__dict__['_expr'] = parser.parseExpression(self._select)
        else:
            self.__dict__['_expr'] = None

        self.__dict__['_sortSpecs'] = []
        for child in self.childNodes:
            #All children should be sort and with-param
            if child.namespaceURI == XSL_NAMESPACE:
                if child.localName == 'sort':
                    self._sortSpecs.append(child)
                elif child.localName != 'with-param':
                    raise XsltException(Error.ILLEGAL_APPLYTEMPLATE_CHILD)
        return

    def instantiate(self, context, processor):

        origState = context.copy()
        context.setNamespaces(self._nss)

        params = {}
        mode = self._mode
        for child in self.childNodes:
            if (child.namespaceURI, child.localName) == (XSL_NAMESPACE, 'with-param'):
                param = child.instantiate(context, processor)[1]
                params[param[0]] = param[1]
            elif (child.namespaceURI, child.localName) == (FT_EXT_NAMESPACE, 'set-mode'):
                mode = child.instantiate(context, processor)[1]
            else:
                context = child.instantiate(context, processor)[0]
        if self._select:
            node_set = self._expr.evaluate(context)
        else:
            node_set = context.node.childNodes
        node_set = filter(lambda x: x.nodeType in g_xpathRecognizedNodes,
                          node_set)
        size = len(node_set)
        if size > 1 and self._sortSpecs:
            node_set = self._sortSpecs[0].instantiate(context, processor,
                                                      node_set,
                                                      self._sortSpecs[1:])[1]
        for ctr in range(size):
            node = node_set[ctr]
            if mode:
                context.setNodePosSizeMode((node,ctr+1,size,mode))
            else:
                context.setNodePosSize((node,ctr+1,size))
            processor.applyImports(context, mode, params)

        context.set(origState)
        return (context,)

    def __getinitargs__(self):
        return (None, self.namespaceURI, self.localName, self.prefix,
                self.baseUri)

    def __getstate__(self):
         base_state = XsltElement.__getstate__(self)
         new_state = (base_state, self._nss, self._mode, self._sortSpecs, self._select, self._expr)
         return new_state

    def __setstate__(self, state):
        XsltElement.__setstate__(self, state[0])
        self._nss = state[1]
        self._mode = state[2]
        self._sortSpecs = state[3]
        self._select = state[4]
        self._expr = state[5]
        return


