########################################################################
#
# File Name:            ParsedIdKeyPattern.py.py
# 
# Documentation:        http://docs.4suite.com/4XSLT/ParsedIdKeyPattern.py.py.html
# 
"""
Parse class to handle XSLT ID Key patterns
WWW: http://4suite.com/4XSLT        e-mail: support@4suite.com

Copyright (c) 1999-2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from xml.xpath import ParsedToken
from xml.xslt import XsltException, Error, XPattern

import types
try:
    g_stringTypes= [types.StringType, types.UnicodeType]
except:
    g_stringTypes= [types.StringType]

g_operandMap = {XPattern.ID:'id',
                XPattern.KEY:'key'
                }

class ParsedIdKeyPattern(ParsedToken.ParsedToken):
    def __init__(self, op, st1, st2):
        ParsedToken.ParsedToken.__init__(self,'ID_KEY_PATTERN')
        #FIXME: Are these necessary with BisonGen???
        if type(op) != type(0):
            raise XsltException(Error.INVALID_OPERAND_IN_PATTERN, str(op))
        if type(st1) not in g_stringTypes:
            raise XsltException(Error.INVALID_OPERAND_IN_PATTERN, str(st1))
        if type(st2) not in g_stringTypes and st2 != None:
            raise XsltException(Error.INVALID_OPERAND_IN_PATTERN, str(st2))
        if not op in [XPattern.ID,XPattern.KEY]:
            raise XsltException(Error.INVALID_OPERAND_IN_PATTERN, str(op))
        if op == XPattern.ID and st1 == None or st2 != None:
            raise XsltException(Error.INVALID_OPERAND_IN_PATTERN, str(op))
        if op == XPattern.KEY and st1 != None or st2 != None:
            raise XsltException(Error.INVALID_OPERAND_IN_PATTERN, str(op))
        self._op = op
        self._st1 = st1
        self._st2 = st2

    def match(self,context):
        if self._op == XPattern.ID:
            #FIXME: Find the nodes with the given id
            return []
        if self._op == XPattern.KEY:
            #FIXME: Find any key matches
            return []
        return []

    def dump(self,f):
        f.write(g_operandMap[self._op])
        f.write('(')
        f.write(self._st1)
        if self._st2:
            f.write(',')
            f.write(self._st2)
        f.write(')')

