import cStringIO
from xml.dom import DOMException
from xml.dom import NAMESPACE_ERR
from xml.dom.ext import Print, PrettyPrint
from xml.dom.ext.reader import Sax, Sax2
from xml.sax import sax2exts, saxexts


def GetExceptionName(code):
    import types
    from xml import dom
    for (name,value) in vars(dom).items():
        if (type(value) == types.IntType and value == code):
            return name


source_1 = """<?xml version="1.0" encoding="iso-8859-1"?>
<Xafp type="multimedia" uno="afp_wbs_doc_010110105314.g5kw25ak">
  <Head>
    <Name>GB-OTAN-sant</Name>
    <DateReleased>20010110T105314Z</DateReleased>
    <Source>AFP</Source>
  </Head>
  <NewsLines>
    <HeadLine>La polmique loin d'tre apaise par l'annonce de tests 
Londres</HeadLine>
    <DateLine>LONDRES</DateLine>
  </NewsLines>
</Xafp>
"""


expected_1 = """La pol\351mique loin d'\352tre apais\351e par l'annonce de tests \340\012Londres"""


expected_2 = """"""


def Test(tester):
    tester.startGroup("Olivier Deckmyn's <odeckmyn@teaser.fr> Q about encodings.")

    tester.startTest('DOM from SAX')
    p = saxexts.make_parser("xml.sax.drivers.drv_xmlproc")
    reader = Sax.Reader(parser=p)
    doc = reader.fromString(source_1)
    nodes = doc.getElementsByTagName('HeadLine')
    result = nodes[0].firstChild.nodeValue

    if result != expected_1:
        tester.error('Expected\n"""%s"""\ngot\n"""%s"""'%(repr(expected_1), repr(result)))
    tester.testDone()

    tester.startTest('DOM from SAX2')
    p = sax2exts.make_parser("xml.sax.drivers2.drv_xmlproc")
    reader = Sax2.Reader(parser=p)
    doc = reader.fromString(source_1)
    nodes = doc.getElementsByTagName('HeadLine')
    result = nodes[0].firstChild.nodeValue

    if result != expected_1:
        tester.error('Expected\n"""%s"""\ngot\n"""%s"""'%(repr(expected_1), repr(result)))
    tester.testDone()

    return tester.groupDone()


if __name__ == '__main__':
    import sys
    sys.path.append('.')
    import TestSuite

    tester = TestSuite.TestSuite()
    retVal = Test(tester)
    sys.exit(retVal)

