
import os
DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

class LiteralFactory:
    def __init__(self):
        self.ctr = -1
    def next(self):
        self.ctr = self.ctr + 1
        return (self.ctr,"TEXT%d"%self.ctr)

    def reset(self):
        self.ctr = -1

class RepoFactory:
    def __init__(self):
        self.objNames = ['pk_boolean','pk_long','pk_short','pk_string','pk_ulong','pk_ushort']
        self.ctr = 0
    def next(self):
        from Ft.Ods import Database
        db = Database.Database()
        name = self.objNames[self.ctr]
        obj = db.schema().resolve(name)
        self.ctr = self.ctr + 1
        return name,obj

    def nextId(self):
        return self.next()._4ods_getId()

    def reset(self):
        self.ctr = 0

from Ft.Ods.Collections import DictionaryOfPersistentObjects
from Ft.Ods import Collections

def test_dictionary(d):

    if isinstance(d,DictionaryOfPersistentObjects.DictionaryOfPersistentObjects):
        curFac = RepoFactory()
    else:
        curFac = LiteralFactory()

    key,val = curFac.next()

    print "**test Bind**"
    d.bind(key,val)

    try:
        d.bind(key,val)
    except:
        pass
    else:
        print "No exception thrown"

    print "**Bind works**"

    print "**Test lookup**"
    if d.lookup(key) != val:
        raise "lookup failed"

    try:
        d.lookup("Rah Rah Rah")
    except:
        pass
    else:
        raise "No exception thrown"

    print "**Lookup works**"

    print "**TEst contins_key**"

    if not d.contains_key(key):
        raise "Contains key failed"
    if d.contains_key("Boo"):
        raise "Contains key failed"
    print "**Contains key works**"

    print "**Test unbind**"

    d.unbind(key)

    if d.contains_key(key):
        raise "Unbind failed"
    try:
        d.unbind(key)
    except:
        pass
    else:
        raise "No Exception thrown"

    print "**unbind works**"

    print "**Test pythonic interface**"

    if len(d) != 0:
        raise "__len__ failed"

    d.bind(key,val)

    if len(d) != 1:
        raise "__len__ failed"

    if d[key] != val:
        raise "__getitem__ failed"

    key1,val1 = curFac.next()

    d[key1] = val1

    if d[key1] != val1:
        raise "__setitem__ failed"

    del d[key1]

    if d.contains_key(key1):
        raise "__delitem__ failed"

    d.clear()

    if d.contains_key(key):
        raise "clear failed"

    d[key] = val

    if not key in d.keys():
        raise "keys failed"

    if not (key,val) in d.items():
        raise "items failed"

    if not val in d.values():
        raise "values failed"

    if not d.has_key(key):
        raise "has key failed"

    d.update({key1:val1})

    if len(d) != 2:
        raise "Update failed"

    if not d.has_key(key1):
        raise "Update failed"

    if not d.get(key1,None):
        raise "get failed"

    if d.get("Boo","Something") != "Something":
        raise "get faile"

    print "**pythonic interface works**"

    print "**test colleciton interface**"

    d._4ods_clearContents()

    ass = Collections.Association(key1,val1)

    d.insert_element(ass)

    if not d.has_key(key1):
        raise "inserT_element failed"

    if not d.contains_element(ass):
        raise "contains_element failed"

    d.remove_element(ass)

    if d.has_key(key1):
        raise "remove element failed"


    print "**colleciton interface works**"
    
    


def test():

    from Ft.Ods.MetaData import PrimitiveType

    dicts = {'LiteralDictionaryOfRepositoryObjects' : (type(''),PrimitiveType.PrimitiveType),
             'LiteralDictionaryOfValues' : (type(1),type('')),
             }

    from Ft.Ods import Database
    db = Database.Database()
    db.open(DBNAME)
    tx = db.new()

    tx.begin()

    for dict in dicts.keys():
        print "*"*10 + "Test %s" % dict + "*"*10
        m = __import__("Ft.Ods.Collections.%s" % dict).Ods.Collections
        m = getattr(m,dict)
        c = getattr(m,dict)
        i = apply(c,dicts[dict])
        test_dictionary(i)

    tx.abort()
                                            



if __name__ == '__main__':
    test()
