import os
DBNAME=os.environ.get("ODS_TEST_DB","ods:test")


from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.StorageManager import Adapters
from Ft.Ods.Collections import CollectionBase

class DummyCollection(CollectionBase.CollectionBase):

    _collectionType = Constants.CollectionTypes.SET

    def __init__(self,cid=None,subType=None,values=None):
        self.changes = []
        CollectionBase.CollectionBase.__init__(self,cid=cid,subType=subType,values=values)

    def _4ods_getSubType(self):
        return Constants.Types.STRING

    def _4ods_getChanges(self):
        return self.changes


    def __getitem__(self,i):
        return self._values[i]
    def __setitem__(self,i,v):
        self._values[i] = v
    def __delitem__(self,i):
        del self._values[i]

    def insert(self,i,v):
        self._values.insert(i,v)

    def append(self,v):
        self._values.append(v)

    def count(self,v):
        return self._values.count(v)

    def remove(self,v):
        self._values.remove(v)


def test(tester):

    tester.startGroup("Collection Base")

    #Can't have a full DB here
    tester.startTest("Init DB")
    manager = Adapters.GetManager()
    adapter = Adapters.GetAdapter()
    if manager.exists(DBNAME):
        manager.reinit(DBNAME,adapter)
    else:
        manager.create(DBNAME)
        manager.init(DBNAME)

    tester.testDone()
    

    #Test the low level Collection Base
    from Ft.Ods import Database
    db = Database.Database()
    db.open(DBNAME)


    tester.startTest("Creation")
    tx = db.new()
    tx.begin()

    c = DummyCollection(values=['1','2','3'])
    c2 = DummyCollection(values=['1','2','3'])
    c3 = DummyCollection()


    tester.testDone()

    tester.startTest('__cmp__')

    tester.testResults(1,c == ['1','2','3'],done=0,msg='cmp1')
    tester.testResults(0,c == ['1','2'],done=0,msg='cmp2')
    tester.testResults(1,c == c2,done=0,msg='cmp3')
    tester.testResults(0,c == c3,done=0,msg='cmp4')

    tester.testDone()

    tester.startTest("__len__")
    tester.testResults(3,len(c),done=0,msg='len1')
    tester.testDone()

    
    tester.startTest('__getslice__')

    tester.testResults(['1'],c[:1],done=0,msg='getslice1')
    tester.testResults(['1','2'],c[:-1],done=0,msg='getslice2')
    tester.testResults(['2','3'],c[-2:],done=0,msg='getslice3')
    tester.testResults(['2'],c[1:-1],done=0,msg='getslice4')

    tester.testDone()

    tester.startTest('__setslice__')

    c[0:0] = ['4']
    tester.testResults('4',c[0],done=0,msg='setslice1')
    c[0:3] = ['4']
    tester.testResults(['4','3'],c[:],done=0,msg='setslice2')

    tester.testDone()

    tester.startTest('__delslice__')

    del c[1:2]
    tester.testResults(['4'],c[:],done=0,msg='delslice1')

    tester.testDone()

    
    tester.startTest('__add__')

    tester.testResults(1,['4','5'] == (c + ['5']),done=0,msg='add1')
    tester.testResults(1,['4','1','2','3'] == (c + c2),done=0,msg='add2')

    tester.testDone()

    tester.startTest('__mul__')

    tester.testResults(1,['1','2','3','1','2','3'] == (c2*2),done=0,msg='mul1')

    tester.testDone()

    tester.startTest('reverse')
    c2.reverse()
    tester.testResults(1,['3','2','1'] == (c2),done=0,msg='reverse1')
    tester.testDone()

    tester.startTest('sort')
    c2.sort()
    tester.testResults(1,['1','2','3'] == (c2),done=0,msg='sort1')
    tester.testDone()

    tester.startTest('extend')
    c.extend(['a','b'])
    tester.testResults(1,['4','a','b'] == (c),done=0,msg='extend1')
    tester.testDone()

    tester.startTest("cardinality")
    tester.testResults(3,c.cardinality(),done=0,msg='cardinality1')
    tester.testDone()

    tester.startTest("is_empty")
    tester.testResults(0,c.is_empty(),done=0,msg='is_empty1')
    tester.testResults(1,c3.is_empty(),done=0,msg='is_empty2')
    tester.testDone()


    tester.startTest("contains_element")
    tester.testResults(1,c.contains_element('a'),done=0,msg='contains_element1')
    tester.testResults(0,c.contains_element('7'),done=0,msg='contains_element2')
    tester.testDone()

    tester.startTest("insert_element")
    c.insert_element('c')
    tester.testResults(1,c.contains_element('c'),done=0,msg='insert_element1')
    tester.testDone()

    tester.startTest("remove_element")
    c.remove_element('c')
    tester.testResults(0,c.contains_element('c'),done=0,msg='remove_element1')
    tester.testDone()

    tester.startTest("copy")

    c4 = c.copy(deep=0)
    c5 = c.copy(deep=1)


    tester.testResults(0,len(c4),done=0,msg='copy1')
    tester.testResults(1,c5 == c,done=0,msg='copy2')
    tester.testDone()

    tester.startTest('pop')
    rt = c.pop()
    tester.testResults('b',rt,done=0,msg='pop1')
    tester.testResults(1,['4','a'] == c,done=0,msg='pop2')

    tester.testDone()

    tx.abort()
    tester.groupDone()
    db.close()
    return

if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    test(tester)


