#!/usr/bin/env python

import string

def DumpSchema(db,tab=0):
    rt = db.query("select metaid,metaclass from _ftods_metaname where metaclass='Repository'")
    if len(rt) != 1:
        raise "Multiple Repositories Found"

    DumpDefiningScope(db, rt[0], tab)

    print "****Types****"
    rt = db.query("select metaid,metaclass from _ftods_metaname where metaclass='PrimitiveType'")
    for r in rt:
        DumpType(r, db)
    rt = db.query("select metaid,metaclass from _ftods_metaname where metaclass='ScopedNameType'")
    for r in rt:
        DumpType(r, db)
    rt = db.query("select metaid,metaclass from _ftods_metaname where metaclass='CollectionType'")
    for r in rt:
        DumpType(r, db)


def DumpDefiningScope(db,info,tab):
    print (tab * '  ') + str(info['metaclass']) + ' with metaId: ' + str(info['metaid'])
    conts = []
    if info['metaclass'] in ['Repository','Module']:
        res = db.query('select defines from _ftods_module where metaid = %s' % info['metaid'])
        conts = ArrayToList(res[0]['defines'])
    elif info['metaclass'] == 'Class':
        res = db.query('select defines,extender from _ftods_class where metaid = %s' % info['metaid'])
        if res[0]['extender']:
            print ((tab+1) * '  ') + 'Extends MetaId %s' % res[0]['extender']
        conts = ArrayToList(res[0]['defines'])
    elif info['metaclass'] == 'Attribute':
        res = db.query('select type from _ftods_attribute where metaid = %s' % info['metaid'])
        conts = [res[0]['type']]
    elif info['metaclass'] == 'Relationship':
        res = db.query('select type from _ftods_relationship where metaid = %s' % info['metaid'])
        conts = [res[0]['type']]
        pass
    elif info['metaclass'] == 'PrimitiveType':
        pass
    elif info['metaclass'] == 'ScopedNameType':
        pass
    elif info['metaclass'] == 'CollectionType':
        pass
    else:
        raise info['metaclass']
    for d in conts:
        rt = db.query('select metaid,metaclass from _ftods_metaname where metaid = %s' % d)
        DumpDefiningScope(db,rt[0],tab+1)


def DumpType(info,db,tab=0):
    print (tab * '  ') + str(info['metaclass']) + ' with metaId: ' + str(info['metaid'])
    if info['metaclass'] == 'PrimitiveType':
        res = db.query('select kind from _ftods_primitivetype where metaid = %s' % info['metaid'])
        print ((tab+1) * '  ') + 'Kind = %s' % res[0]['kind']
    elif info['metaclass'] == 'ScopedNameType':
        res = db.query('select reference from _ftods_scopednametype where metaid = %s' % info['metaid'])
        print ((tab+1) * '  ') + 'Reference MetaId = %s' % res[0]['reference']
    elif info['metaclass'] == 'CollectionType':
        res = db.query('select subtype from _ftods_metacollection where metaid = %s' % info['metaid'])
        print ((tab+1) * '  ') + 'Subtype = %s' % res[0]['subtype']
    else:
        raise info['metaclass']
    conts = []
    for d in conts:
        rt = db.query('select metaid,metaclass from _ftods_metaname where metaid = %s' % d)
        DumpTypeScope(rt[0],db,tab+1)


def ArrayToList(arr):
    arr = arr[1:-1]
    if not arr:
        return []
    return string.split(arr,',')


def DumpBindings(db):
    entries = db.query('select * from _ftods_binding;')
    for entry in entries:
        print "%s bound to class %s (oid=%s)" % (entry['name'],ResolveClassName(db,entry['classid']),entry['objid'])


def DumpExtents(db):
    entries = db.query("select * from _ftods_class");
    for entry in entries:
        print "Class %s" % entry['name']
        extents = db.query('select * from class__%s' % entry['metaid'])
        for instance in extents:
            print "\tInstance oid = %s" % instance['objid']
            print "\t\trefcount = %s" % instance['refcount']
            #print "\t\tactualClass = '%s'" % ResolveClassName(db,instance['actualclass']) 
            for attr in instance.keys():
                if attr in ['objid','refcount','actualclass']:
                    continue
                attrName,attrClass = ResolveAttrName(db,attr)
                if attrClass == 'Attribute':
                    print "\t\t%s %s = %s" % (attrClass,attrName,str(instance[attr]))
                elif attrClass == 'Relationship':
                    invClass = ResolveClassNameFromOid(db,int(instance[attr]))
                    if invClass != 'Unknown Class':
                        #1-? on this side
                        print "\t\t%s %s with %s (oid=%s)" % (attrClass,attrName,invClass,str(instance[attr]))
                    else:
                        #A collection
                        #n-?
                        rt = db.query('select * from _ftods_collection where objid = %s' % instance[attr])
                        print "\t\t%s %s contains:" %  (attrClass,attrName)
                        for r in rt:
                            t,val = ResolveCollectionValue(db,r)
                            print "\t\t\t%s %s" % (t,val)


def DumpInstance(db,name):
    rt = db.query("select * from _ftods_binding where name = '%s'" % name)
    if len(rt) == 0:
        raise "No instance called " + name
    DumpClassInstance(db,rt[0]['objid'],rt[0]['classid'])


g_displayedOids = []


def DumpClassInstance(db,oid,classId,tab=0):
    rt = db.query("select * from _ftods_metaname where metaid = %s" % str(classId))
    inst = db.query("select * from class__%s where objid = %s" % (str(classId),str(oid)))
    if inst[0]['actualclass'] != classId:
        #Dump the bottom class only
        DumpClassInstance(db,oid,inst[0]['actualclass'])

    print tab*'  ' + "Class " + rt[0]['name'] + ' (oid=%s)' % str(oid)

    if str(oid) in g_displayedOids:
        return
    g_displayedOids.append(str(oid))

    attrDict = GetAllClassAttributes(db,oid)

    for klass in attrDict.keys():
        kName,attrs,curInst = attrDict[klass]
        for attr in attrs:
            print tab* '  ' + '  Attribute %s::%s = %s' % (kName,attr['name'],str(curInst[0]['attr__%s' % attr['metaid']]))

    relDict = GetAllClassRelations(db,oid)

    for klass in relDict.keys():
        kName,rels,curInst = relDict[klass]
        for rel in rels:
            invClass = ResolveClassNameFromOid(db,int(curInst[0]['rel__%s' % rel['metaid']]))
            if invClass == 'Unknown Class':
                print '  ' * tab + '  Relationship %s::%s contains:' % (kName,rel['name'])
                rt = db.query('select * from _ftods_collection where objid = %s' % int(curInst[0]['rel__%s' % rel['metaid']]))
                for r in rt:
                    t,val = ResolveCollectionValue(db,r)
                    subOid = string.split(val[1:-1],'=')[1]
                    nameRt = db.query_field("select metaid from _ftods_metaname where name = '%s'" % t,'metaid')
                    DumpClassInstance(db,subOid,nameRt[0],tab+2)
            else:
                print '  ' * tab + '  Relationship %s::%s contains:' % (kName,rel['name'])
                nameRt = db.query_field("select metaid from _ftods_metaname where name = '%s'" % invClass,'metaid')
                DumpClassInstance(db,int(curInst[0]['rel__%s' % rel['metaid']]),nameRt[0],tab+2)


def GetAllClassAttributes(db,oid):
    rt = db.query("select metaid,name from _ftods_metaname where metaclass='Class'")
    klasses = []
    for klass in rt:
        res = db.query('select * from class__%s where objid = %s' % (klass['metaid'],oid))
        if len(res):
            klasses.append(klass)
    kDict = {}
    for k in klasses:
        attrs = db.query("select * from _ftods_metaname where definingscope = %s and metaclass = 'Attribute'" % k['metaid'])
        inst = db.query("select * from class__%s where objid = %s" % (k['metaid'],str(oid)))
        kDict[k['metaid']] = (k['name'],attrs,inst)
    return kDict


def GetAllClassRelations(db,oid):
    rt = db.query("select metaid,name from _ftods_metaname where metaclass='Class'")
    klasses = []
    for klass in rt:
        res = db.query('select * from class__%s where objid = %s' % (klass['metaid'],oid))
        if len(res):
            klasses.append(klass)
    kDict = {}
    for k in klasses:
        rels = db.query("select * from _ftods_metaname where definingscope = %s and metaclass = 'Relationship'" % k['metaid'])
        inst = db.query("select * from class__%s where objid = %s" % (k['metaid'],str(oid)))
        kDict[k['metaid']] = (k['name'],rels,inst)
    return kDict


def ResolveCollectionValue(db,entry):
    from Ft.Ods.Drivers import Postgres
    if int(entry['type']) < 20:
        #Internal type
        if int(entry['type']) == Postgres.g_odmgTypeIndices.index('string'):
            return "string",entry['stringvalue']
        elif int(entry['type']) in map(lambda x: g_odmgTypeIndices.index(x), ['short', 'long']):
            return "int",int(entry['intvalue'])
        elif int(entry['type']) in map(lambda x: g_odmgTypeIndices.index(x), ['float', 'double']):
            return "float",float(entry['floatvalue'])
        return "Unknown Internal Type","????"
    #It is a class
    return ResolveClassName(db,int(entry['type'])),' (oid=%s)' % entry['intvalue']


def ResolveClassNameFromOid(db, oid):
    rt = db.query("select metaid,name from _ftods_class")
    for klass in rt:
        res = db.query('select * from class__%s where objid = %s' % (klass['metaid'],oid))
        if len(res) and res[0]['actualclass'] and int(klass['metaid']) == int(res[0]['actualclass']):
            return klass['name']
    return "Unknown Class"


def ResolveClassName(db, metaId):
    rt = db.query('select name from _ftods_class where metaid = %s' % metaId)
    if len(rt):
        return rt[0]['name']
    return 'Unknown Class'


def ResolveAttrName(db, attr):
    res = string.split(attr,'_')
    id = res[-1]
    typ = res[0]
    rt = db.query('select metaclass from _ftods_metaname where metaid = %s' % str(id))
    mclass = rt[0]['metaclass']
    if typ == 'attr':
        rt = db.query('select name from _ftods_attribute where metaid = %s' % str(id))
        name = rt[0]['name']
    if typ == 'rel':
        rt = db.query('select name from _ftods_relationship where metaid = %s' % str(id))
        name = rt[0]['name']
    return name,mclass


def PrintUsage():
    print "dig_postgres.py -b -e -s dbName <instanceName>"
    print "Options"
    print "  -b  Print All Bindings"
    print "  -s  Print Schema"
    print "  -e  Print All Extents"

    
if __name__ == '__main__':
    import sys
    from Ft.Lib import PgDatabase
    import getopt

    #dig_postgres -s -b -e dbName <instanceName>
    #Dumps all information if no command-line options are specified
    #-s = Dump Schema
    #-b = Dumb Bindings
    #-e = Dump Extents

    optList,args = getopt.getopt(sys.argv[1:],'sbe')

    dumpBind = 0
    dumpSchema = 0
    dumpExtents = 0

    for opt in optList:
        if opt[0] == '-b':
            dumpBind = 1
        elif opt[0] == '-e':
            dumpExtents = 1
        elif opt[0] == '-s':
            dumpSchema = 1
        else:
            PrintUsage()
            sys.exit(-1)

    if len(args) == 2:
        dbName = args[0]
        instName = args[1]
    elif len(args) == 1:
        dbName = args[0]
        instName = None
    else:
        PrintUsage()
        sys.exit(-1)

    if not(dumpBind or dumpExtents or dumpSchema or instName):
        dumpBind,dumpExtents,dumpSchema = [1,1,1]

    dbName = 'ftods__' + dbName
    db = PgDatabase.Database(dbName)
    if dumpSchema:
        DumpSchema(db)
        print '\n'*2
    if dumpBind:
        DumpBindings(db)
        print '\n'*2
    if dumpExtents:
        DumpExtents(db)
        print '\n' *2
    if instName:
        DumpInstance(db,instName)
