

from Ft.Ods import StorageManager
from Ft.Ods.StorageManager import Adapters
from Ft.Ods.StorageManager.Adapters import Constants

import os, string, time
DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

class DummyFactory:
    def _4ods_endTransaction(self,*args):
        pass


class DummyEnum:
    def __init__(self,v,n):
        self._v = v
        self._n = n

class DummyBlobOwner:
    def __init__(self):
        pass

    def _4ods_setBlobId(self,name,bid):
        self.bid = bid

    def _pseudo_del(self):pass

class DummyPO:
    def __init__(self,pt,rid=0):
        self._persistentType = pt
        self.meta_kind = DummyEnum(14,'pt_short')

        self.rid = rid

    def _4ods_setRepositoryId(self,rid):
        self.rid = rid
    def _4ods_getId(self):
        return self.rid

    def _4ods_getFullTuple(self):
        return [self.rid] + self.values
    def _pseudo_del(self):pass

    _extents = ('extent',)

    _tupleTypes = (13, 41, 9, 9, 37, 39, 39, 39, 39, 39, 39, 39, 39, 41)
    _tupleNames = ('_repoId', 'meta_kind', 'name', 'comment', 'definedIn', 'collections', 'dictionaries', 'specifiers', 'unions', 'operations', 'properties', 'constants', 'type_defs', 'primitive_kind')

    values = [14,"Test Type","Comment",0,0,0,0,0,0,0,0,0,3]


class DummyCollection:

    def __init__(self,collectionType,subType,cid = 0,changes = []):
        self._collectionType = collectionType
        self.subType = subType

        self.cid = cid
        self.changes = changes

    def _4ods_getSubType(self):
        return self.subType

    def _4ods_setCid(self,cid):
        self.cid = cid


    def _4ods_getChanges(self):
        return self.changes

    def _4ods_getId(self):
        return self.cid

    def _4ods_isModified(self):
        return len(self.changes)
    def _pseudo_del(self):pass

    _persistentType = Constants.ObjectTypes.COLLECTION
    _extents = ()

def test_repo(tester):

    tester.startGroup("Storage Manager Repos")

    tester.startTest("Init DB")
    mang = Adapters.GetManager()
    adapter = Adapters.GetAdapter()

    if mang.exists(DBNAME):
        mang.reinit(DBNAME,adapter)
    else:
        mang.create(DBNAME)
        mang.init(DBNAME)


    #Force an extent in there
    db = mang.connect(DBNAME) 
    adapter.begin(db)
    adapter.addExtent(db,'extent',Constants.ObjectTypes.REPO)
    adapter.commit(db)

    tester.testDone()

    f = DummyFactory()
    sm = StorageManager.StorageManager(f,DBNAME)

    tester.startTest("Test new RepoObject")
    sm.begin()
    ro = DummyPO(Constants.ObjectTypes.REPO)
    sm._4ods_registerNewPersistentObject(ro)
    #Get the extent
    ext = sm._4ods_lookup('extent')
    tester.testResults(1,len(ext.data),done=0,msg='ext 1')
    sm.commit()

    tester.testDone()

    rid = ro._4ods_getId()

    tester.startTest("Get the object")
    sm = StorageManager.StorageManager(f,DBNAME)
    sm.begin()
    ro = sm._4ods_getRepositoryObject(rid)
    tester.testResults(0,ro == None,done=0,msg="get1")
    sm.commit()
    tester.testDone()

    tester.startTest("Check Extent")
    sm = StorageManager.StorageManager(f,DBNAME)
    sm.begin()

    ext = sm._4ods_lookup('extent')
    tester.testResults(1,len(ext.data),done=0,msg='ext 1')
    tester.testDone()


    #Delete it
    tester.startTest("Delete")
    ro = DummyPO(Constants.ObjectTypes.REPO,rid)
    sm._4ods_deletePersistentObject(ro)
    sm.commit()
    sm = StorageManager.StorageManager(f,DBNAME)
    sm.begin()
    try:
        sm._4ods_getRepositoryObject(rid)
    except:
        pass
    else:
        tester.error("No Exception")
    tester.testDone()

    tester.groupDone()

    sm.abort()
    
def test_collection(tester):

    tester.startGroup("Storage Manager Collections")

    tester.startTest("Init DB")
    mang = Adapters.GetManager()
    adapter = Adapters.GetAdapter()

    if mang.exists(DBNAME):
        mang.reinit(DBNAME,adapter)
    else:
        mang.create(DBNAME)
        mang.init(DBNAME)

    tester.testDone()


    tester.startTest("Create Collection")
    f = DummyFactory()
    sm = StorageManager.StorageManager(f,DBNAME)

    sm.begin()

    changes = [(Constants.CollectionChanges.APPEND,[(0,0),(0,1)])]

    col = DummyCollection(Constants.CollectionTypes.BAG,Constants.Types.BOOLEAN,changes=changes)

    sm._4ods_registerNewPersistentObject(col)

    sm.commit()

    tester.testDone()

    cid = col.cid
    tester.startTest("Modify Collection")
    sm = StorageManager.StorageManager(f,DBNAME)
    sm.begin()

    
    changes = [(Constants.CollectionChanges.INSERT,[(0,0)])]
    col = DummyCollection(Constants.CollectionTypes.BAG,Constants.Types.BOOLEAN,cid=cid,changes=changes)
    sm._4ods_registerModifiedPersistentObject(col)

    sm.commit()

    tester.testDone()
    tester.startTest("Delete Collection")

    sm = StorageManager.StorageManager(f,DBNAME)
    sm.begin()

    col = DummyCollection(Constants.CollectionTypes.BAG,Constants.Types.BOOLEAN,cid=cid)
    sm._4ods_deletePersistentObject(col)

    sm.commit()

    

    tester.testDone()
    tester.groupDone()


def test_blobs(tester):

    tester.startGroup('Storage Manager Blobs')
    tester.startTest("Init DB")
    mang = Adapters.GetManager()
    adapter = Adapters.GetAdapter()

    if mang.exists(DBNAME):
        mang.reinit(DBNAME,adapter)
    else:
        mang.create(DBNAME)
        mang.init(DBNAME)

    tester.testDone()

    tester.startTest("Create Blob")

    f = DummyFactory()
    sm = StorageManager.StorageManager(f,DBNAME)
    sm.begin()
    inst = DummyBlobOwner()
    sm._4ods_registerNewBlob(inst,'foo',"This is a test"*500)
    sm.commit()
    tester.testDone()

    tester.startTest("Fetch Blob")
    bid = inst.bid

    sm = StorageManager.StorageManager(f,DBNAME)
    sm.begin()
    tester.testResults(len("This is a test"*500),len(sm._4ods_readBlob(bid)),done=0,msg='len new blob')
    sm._4ods_registerModifiedBlob(bid,"Another Test")
    sm.commit()
    tester.testDone()

    tester.startTest("Modify Blob")
    sm = StorageManager.StorageManager(f,DBNAME)
    sm.begin()
    tester.testResults("Another Test",sm._4ods_readBlob(bid),done=0,msg='read 1')
    sm._4ods_deleteBlob(inst,'foo',bid)
    sm.commit()
    
    sm = StorageManager.StorageManager(f,DBNAME)
    sm.begin()
    try:
        print sm._4ods_readBlob(bid)
    except:
        pass
    else:
        tester.error("Blob not deleted")
    

    sm.abort()


def test(tester):
    test_repo(tester)
    test_collection(tester)
    test_blobs(tester)

if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    test(tester)
