import os, sys

#
#Note: While testing it might be useful to use
#
#export TRACEOUT=4ODS:4Suite.PgDatabase
#export TRACEOUT_FLAGS=LABEL
#
#Requires the database to be set up with the schema from varsity1.odl or varsity2.odl
#
#Typical test process:
#
#python ../Tools/4odb_destroy.py test
#python ../Tools/4odb_create.py test
#python basic_test1.py
#


from Ft.Ods.StorageManager.Adapters import Constants


odl_string = '''
module Test {

class Professor;

class Student (extent students)
{
	attribute string name;
	attribute unsigned short student_id;
        attribute short age;
        attribute double weight;
	readonly attribute string ssn;
	relationship Professor learns_from inverse Test::Professor :: teaches;
};

class Professor
{
	attribute string name;
	attribute unsigned short employee_id;
	attribute string ssn;
	relationship list<::Test::Student> teaches inverse Student :: learns_from;
};
};

'''
#'

from Ft import Ods

from Ft.Ods import Database
import test_util
DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

def test_init():
    test_util.DbInit(DBNAME,odl_string)
    return


def test1():
    import circular_reference_classes

    p = circular_reference_classes.ProfessorImp()
    s = circular_reference_classes.StudentImp()

    return

def test2():

    import circular_reference_classes
    db = Database.Database()
    db.open(DBNAME)

    tx = db.new()
    tx.begin()
    p = circular_reference_classes.ProfessorImp()
    s = circular_reference_classes.StudentImp()

    tx.commit()
    db.close()

def test3():
    import circular_reference_classes
    db = Database.Database()
    db.open(DBNAME)

    tx = db.new()
    tx.begin()
    s = circular_reference_classes.StudentImp()

    ext = db.lookup('ext')
    
    tx.commit()
    db.close()
    return

def test4():
    
    import circular_reference_classes
    db = Database.Database()
    db.open(DBNAME)

    tx = db.new()
    tx.begin()
    s = circular_reference_classes.StudentImp()

    db.bind(s,"Foo")

    tx.commit()
    db.close()

    del s

    return

def test5():
    
    import circular_reference_classes
    db = Database.Database()
    db.open(DBNAME)
    tx = db.new()
    tx.begin()

    p = circular_reference_classes.ProfessorImp()
    s = circular_reference_classes.StudentImp()
    s.form_learns_from(p)

    db.bind(p,"Prof")

    tx.commit()
    db.close()
    Ods.FreePersistentObject(p)

    return


def test6():
    import circular_reference_classes
    db = Database.Database()
    db.open(DBNAME)
    tx = db.new()
    tx.begin()

    p = db.lookup("Prof")

    tx.commit()
    db.close()

    Ods.FreePersistentObject(p)

    return


def test7():
    import circular_reference_classes
    db = Database.Database()
    db.open(DBNAME)
    tx = db.new()
    tx.begin()

    p = db.lookup("Prof")

    tx.commit()
    db.close()

    Ods.FreePersistentObject(p)

    return


def test8():
    import circular_reference_classes
    db = Database.Database()
    db.open(DBNAME)
    tx = db.new()
    tx.begin()

    p = db.lookup("Prof")

    tx.commit()
    db.close()

    Ods.FreePersistentObject(p)

    return


def test(tester):
    tester.startGroup("Circular Refernece")

    from Ft.Lib import Cyclops
    z = Cyclops.CycleFinder()


    tester.startTest("Test Init")
    z.run(test_init,())
    z.find_cycles(1)
    if len(z.cycles) != 0:
        z.show_cycles()
        tester.testError("Should not be cycles")
    z.clear()
    tester.testDone()

    tester.startTest("Test 1")
    z.run(test1,())
    z.find_cycles(1)
    if len(z.cycles) != 0:
        z.show_cycles()
        tester.testError("Should not be cycles")
    z.clear()
    tester.testDone()

    tester.startTest("Test 2")
    z.run(test2,())
    z.find_cycles(1)
    if len(z.cycles) != 0:
        z.show_cycles()
        tester.testError("Should not be cycles")
    z.clear()
    tester.testDone()

    tester.startTest("Test 3")
    z.run(test3,())
    z.find_cycles(1)
    if len(z.cycles) != 0:
        z.show_cycles()
        tester.testError("Should not be cycles")
    z.clear()
    tester.testDone()

    tester.startTest("Test 4")
    z.run(test4,())
    z.find_cycles(1)
    if len(z.cycles) != 0:
        z.show_cycles()
        tester.testError("Should not be cycles")
    z.clear()
    tester.testDone()

    tester.startTest("Test 5")
    z.run(test5,())
    z.find_cycles(1)
    if len(z.cycles) != 0:
        z.show_cycles()
        tester.testError("Should not be cycles")
    z.clear()
    tester.testDone()

    tester.startTest("Test 6")
    z.run(test6,())
    z.find_cycles(1)
    if len(z.cycles) != 0:
        z.show_cycles()
        tester.testError("Should not be cycles")
    z.clear()
    tester.testDone()

    tester.startTest("Test 7")
    z.run(test7,())
    z.find_cycles(1)
    if len(z.cycles) != 0:
        z.show_cycles()
        tester.testError("Should not be cycles")
    z.clear()
    tester.testDone()

    tester.groupDone()

if __name__ == '__main__':

    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1, 1)

    retVal = test(tester)
    sys.exit(retVal)


