import os

#
#Note: While testing it might be useful to use
#
#export TRACEOUT=4ODS:4Suite.PgDatabase
#export TRACEOUT_FLAGS=LABEL
#
#Requires the database to be set up with the schema from varsity1.odl or varsity2.odl
#
#Typical test process:
#
#python ../Tools/4odb_destroy.py test
#python ../Tools/4odb_create.py test
#python basic_test5.py
#


odl_string = '''
module Test {
class Other;

class Foo {

  attribute bag<boolean> bb;
  attribute set<boolean> bs;
  attribute list<boolean> bl;

  attribute bag<short> sb;
  attribute set<short> ss;
  attribute list<short> sl;

  attribute bag<string> s2b;
  attribute set<string> s2s;
  attribute list<string> s2l;
  
  attribute bag<unsigned short> usb;
  attribute set<unsigned short> uss;
  attribute list<unsigned short> usl;

  attribute bag<long> lb;
  attribute set<long> ls;
  attribute list<long> ll;
  
  attribute bag<unsigned long> ulb;
  attribute set<unsigned long> uls;
  attribute list<unsigned long> ull;
  
  attribute bag<long long> llb;
  attribute set<long long> lls;
  attribute list<long long> lll;

  attribute bag<float> fb;
  attribute set<float> fs;
  attribute list<float> fl;

  attribute bag<double> db;
  attribute set<double> ds;
  attribute list<double> dl;


//  attribute dictionary<boolean,boolean> bbd;
//  attribute dictionary<boolean,short> bsd;
//  attribute dictionary<boolean,string> bs2d;
//  attribute dictionary<boolean,unsigned short> busd;
//  attribute dictionary<boolean,long> bld;
//  attribute dictionary<boolean,unsigned long> buld;
//  attribute dictionary<boolean,long long> blld;
//  attribute dictionary<boolean,float> bfd;
//  attribute dictionary<boolean,double> bdd;

//  attribute dictionary<short,boolean> sbd;
//  attribute dictionary<short,short> ssd;
//  attribute dictionary<short,string> ss2d;
//  attribute dictionary<short,unsigned short> susd;
//  attribute dictionary<short,long> sld;
//  attribute dictionary<short,unsigned long> suld;
//  attribute dictionary<short,long long> slld;
//  attribute dictionary<short,float> sfd;
//  attribute dictionary<short,double> sdd;

//  attribute dictionary<string,boolean> s2bd;
//  attribute dictionary<string,short> s2sd;
//  attribute dictionary<string,string> s2s2d;
//  attribute dictionary<string,unsigned short> s2usd;
//  attribute dictionary<string,long> s2ld;
//  attribute dictionary<string,unsigned long> s2uld;
//  attribute dictionary<string,long long> s2lld;
//  attribute dictionary<string,float> s2fd;
//  attribute dictionary<string,double> s2dd;

//  attribute dictionary<unsigned short,boolean> usbd;
//  attribute dictionary<unsigned short,short> ussd;
//  attribute dictionary<unsigned short,string> uss2d;
//  attribute dictionary<unsigned short,unsigned short> ususd;
//  attribute dictionary<unsigned short,long> usld;
//  attribute dictionary<unsigned short,unsigned long> usuld;
//  attribute dictionary<unsigned short,long long> uslld;
//  attribute dictionary<unsigned short,float> usfd;
//  attribute dictionary<unsigned short,double> usdd;

//  attribute dictionary<long,boolean> lbd;
//  attribute dictionary<long,short> lsd;
//  attribute dictionary<long,string> ls2d;
//  attribute dictionary<long,unsigned short> lusd;
//  attribute dictionary<long,long> lld;
//  attribute dictionary<long,unsigned long> luld;
//  attribute dictionary<long,long long> llld;
//  attribute dictionary<long,float> lfd;
//  attribute dictionary<long,double> ldd;

//  attribute dictionary<unsigned long,boolean> ulbd;
//  attribute dictionary<unsigned long,short> ulsd;
//  attribute dictionary<unsigned long,string> uls2d;
//  attribute dictionary<unsigned long,unsigned short> ulusd;
//  attribute dictionary<unsigned long,long> ulld;
//  attribute dictionary<unsigned long,unsigned long> ululd;
//  attribute dictionary<unsigned long,long long> ullld;
//  attribute dictionary<unsigned long,float> ulfd;
//  attribute dictionary<unsigned long,double> uldd;

//  attribute dictionary<long long,boolean> llbd;
//  attribute dictionary<long long,short> llsd;
//  attribute dictionary<long long,string> lls2d;
//  attribute dictionary<long long,unsigned short> llusd;
//  attribute dictionary<long long,long> llld2;
//  attribute dictionary<long long,unsigned long> lluld;
//  attribute dictionary<long long,long long> lllld;
//  attribute dictionary<long long,float> llfd;
//  attribute dictionary<long long,double> lldd;

//  attribute dictionary<float,boolean> fbd;
//  attribute dictionary<float,short> fsd;
//  attribute dictionary<float,string> fs2d;
//  attribute dictionary<float,unsigned short> fusd;
//  attribute dictionary<float,long> fld;
//  attribute dictionary<float,unsigned long> fuld;
//  attribute dictionary<float,long long> flld;
//  attribute dictionary<float,float> ffd;
//  attribute dictionary<float,double> fdd;

//  attribute dictionary<double,boolean> dbd;
//  attribute dictionary<double,short> dsd;
//  attribute dictionary<double,string> ds2d;
//  attribute dictionary<double,unsigned short> dusd;
//  attribute dictionary<double,long> dld;
//  attribute dictionary<double,unsigned long> duld;
//  attribute dictionary<double,long long> dlld;
//  attribute dictionary<double,float> dfd;
//  attribute dictionary<double,double> ddd;

  relationship set<Other> os inverse Other::so;
  relationship bag<Other> ob inverse Other::bo;
  relationship list<Other> ol inverse Other::lo;
  
};


class Other {

  relationship Foo so inverse Foo::os;
  relationship Foo bo inverse Foo::ob;
  relationship Foo lo inverse Foo::ol;

};

};
'''

DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

import sys, test_util
from Ft.Ods import Database
from Ft.Ods.StorageManager.Adapters import Constants


def test_collection(tester,name,attr,vals):

    db = Database.Database()

    tester.startGroup("Test %s" % name)


    #First make sure it is empty

    tx = db.new()
    tx.begin()
    f = db.lookup("Foo")

    tester.startTest("Test Cardinality")
    tester.testResults(0,getattr(f,attr).cardinality(),done=0,msg="Empty Collection not empty")
    tester.testDone()

    #Now set it with all of the values
    tester.startTest("Test Insert")
    for val in vals:
        getattr(f,attr).insert_element(val)
    tester.testDone()

    tester.startTest("Verify Insert")
    #Verify that all values are in the collection
    contents = getattr(f,attr)
    for ctr in range(len(vals)):
        curVal = contents[ctr]
        tester.testResults(1,curVal in vals,done=0,msg="%s not in collection"%str(curVal))

    tx.commit()
    tester.testDone()

    #Now reload an retest
    tx = db.new()
    tx.begin()
    f = db.lookup("Foo")

    #Verify that all values are in the collection
    tester.startTest("Verify the write/read")
    contents = getattr(f,attr)
    for ctr in range(len(vals)):
        curVal = contents[ctr]
        tester.testResults(1,curVal in vals,done=0,msg="%s not in collection after read"%str(curVal))

    tester.testDone()

    #Remove all of the values
    tester.startTest("Remove all elements")
    for val in vals:
        getattr(f,attr).remove_element(val)
    tester.testDone()

    #Make sure it is empty
    tester.startTest("Test Cardinality")
    tester.testResults(0,getattr(f,attr).cardinality(),done=0,msg="Empty Collection not empty")
    tester.testDone()

    tx.commit()

    #Now reload an retest
    tx = db.new()
    tx.begin()
    f = db.lookup("Foo")
    
    #Make sure it is empty
    tester.startTest("Test Cardinality")
    tester.testResults(0,getattr(f,attr).cardinality(),done=0,msg="Empty Collection not empty")
    tester.testDone()
    tx.abort()

    tester.groupDone()

DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

import sys, test_util
from Ft.Ods import Database
from Ft.Ods.StorageManager.Adapters import Constants

from Ft.Ods import FtodsBindingException
from Ft.Lib import Uuid

def test(tester):

    tester.startGroup("Collections")

    test_util.DbInit(DBNAME,odl_string)

    tester.startTest('Testing Generated syntax')
    try:
        from Test import Foo
        reload(Foo)
    except:
        tester.error('Error in syntax', 1)


    tester.testDone()


    db = Database.Database()
    db.open(DBNAME)

    tx = db.new()
    tx.begin()

    #Create an instance of Foo to play with

    f = Foo.new()

    db.bind(f,"Foo")

    tx.commit()

    test_collection(tester,'Boolean Bag','bb',[0,1])
    test_collection(tester,'Boolean Set','bs',[0,1])
    test_collection(tester,'Boolean List','bl',[0,1])
    test_collection(tester,'Signed Short Bag','sb',[-100,0,100])
    test_collection(tester,'Signed Short Set','ss',[-100,0,100])
    test_collection(tester,'Signed Short List','sl',[-100,0,100])
    test_collection(tester,'String Bag','s2b',["Hello","","Long String"*100])
    test_collection(tester,'String Set','s2s',["Hello","","Long String"*100])
    test_collection(tester,'String List','s2l',["Hello","","Long String"*100])
    test_collection(tester,'Unsigned Short Bag','usb',[0,100,1000])
    test_collection(tester,'Unsigned Short Set','uss',[0,100,1000])
    test_collection(tester,'Unsigned Short List','usl',[0,100,1000])
    test_collection(tester,'Signed Long Bag','lb',[-100,0,100])
    test_collection(tester,'Signed Long Set','ls',[-100,0,100])
    test_collection(tester,'Signed Long List','ll',[-100,0,100])
    test_collection(tester,'Unsigned Long Bag','ulb',[0,100,1000])
    test_collection(tester,'Unsigned Long Set','uls',[0,100,1000])
    test_collection(tester,'Unsigned Long List','ull',[0,100,1000])
    test_collection(tester,'Signed Long Long Bag','llb',[-100,0,100])
    test_collection(tester,'Signed Long Long Set','lls',[-100,0,100])
    test_collection(tester,'Signed Long Long List','lll',[-100,0,100])
    test_collection(tester,'Float Bag','fb',[0.23,100.2,1000.1])
    test_collection(tester,'Float Set','fs',[0.23,100.2,1000.1])
    test_collection(tester,'Float List','fl',[0.23,100.2,1000.1])
    test_collection(tester,'Double Bag','db',[0.23,100.2,1000.1])
    test_collection(tester,'Double Set','ds',[0.23,100.2,1000.1])
    test_collection(tester,'Double List','dl',[0.23,100.2,1000.1])

    db.close()

if __name__ == '__main__':

    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1, 1)

    retVal = test(tester)
    sys.exit(retVal)


