

odl_string = '''
module Test {

class Professor;

class Student (extent students)
{
	relationship Professor learns_from inverse Test::Professor :: teaches;
};

class Professor
{
	relationship ::Test::Student teaches inverse Student :: learns_from;
};
};

'''
#'

import test_util,os,sys

DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

test_util.DbInit(DBNAME,odl_string)

from Test import Professor
from Test import Student
reload(Professor)
reload(Student)

g_professors = 0
g_students = 0

class ProfessorImp(Professor.Professor_stub):

    def __init__(self):
        Professor.Professor_stub.__init__(self)
        
        global g_professors
        g_professors = g_professors + 1

    def __del__(self):
        global g_professors
        g_professors = g_professors - 1

class StudentImp(Student.Student_stub):

    def __init__(self):
        Student.Student_stub.__init__(self)
        
        global g_students
        g_students = g_students + 1

    def __del__(self):
        global g_students

        g_students = g_students - 1


def test(tester):
    from Ft.Ods import Database

    from Ft.Lib import Cyclops
    z = Cyclops.CycleFinder()

    #Create an instance and see if it goes away

    db = Database.Database()
    db.open(DBNAME)

    tester.startGroup("Memory")
    
    tester.startTest("Creation")
    tx = db.new()
    tx.begin()

    z.register(tx)

    p = ProfessorImp()

    tx.commit()

    p = None

    tester.testResults(0,g_professors,done=0,msg="Wrong number or Professors")

    z.find_cycles()
    if len(z.cycles) != 0:
        z.show_cycles()
        tester.testError("Should not be cycles")

    tester.testDone()

    tester.startTest("Do it with a relationship")
    z2 = Cyclops.CycleFinder()

    tx = db.new()
    tx.begin()

    z2.register(tx)

    p = ProfessorImp()
    s = StudentImp()

    s.learns_from = p

    tx.commit()

    s.drop_learns_from(p)
    p = None
    s = None

    tester.testResults(0,g_professors,done=0,msg="Wrong number or Professors")
    tester.testResults(0,g_students,done=0,msg="Wrong number of Students")
    z2.find_cycles()
    if len(z2.cycles) != 0:
        z2.show_cycles()
        tester.testError("Should not be cycles")

    tester.testDone()

    tester.groupDone()

    db.close()






if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1, 1)

    retVal = test(tester)
    sys.exit(retVal)




