
import os
DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

from Ft.Ods import PersistentObject

from Ft.Ods import Database
from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.StorageManager import Adapters
from Ft.Ods.PersistentObject import TupleDefinitions

from Ft.Ods.MetaData import MetaKind
from Ft.Ods.MetaData import PrimitiveKind

td = TupleDefinitions

class DummyCollection:
    pass

class DummyPersistentObject(PersistentObject.PersistentObject):

    _persistentType = Constants.ObjectTypes.REPO

    def __init__(self,data=None):
        PersistentObject.PersistentObject.__init__(self,data)

        if data is None:
            self.__dict__['meta_kind'] = MetaKind.MetaKind.mk_primitive_type
            self.__dict__['primitive_kind'] = PrimitiveKind.PrimitiveKind.pk_short
            self.__dict__['_repoId'] = 0

    def _4ods_setRepositoryId(self,rid):
        self.__dict__['_repoId'] = rid
    def _4ods_getRepositoryId(self):
        return self.__dict__['_repoId']
    _4ods_getId = _4ods_getRepositoryId


    #Give it some things
    #Make it look like a PrimitiveType

    _tupleDefinitions = {'_repoId':{td.TYPE:13,
                                    td.READONLY:1
                                    },
                         'meta_kind':{td.TYPE:41,
                                      td.READONLY:1,
                                      td.ENUM_CLASS:MetaKind.MetaKind,
                                      },
                         'name':{td.TYPE:9},
                         'comment':{td.TYPE:9},
                         'definedIn':{td.TYPE:37,
                                      td.READONLY:1,
                                      td.RELATIONSHIP:1},
                         'collections':{td.TYPE:39,
                                        td.READONLY:1,
                                        td.COLLECTION_CLASS:DummyCollection,
                                        },
                         'dictionaries':{td.TYPE:39,
                                        td.READONLY:1,
                                        td.COLLECTION_CLASS:DummyCollection,
                                        },
                         'specifiers':{td.TYPE:39,
                                        td.READONLY:1,
                                        td.COLLECTION_CLASS:DummyCollection,
                                        },
                         'unions':{td.TYPE:39,
                                        td.READONLY:1,
                                        td.COLLECTION_CLASS:DummyCollection,
                                        },
                         'operations':{td.TYPE:39,
                                        td.READONLY:1,
                                        td.COLLECTION_CLASS:DummyCollection,
                                        },
                         'properties':{td.TYPE:39,
                                        td.READONLY:1,
                                        td.COLLECTION_CLASS:DummyCollection,
                                        },
                         'constants':{td.TYPE:39,
                                        td.READONLY:1,
                                        td.COLLECTION_CLASS:DummyCollection,
                                        },
                         'type_defs':{td.TYPE:39,
                                        td.READONLY:1,
                                        td.COLLECTION_CLASS:DummyCollection,
                                        },
                         'primitive_kind':{td.TYPE:41,
                                      td.READONLY:1,
                                      td.ENUM_CLASS:PrimitiveKind.PrimitiveKind,
                                      },
                         }
                                       

    
    _tupleNames = ('_repoId','meta_kind','name','comment','definedIn','collections', 'dictionaries', 'specifiers', 'unions', 'operations', 'properties', 'constants', 'type_defs', 'primitive_kind')

    _localExtents = ()

from Ft.Ods import MetaData
MetaData.SetClassData(DummyPersistentObject)

def test(tester):

    tester.startGroup("Persistent Object")

    #Can't have a full DB here
    tester.startTest("Init DB")
    manager = Adapters.GetManager()
    adapter = Adapters.GetAdapter()
    if manager.exists(DBNAME):
        manager.reinit(DBNAME,adapter)
    else:
        manager.create(DBNAME)
        manager.init(DBNAME)

    tester.testDone()
    
    tester.startTest("Create Object")

    #Create a new module, set some info and commit
    db = Database.Database()
    db.open(DBNAME)
    tx = db.new()
    tx.begin()

    d = DummyPersistentObject()

    d.name = "Test Type"
    d.comment = "Test Comment"

    tester.testResults("Test Type",d.name,done=0,msg="Name")
    tester.testResults("Test Comment",d.comment,done=0,msg="Comment")

    tx.commit()

    tester.testDone()
    tester.startTest("Read Object")
    rid = d._repoId


    tx = db.new()
    tx.begin()

    d2 = db._4ods_getPersistentObjectById(Constants.ObjectTypes.REPO,rid)

    tester.testResults("Test Type",d2.name,done=0,msg="Name")
    tester.testResults("Test Comment",d2.comment,done=0,msg="Comment")

    tester.testDone()

    tester.groupDone()

    db.close()

    return

if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    test(tester)
