import os

#
#Note: While testing it might be useful to use
#
#export TRACEOUT=4ODS:4Suite.PgDatabase
#export TRACEOUT_FLAGS=LABEL
#
#Requires the database to be set up with the schema from varsity1.odl or varsity2.odl
#
#Typical test process:
#
#python ../Tools/4odb_destroy.py test
#python ../Tools/4odb_create.py test
#python basic_test5.py
#


odl_string = '''
module Test {
class Foo {

  attribute boolean b;
  //attribute char c;
  attribute short s;
  attribute string s2;
  attribute unsigned short us;
  //attribute date d; Not Supported
  //attribute time t; Not Supported
  //attribute timestamp ts; Not Supported
  attribute long l;
  attribute unsigned long ul;
  attribute long long ll;
  attribute float f;
  attribute double d;
  //attribute octet o; Not Supported
  //attribute interval i; Not Supported
};
};
'''

DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

import sys, test_util
from Ft.Ods import Database
from Ft.Ods.StorageManager.Adapters import Constants


def test_type(tester,name,attr,vals):

    db = Database.Database()


    tester.startTest("Test %s" % name)


    for val in vals:

        tx = db.new()
        tx.begin()
        f = db.lookup("Foo")

        setattr(f,attr,val)

        tx.commit()

        tx = db.new()
        tx.begin()
        f = db.lookup("Foo")

        tester.testResults(val,getattr(f,attr),done=0,msg="Wrong value")
        tx.commit()

    tester.testDone()

from Ft.Lib import Uuid

def test(tester):

    tester.startGroup("Types")

    test_util.DbInit(DBNAME,odl_string)

    tester.startTest('Testing Generated syntax')
    try:
        from Test import Foo
        reload(Foo)
    except:
        tester.error('Error in syntax', 1)


    tester.testDone()

    db = Database.Database()
    db.open(DBNAME)

    tx = db.new()
    tx.begin()

    #Create an instance of Foo to play with

    f = Foo.new()

    db.bind(f,"Foo")

    tx.commit()


    #Test booleans
    test_type(tester,"Boolean",'b',[1,0])
    # XXX maximum int in python is 2147483647 (2**31-1)
    # NB Python Library Reference 2.1.4:
    #    Plain integers (also just called integers) are implemented
    #    using long in C, which gives them at least 32 bits of precision.
    test_type(tester,"Short",'s',[1-2**30,-20000,-1000,0,1000,20000,2**30-1])
    test_type(tester,"String",'s2',["TEST1","","A really big string"*100])
    # XXX 4ODS represents unsigned short by python integers (signed short)
    test_type(tester,"Unsigned Short",'us',[0,1000,20000,2**30-1])
    test_type(tester,"Long",'l',[-100000,-10000, -1000, 0,1000,10000,100000])
    test_type(tester,"Unsigned Long",'ul',[0,1000,10000,100000])
    test_type(tester,"Long Long",'ll',[0,1000,10000,100000,Uuid.GenerateUuid()])
    test_type(tester,"Float",'f',[ -1000.4, 0.234,1000.5])
    test_type(tester,"Double",'d',[-100000.123,-10000.234, -1000.432, 0,1000.567,10000.765,100000.423])

    db.close()
    tester.groupDone()



if __name__ == '__main__':

    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1, 1)

    retVal = test(tester)
    sys.exit(retVal)


