

def test(tester=None):
    if tester is None:
        import TestSuite
        tester = TestSuite.TestSuite()

    tester.startGroup('Core Function Library')

    tester.startTest('Creating test environment')

    from xml import xpath
    from xml.xpath import ParsedExpr
    from xml.xpath import Context

    from Ft.Lib import boolean

    import DomTree
    context1 = Context.Context(DomTree.CHILD1,1,3)
    context2 = Context.Context(DomTree.CHILD2,2,3)

    import DummyExpr
    from DummyExpr import boolT, boolF
    from DummyExpr import num1, num3, numN4, num4p5
    from DummyExpr import nodeset0, nodeset1, nodeset2, nodeset3, nodeset4
    from DummyExpr import strPi, strText, strPiText, strSpace

    translateFrom = DummyExpr.DummyStringExpr('abcdefg')
    translateTo = DummyExpr.DummyStringExpr('ABCDEFG')

    tests = [('last', [], context1, 3),
             ('last', [], context2, 3),
             ('position', [], context1, 1),
             ('position', [], context2, 2),
             ('count', [nodeset2], context1, 2),
             ('id', [num1], context1, [DomTree.CHILD2]),
             ('local-name', [nodeset0], None, ''),
             ('local-name', [nodeset4], None, 'CHILD3'),
             ('namespace-uri', [nodeset0], None, ''),
             ('namespace-uri', [nodeset4], None, 'http://foo.com'),
             ('name', [nodeset4], None, 'foo:CHILD3'),
             ('string', [nodeset3], None, '\n    Text1\n  '),
             ('concat', [nodeset3, strPi, strText], None, '\n    Text1\n  3.14Hi'),
             ('starts-with', [nodeset3, strPi], None, boolean.false),
             ('starts-with', [nodeset3, nodeset3], None, boolean.true),
             ('contains', [nodeset3, strPi], None, boolean.false),
             ('contains', [nodeset3, nodeset3], None, boolean.true),
             ('substring-before', [strPiText, strText], None, '3.14'),
             ('substring-after', [strPiText, strPi], None, 'Hi'),
             ('substring', [strPiText, strPi], None, '14Hi'),
             ('substring', [strPiText, strPi, num1], None, '1'),
             ('string-length', [strPiText], None, 6),
             ('normalize-space', [strSpace], None, 'Ht There Mike'),
             ('translate', [strSpace, translateFrom, translateTo], None, 'Ht    \t ThErE\t   MikE'),
             ('boolean', [strPiText], None, boolean.true),
             ('not', [strPiText], None, boolean.false),
             ('true', [], None, boolean.true),
             ('false', [], None, boolean.false),
             ('number', [], context1, xpath.NaN),
             ('floor', [strPi], None, 3),
             ('ceiling', [strPi], None, 4),
             ('round', [strPi], None, 3),
             ]

    tester.testDone()

    for (name, args, context, expected) in tests:
        p = apply(ParsedExpr.ParsedFunctionCallExpr, (name, args))
        tester.startTest('Evaluating %s' % repr(p))
        result = p.evaluate(context)
        try:
            tester.compare(expected,result)
        except:
            print repr(expected)
            print repr(result)
            raise
        tester.testDone()

    tester.groupDone()

if __name__ == '__main__':
    test()
