def test(tester=None):
    if tester is None:
        import TestSuite
        tester = TestSuite.TestSuite()
        
    tester.startGroup('ParsedToken')
    
    tester.startTest('Checking syntax')
    from xml.xpath import XPath
    from xml.xpath import ParsedToken
    from xml.xpath import ParsedExpr
    from xml.xpath import ParsedPredicateList
    from xml.xpath import ParsedNodeTest
    from xml.xpath import ParsedStep
    from xml.xpath import ParsedRelativeLocationPath
    from xml.xpath import ParsedAbbreviatedRelativeLocationPath
    from xml.xpath import ParsedAbbreviatedAbsoluteLocationPath
    from xml.xpath import ParsedAbsoluteLocationPath
    from xml.xpath import ParsedLocationPath
    from xml.xpath import ParsedAxisSpecifier
    tester.testDone()
    
    tester.startTest('Creating test environment')
    token = ParsedToken.ParsedToken('BASE')
    expr = ParsedExpr.ParsedLiteralExpr('"TEST LITERAL"')
    pl = ParsedPredicateList.ParsedPredicateList([expr])
    nt = ParsedNodeTest.ParsedNodeTest(XPath.WILDCARD_NAME,'TEST WILDCARD')
    as = ParsedAxisSpecifier.ParsedAxisSpecifier(XPath.ANCESTOR)
    step = ParsedStep.ParsedStep(as,nt,pl)
    rlp = ParsedRelativeLocationPath.ParsedRelativeLocationPath(step, step)
    arlp = ParsedAbbreviatedRelativeLocationPath.ParsedAbbreviatedRelativeLocationPath(rlp, step)
    alp = ParsedAbsoluteLocationPath.ParsedAbsoluteLocationPath(rlp)
    aalp = ParsedAbbreviatedAbsoluteLocationPath.ParsedAbbreviatedAbsoluteLocationPath(arlp)

    tests = {'EXPRESSION' : [expr, 'ParsedExpression'],
             'PREDICATE_LIST' : [pl],
             'NODE_TEST' : [nt, 'ParsedNodeTest'],
             'AXIS_SPECIFIER' : [as],
             'STEP' : [step],
             'RELATIVE_LOCATION_PATH' : [rlp],
             'ABBREVIATED_RELATIVE_LOCATION_PATH' : [arlp],
             'ABSOLUTE_LOCATION_PATH' : [alp],
             'ABBREVIATED_ABSOLUTE_LOCATION_PATH' : [aalp],
             }
             
    tester.testDone()

    tester.startTest('Checking ' + token.__class__.__name__)
    if not token.isParsedToken(token):
        tester.error()
    if token.isParsedToken(2):
        tester.error()
    if not token.isa('BASE'):
        tester.error()
    if token.isa('NOTBASE'):
        tester.error()
    tester.testDone()
    
    for name,info in tests.items():
        if len(info) > 1:
            (token, test_name) = info
        else:
            token = info[0]
            test_name = token.__class__.__name__
        tester.startTest('Checking ' + test_name)
        if not token.isParsedToken(token):
            tester.error()
        if not token.isa(name):
            tester.error()
        tester.testDone()

    return tester.groupDone()

if __name__ == '__main__':
    import sys
    
    retval = test()
    sys.exit(retval)
