#Based on Andy Turk's RTF FAQ (18 June 2000).  Interesting because of the fact that his RTF consists of a grove of attributes.  Note that his xsl:value-ofs had to be changed to xsl:copy-ofs
from xml.xslt import XsltException, Error
import sys
sys.path.append("../")
import test_harness


#This one should bomb: implicit conversion from RTF to node-set
sheet_1 = """<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0"
  >

  <xsl:template match="row">
    <xsl:variable name="attrs"><xsl:copy-of select="@*"/></xsl:variable>
    <tr>
      <xsl:for-each select="../column">
        <td>
          <xsl:for-each select="@*">
            <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
          </xsl:for-each>
          <xsl:for-each select="$attrs">
            <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
          </xsl:for-each>
        </td>
      </xsl:for-each>
    </tr>
  </xsl:template>
  <xsl:template match='/'>
    <docelem>
      <xsl:apply-templates select='*'/>
    </docelem>
  </xsl:template>

</xsl:stylesheet>"""


#Do it the right way
sheet_2 = """<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0"
  >

  <xsl:template match="row">
    <xsl:variable name="attrs" select="@*"/>
    <tr>
      <xsl:for-each select="../column">
        <td>
          <xsl:for-each select="@*">
            <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
          </xsl:for-each>
          <xsl:for-each select="$attrs">
            <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
          </xsl:for-each>
        </td>
      </xsl:for-each>
    </tr>
  </xsl:template>
  <xsl:template match='/'>
    <docelem><xsl:apply-templates select='*'/></docelem>
  </xsl:template>

</xsl:stylesheet>"""


#Do it by using the node-set extension
sheet_3 = """<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:ft="http://namespaces.4suite.org/xpath/extensions"
  exclude-result-prefixes="ft"
  version="1.0"
  >

  <xsl:template match="row">
    <xsl:variable name="attrs"><xsl:copy-of select="@*"/></xsl:variable>
    <tr>
      <xsl:for-each select="../column">
        <td>
          <xsl:for-each select="@*">
            <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
          </xsl:for-each>
          <xsl:for-each select="ft:node-set($attrs)">
            <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
          </xsl:for-each>
        </td>
      </xsl:for-each>
    </tr>
  </xsl:template>
  <xsl:template match='/'>
    <docelem>
      <xsl:apply-templates select='*'/>
    </docelem>
  </xsl:template>

</xsl:stylesheet>"""


xml_source="""<matrix>
 <column c="1"/>
 <column c="2"/>
 <row r="1"/>
 <row r="2"/>
</matrix>
"""

expected = """<?xml version='1.0' encoding='UTF-8'?>
<docelem>
 
 
 <tr><td r='1' c='1'/><td r='1' c='2'/></tr>
 <tr><td r='2' c='1'/><td r='2' c='2'/></tr>
</docelem>
"""


def Test(tester):

    tester.startGroup("Borrowed: at_20000618, test 1")

    source = test_harness.FileInfo(string=xml_source)
    sty = test_harness.FileInfo(string=sheet_1)
    xtest = test_harness.XsltTester(source,
                                    [sty],
                                    "",
                                    expectedException = XsltException,
                                    exceptionValues = {'errorCode':Error.INVALID_FOREACH_SELECT},
                                    )

    res = xtest.test(tester)
    tester.groupDone()

    if not res:
        return res

    tester.startGroup("Borrowed: at_20000618, test 2")

    source = test_harness.FileInfo(string=xml_source)
    sty = test_harness.FileInfo(string=sheet_2)
    xtest = test_harness.XsltTester(source,
                                    [sty],
                                    expected,
                                    )

    res = xtest.test(tester)
    tester.groupDone()

    if not res:
        return res

    tester.startGroup("Borrowed: at_20000618, test 3")

    source = test_harness.FileInfo(string=xml_source)
    sty = test_harness.FileInfo(string=sheet_3)
    xtest = test_harness.XsltTester(source,
                                    [sty],
                                    expected,
                                    )

    res = xtest.test(tester)
    tester.groupDone()

    return res


if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    Test(tester)
