#!/usr/bin/env python
########################################################################
#
# File Name:   test.py
#
# Docs:        http://docs.4suite.com/XSLT/test_suite/test.py.html
#
"""
WWW: http://4suite.com/XSLT        e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import sys, os, time
from xml.xslt import XPatternParserBase
from Ft.Lib import TestSuite


#Of the form [('dir', [testfile,...], [ignored,..]), ...]

"""
Temporarily removed

  main:

  borrowed:
         'sm_20000304',
         'w3c_xslt_spec',

"""

g_testSpec = [
    ('',['test_avt',
         'basic_test',
         'test_apply_templates',
         'test_call_template',
         'test_choose',
         'test_command_line',
         'test_copy',
         'test_copy_of',
         'test_disoutesc',
         'test_elem_attr',
         'test_extensions_1',
         'test_extensions_2',
         'test_ft_write_file',
         'test_general_errors',
         'test_handler_override',
         'test_if',
         'test_import',
         'test_include',
         'test_key',
         'test_message',
         'test_number_sort',
         'test_output',
         'test_patterns',
         'test_pi',
         'test_resource_leak',
         'test_variable',
         'test_ws_handling',
         'test_xslfunctions',
         ],
        # Ignored .py files
        ['TestSuite',
         'test',
         'test_profile',
         'test_performance_matrix',
         'test_harness',
         # just until is done
         'test_compilied_stylesheet',
         ],
      
    ),
    ('borrowed',['ag_20000713',
                 'af_20000922',
                 'at_20000618',
                 'ar_20001110',
                 'ce_20000527',
                 'ce_20000819',
                 'cs_20000103',
                 'da_20000714',
                 'dc_19991210',
                 'dc_20000110',
                 'dc_20000210',
                 'dc_20000225',
                 'dc_20000510',
                 'dc_20000921',
                 #'dc_20001112',  Output way different
                 #'dc_20001012',  Missing stylesheet?
                 'dh_20000530',
                 'dp_20000306',
                 'dp_20000310',
                 'ed_20000428',
                 'em_20001125',
                 'ev_20000324',
                 'fc_20001125',
                 'fn_20000913',
                 'gg_20000722',
                 'gkh_20000804',
                 'gkh_20001016',
                 'js_20001016',
                 'js_20001004',
                 'js_20001012',
                 'jt_20000808',
                 'jt_20001117',
                 'jt_20001120',
                 'km_19991209',
                 'km_20000204',
                 'km_20000207',
                 'km_xmascracker',
                 'lmg_20001031',
                 'mn_19991221',
                 'mn_20001128',
                 'ms_20000329',
                 'oc_20001122',
                 'ob_20000613',
                 'rt_20000515',
                 'pt_20000805',
                 'sb_20000628',
                 'sm_19991228',
                 'sm_20000304',
                 'sm_20000509',
                 'sm_20001129',
                 'sp_20000818',
                 'spec_attribute_set',
                 'spec_namespace_alias',
                 'st_20000210',
                 'st_20000324',
                 'tm_19991120',
                 'tk_20001125',
                 'uo_20000817',
                 'uo_20000923',
                 'uo_20000929',
                 #'uo_20001030',  Duplicate xmlns declarations
                 'uo_20001208',
                 'wh_20000607',
                 #'w3c_xslt_spec',
                 'mo_20001217',
                 ],
                # Ignored .py files
                [
                 ],
     ),
    ]


def DirCheck(tester,tests,ignored):
    import glob
    tester.startTest("Dir Check")
    files = glob.glob('*.py')
    omitted = 0
    for file in files:
        # Just the base filename (no ext)
        mod_name = os.path.basename(file)[:-3]
        if mod_name not in tests and mod_name not in ignored:
            tester.warning('Warning: '+ file + ' is omitted from test list.\n')
            #sys.stderr.write('Warning: '+ file + ' is omitted from test list.\n')
            omitted = 1
    tester.testDone()
    if omitted:
        time.sleep(5)

def Test(tester,tests, force=0):
    total = 0.0
    failed = []

    for file in tests:
        m = __import__(file)
        cont = 1
        try:
            s = time.time()
            cont = m.Test(tester)
            total = total + (time.time() - s)
        except Exception, e:
            print file+":"
            cont = 0
            import traceback, cStringIO
            st = cStringIO.StringIO()
            traceback.print_exc(e,file=st)
            sys.stderr.write(st.getvalue() + '\n')
        if not cont:
            failed.append(file)
            if not force:
                sys.stderr.write("Termination due to failure of %s\n"%file)
                sys.exit(1)
    if failed:
        sys.stderr.write("The following tests failed: %s\n"%str(failed))
    return total


def Main():
    tester = TestSuite.TestSuite(1, 1)
    testsuites = map(lambda x: x[0], g_testSpec)
    test_specs = g_testSpec
    force_flag = 0
    for arg in sys.argv:
        if arg == '--force':
            force_flag = 1
        elif arg in testsuites:
            test_specs = [g_testSpec[testsuites.index(arg)]]
    total = 0.0
    curr_dir = os.getcwd()
    for (path,tests,ignored) in test_specs:
        dir = os.path.join(curr_dir,path)
        os.chdir(dir)
        tests.sort()
        tester.startGroup('Dir Check of %s' % dir)
        DirCheck(tester,tests,ignored)
        tester.groupDone()
        total = total + Test(tester,tests, force_flag)
    os.chdir(curr_dir)
    print
    print '-'*50
    print 'Total time for tests (in seconds): %0.2f' % total

if __name__ == '__main__':
    Main()
