import os, shutil, time, sys
import Ft
from xml.xslt import _4xslt

if sys.platform[:3] == 'win':
    FILE_PATH = './'
else:
    FILE_PATH = '/tmp/'

source_1 = """<?xml version = "1.0"?>
<!DOCTYPE ADDRBOOK SYSTEM "addr_book.dtd">
<ADDRBOOK>
    <ENTRY ID="pa">
        <NAME>Pieter Aaron</NAME>
        <ADDRESS>404 Error Way</ADDRESS>
        <PHONENUM DESC="Work">404-555-1234</PHONENUM>
        <PHONENUM DESC="Fax">404-555-4321</PHONENUM>
        <PHONENUM DESC="Pager">404-555-5555</PHONENUM>
        <EMAIL>pieter.aaron@inter.net</EMAIL>
    </ENTRY>
    <ENTRY ID="en">
        <NAME>Emeka Ndubuisi</NAME>
        <ADDRESS>42 Spam Blvd</ADDRESS>
        <PHONENUM DESC="Work">767-555-7676</PHONENUM>
        <PHONENUM DESC="Fax">767-555-7642</PHONENUM>
        <PHONENUM DESC="Pager">800-SKY-PAGEx767676</PHONENUM>
        <EMAIL>endubuisi@spamtron.com</EMAIL>
    </ENTRY>
    <ENTRY ID="vz">
        <NAME>Vasia Zhugenev</NAME>
        <ADDRESS>2000 Disaster Plaza</ADDRESS>
        <PHONENUM DESC="Work">000-987-6543</PHONENUM>
        <PHONENUM DESC="Cell">000-000-0000</PHONENUM>
        <EMAIL>vxz@magog.ru</EMAIL>
    </ENTRY>
</ADDRBOOK>"""

sheet_1 = """<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
version="1.0">

  <xsl:strip-space elements='*'/>

  <xsl:param name='title' select='"Untitled"'/>
  
  <xsl:template match="/">
    <HTML>
    <HEAD><TITLE><xsl:value-of select='$title'/></TITLE>
    </HEAD>
    <BODY>
    <TABLE><xsl:apply-templates/></TABLE>
    </BODY>
    </HTML>
  </xsl:template>

  <xsl:template match="ENTRY">
        <xsl:element name='TR'>
        <xsl:apply-templates select='NAME'/>
        </xsl:element>
        <xsl:if test='not(position()=last())'><HR/></xsl:if>
  </xsl:template>

  <xsl:template match="NAME">
    <xsl:element name='TD'>
    <xsl:attribute name='ALIGN'>CENTER</xsl:attribute>
      <B><xsl:apply-templates/></B>
    </xsl:element>
  </xsl:template>

</xsl:stylesheet>
"""

expected_0 = _4xslt.g_usage % (Ft.__version__, '4xslt') + '\n'

expected_1 = """<HTML>
  <HEAD>
    <meta http-equiv='Content-Type' content='text/html; charset=UTF-8'>
    <TITLE>Untitled</TITLE>
  </HEAD>
  <BODY>
    <TABLE>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Pieter Aaron</B>
        </TD>
      </TR>
      <HR>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Emeka Ndubuisi</B>
        </TD>
      </TR>
      <HR>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Vasia Zhugenev</B>
        </TD>
      </TR>
    </TABLE>
  </BODY>
</HTML>
"""

expected_2 = """<HTML>
  <HEAD>
    <meta http-equiv='Content-Type' content='text/html; charset=UTF-8'>
    <TITLE>Untitled</TITLE>
  </HEAD>
  <BODY>
    <TABLE>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Pieter Aaron</B>
        </TD>
      </TR>
      <HR>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Emeka Ndubuisi</B>
        </TD>
      </TR>
      <HR>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Vasia Zhugenev</B>
        </TD>
      </TR>
    </TABLE>
  </BODY>
</HTML>
"""

expected_3 = """<HTML>
  <HEAD>
    <meta http-equiv='Content-Type' content='text/html; charset=UTF-8'>
    <TITLE>Address Book</TITLE>
  </HEAD>
  <BODY>
    <TABLE>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Pieter Aaron</B>
        </TD>
      </TR>
      <HR>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Emeka Ndubuisi</B>
        </TD>
      </TR>
      <HR>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Vasia Zhugenev</B>
        </TD>
      </TR>
    </TABLE>
  </BODY>
</HTML>
"""

import sys

def Test(tester):
    xml_file = FILE_PATH + '4xsltcmdlinetest.xml'
    xslt_file = FILE_PATH + '4xsltcmdlinetest.xslt'
    output_file = FILE_PATH + '4xsltcmdlinetestoutput.xml'
    f = open(xml_file, 'w')
    f.write(source_1)
    f.close()
    f = open(xslt_file, 'w')
    f.write(sheet_1)
    f.close()
    tester.startGroup("test_command_line")

    tester.startTest("test 0")
    pipe = os.popen('4xslt')
    result = pipe.read()
    pipe.close()
    end = time.time()
    tester.testResults(expected_0, result, done=0)
    tester.testDone()

    if sys.platform[:3] != 'win':

        tester.startTest("test 1")
        pipe = os.popen('4xslt %s %s'%(xml_file, xslt_file))
        result = pipe.read()
        pipe.close()
        end = time.time()
        tester.testResults(expected_1,result,done=0)
        tester.testDone()

    if os.path.exists(output_file):
        os.unlink(output_file)


    tester.startTest("test 2")
    os.system('4xslt -o %s %s %s'%(output_file, xml_file, xslt_file))
    end = time.time()
    f = open(output_file, 'r')
    result =  f.read()
    f.close()
    os.unlink(output_file)
    tester.testResults(expected_2,result,done=0)
    tester.testDone()

    if sys.platform[:3] != 'win':
        tester.startTest("test 3")
        pipe = os.popen('4xslt -D "title=Address Book" %s %s'%(xml_file, xslt_file))
        result = pipe.read()
        pipe.close()
        end = time.time()
        tester.testResults(expected_3,result,done=0)
        tester.testDone()

    tester.groupDone()
    return 1

if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    Test(tester)

