import string, os, sys, test_harness
import xml.dom.ext
from xml.xslt import XsltElement, AttributeValueTemplate

class SystemElement(XsltElement):
    def __init__(self, doc, uri='http://foo.org/namespaces/ext-xslt', localName='system', prefix='xsl', baseUri=''):
        XsltElement.__init__(self, doc, uri, localName, prefix, baseUri)

    def setup(self):
        self.__dict__['_nss'] = xml.dom.ext.GetAllNs(self)
        self.__dict__['_command'] = AttributeValueTemplate.AttributeValueTemplate(self.getAttributeNS('', 'command'))
        return

    def instantiate(self, context, processor):
        origState = context.copy()
        context.setNamespaces(self._nss)
        command = self._command.evaluate(context)
        os.system(command)
        context.set(origState)
        return (context,)

    def __getinitargs__(self):
        return (None, self.namespaceURI, self.localName, self.prefix,
                self.baseUri)

    def __getstate__(self):
         base_state = XsltElement.__getstate__(self)
         new_state = (base_state, self._nss, self._command)
         return new_state

    def __setstate__(self, state):
        XsltElement.__setstate__(self, state[0])
        self._nss = state[1]
        self._command = state[2]
        return

ExtElements = {
    ('http://foo.org/namespaces/ext-xslt', 'system'): SystemElement
    }


sheet_str_1 = """<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:ext="http://foo.org/namespaces/ext-xslt"
  extension-element-prefixes="ext"
  version="1.0">

  <xsl:template match="execute-command">
    <docelem>
    <ext:system command="{@cmd}"/>
    </docelem>
  </xsl:template>

</xsl:stylesheet>
"""

if sys.platform[:3] != 'win':
    xml_source_1 = """<execute-command cmd="touch 'foo'"/>"""
    xml_source_2 = """<execute-command cmd="rm -f 'foo'"/>"""
else:
    xml_source_1 = """<execute-command cmd="echo '1' > foo"/>"""
    xml_source_2 = """<execute-command cmd="del foo"/>"""
    
expected_result_1 = """<?xml version='1.0' encoding='UTF-8'?>
<docelem/>
"""
expected_result_2 = expected_result_1



def Test(tester):
    tester.startGroup("Extensions 2, test 1")

    if os.path.exists("foo"):
        os.unlink("foo")

    source = test_harness.FileInfo(string=xml_source_1)
    sty = test_harness.FileInfo(string=sheet_str_1)
    xtest = test_harness.XsltTester(source,
                                    [sty],
                                    expected_result_1,
                                    extensionModules = ['test_extensions_2'],
                                    )

    res = xtest.test(tester)
    tester.startTest("Test File results") 
    tester.testResults(1,os.path.exists("foo"),done=0)
    tester.testDone()
    tester.groupDone()
    if not res:
        if os.path.exists("foo"):
            os.unlink("foo")
        return res


    tester.startGroup("Extensions 2, test 2")
    source = test_harness.FileInfo(string=xml_source_2)
    sty = test_harness.FileInfo(string=sheet_str_1)
    xtest = test_harness.XsltTester(source,
                                    [sty],
                                    expected_result_2,
                                    extensionModules = ['test_extensions_2'],
                                    )

    res = xtest.test(tester)
    tester.startTest("Test File resuilts") 
    tester.testResults(0,os.path.exists("foo"),done=0)
    tester.testDone()
    tester.groupDone()

    return res


    dir(xml.xslt.Processor)
    processor = xml.xslt.Processor.Processor()
    processor.registerExtensionModules(['test_extensions_2'])
    processor.appendStylesheetString(sheet_str_1)
    result = processor.runString(xml_source_1, ignorePis=1)
    print result


if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    Test(tester)

