#Test the overidding of the processor handlers
#This will also test baseUri resolution for includes, imports and documents (some see test_baseUris for more testing)

from xml.xslt import StylesheetReader
from xml.xslt.Processor import Processor
from xml.xslt import pDomlette
from Ft.Lib import Uri
import test_harness, time
import urlparse

BASE_URI = "http://foo.com/"

URIS={BASE_URI+'DOC1':"""<?xml version='1.0' encoding='UTF-8'?><root>DOC1</root>""",
      BASE_URI+'DOC2':"""<?xml version='1.0' encoding='UTF-8'?><root>DOC2</root>""",
      BASE_URI+'STY1':"""<?xml version='1.0' encoding='UTF-8'?><xsl:stylesheet version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'><xsl:template match='/root'>STY1<xsl:value-of select='.'/></xsl:template></xsl:stylesheet>""",
      BASE_URI+'STY2':"""<?xml version='1.0' encoding='UTF-8'?>
      <xsl:stylesheet version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>
        <xsl:include href='STY1'/>
        <xsl:template match='/root'>
          STY2<xsl:value-of select='.'/>
        </xsl:template>
      </xsl:stylesheet>""",
      BASE_URI+'STY3':"""<?xml version='1.0' encoding='UTF-8'?>
      <xsl:stylesheet version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>
        <xsl:import href='STY1'/>
        <xsl:template match='/root'>
          STY3<xsl:value-of select='.'/>
        </xsl:template>
      </xsl:stylesheet>""",
      BASE_URI+'STY4':"""<?xml version='1.0' encoding='UTF-8'?>
      <xsl:stylesheet version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>
        <xsl:template match='/root'>
          STY4<xsl:value-of select='document("DOC2")'/>
        </xsl:template>
      </xsl:stylesheet>""",
      }


class NewStylesheetReader(StylesheetReader.StylesheetReader):

    def fromUri(self, uri, baseUri='', ownerDoc=None, stripElements=None):
        scheme = urlparse.urlparse(uri)[0]
        if scheme in ['', 'http', 'ftp', 'file', 'gopher']:
            uri = urlparse.urljoin(baseUri, uri)
        return self.fromString(URIS[uri], uri, ownerDoc, stripElements)

class NewDocumentReader(pDomlette.PyExpatReader):

    def fromUri(self, uri, baseUri='', ownerDoc=None, stripElements=None):
        scheme = urlparse.urlparse(uri)[0]
        if scheme in ['', 'http', 'ftp', 'file', 'gopher']:
            uri = urlparse.urljoin(baseUri, uri)
        return self.fromString(URIS[uri], uri, ownerDoc, stripElements)


DOC1_STY1_EXPECTED="""<?xml version='1.0' encoding='UTF-8'?>
STY1DOC1"""
DOC1_STY2_EXPECTED="""<?xml version='1.0' encoding='UTF-8'?>
STY1DOC1"""
DOC1_STY3_EXPECTED="""<?xml version='1.0' encoding='UTF-8'?>

          STY3DOC1"""

DOC1_STY4_EXPECTED="""<?xml version='1.0' encoding='UTF-8'?>

          STY4DOC2"""


def TestSimpleUri(tester):

    tester.startGroup("Handler Overide, Simple Uri")
    tester.startTest("Uri Resolution")
    processor = Processor()
    processor.setStylesheetReader(NewStylesheetReader())
    processor.setDocumentReader(NewDocumentReader())
    processor.appendStylesheetUri(BASE_URI+'STY1')
    start = time.time()
    result = processor.runUri(BASE_URI+'DOC1')
    end = time.time()
    tester.testResults(DOC1_STY1_EXPECTED,result,done=0)
    tester.testDone()
    tester.groupDone()
    return 1


def TestRelativeInclude(tester):
    tester.startGroup("Handler Overide, Relative Include") 
    tester.startTest("Uri Resolution")
    processor = Processor()
    processor.setStylesheetReader(NewStylesheetReader())
    processor.setDocumentReader(NewDocumentReader())
    processor.appendStylesheetUri(BASE_URI+'STY2')
    start = time.time()
    result = processor.runUri(BASE_URI+'DOC1')
    end = time.time()
    tester.testResults(DOC1_STY2_EXPECTED,result,done=0)
    tester.testDone()
    tester.groupDone()
    return 1


def TestRelativeImport(tester):
    tester.startGroup("Handler Overide, Relative Import")
    tester.startTest("Uri Resolution")
    processor = Processor()
    processor.setStylesheetReader(NewStylesheetReader())
    processor.setDocumentReader(NewDocumentReader())
    processor.appendStylesheetUri(BASE_URI+'STY3')
    start = time.time()
    result = processor.runUri(BASE_URI+'DOC1')
    end = time.time()
    tester.testResults(DOC1_STY3_EXPECTED,result,done=0)
    tester.testDone()
    tester.groupDone()
    return 1


def TestRelativeDocument(tester):
    tester.startGroup("Handler Overide, Relative Document")
    tester.startTest("Uri Resolution")
    processor = Processor()
    processor.setStylesheetReader(NewStylesheetReader())
    processor.setDocumentReader(NewDocumentReader())
    processor.appendStylesheetUri(BASE_URI+'STY4')
    start = time.time()
    result = processor.runUri(BASE_URI+'DOC1')
    end = time.time()
    tester.testResults(DOC1_STY4_EXPECTED,result,done=0)
    tester.testDone()
    tester.groupDone()
    return 1




def Test(tester):
    res = TestSimpleUri(tester)
    if not res:
        return res
    res = TestRelativeInclude(tester)
    if not res:
        return res
    res = TestRelativeImport(tester)
    if not res:
        return res
    res = TestRelativeDocument(tester)
    return res

if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    Test(tester)
