import sys, cStringIO, os, urllib
from xml.xslt import Processor
from xml.xslt import StylesheetReader
from Ft.Lib import pDomlette
from Ft.Lib import cDomlette
from xml.dom.ext.reader import Sax
from xml.dom.ext.reader import Sax2
from Ft.Lib import Uri
from Ft.Lib import TreeCompare
from Ft.Lib import Cyclops

class FileInfo:
    def __init__(self, uri=None, string=None, validate=0, stripElements=None,
                 baseUri='', nsAware=1):
        self.uri = uri
        self.string = string
        self.validate = validate
        self.stripElements = stripElements or []
        self.baseUri = baseUri
        self.nsAware = nsAware


class XsltTester:
    """The tester will do many things to each file  all end in a XSLT process
    For Source documents
        1.  Read from URI (if possible)
        2.  Read from Stream
        3.  read from string
        4.  Read using cDomlette, pDomlette, and 4DOM (Sax,Sax2,possible HTML)
        5.  Possible use validation
    For Stylesheets
        1.  Read from URI
        2.  Read from Stream
        3.  Read from string
        4.  Use StylesheetReader and FromDocument (from cDomlette, pDomlette, and 4DOM)
        
        """

    def __init__(self,
                 source,
                 stylesheets,
                 expected,
                 cDomletteExpected=None,
                 ignorePis = 1,
                 baseUri = '',
                 topLevelParams = None,
                 writer=None,
                 extensionModules = None,
                 expectedException = None,
                 exceptionValues = None,
                 stylesheetReader = None,
                 documentReader = None,
                 compareFunc = TreeCompare.TreeCompare
                 ):
        self.expected = expected
        self.source = source
        self.stylesheets = stylesheets
        self.cDomletteExpected = cDomletteExpected
        self.ignorePis = ignorePis
        self.baseUri = baseUri
        self.topLevelParams = topLevelParams
        self.writer = writer
        self.extensionModules = extensionModules or []

        self.expectedException = expectedException != None
        self.exception = expectedException or Exception
        self.exceptionValues = exceptionValues or {}

        self.stylesheetReader = stylesheetReader
        self.documentReader = documentReader

        self.compareFunc = compareFunc

        #See if there is a --quick option on the command line
        self.quick = 0
        self.cyclops = 0
        for arg in sys.argv:
            if arg == '--quick':
                self.quick = 1
        for arg in sys.argv:
            if arg == '--quick':
                self.quick = 1
            elif arg == '--cyclops':
                self.cyclops = 1
            

    def test(self,tester,stream=sys.stdout):
        if not self.cyclops:
            return self.dotest(tester,stream)
        else:
            z = Cyclops.CycleFinder()
            res = z.run(self.dotest,(tester,stream))
            tester.startTest("Check for Leaks")
            z.find_cycles(1)
            try:
                tester.testResults(0,len(z.cycles),done=0)
            except:
                z.show_cycles()
                raise
            tester.testDone()
            return res

    def dotest(self,tester,stream=sys.stdout):

        self.stream = stream

        res = 1

        if self.source.uri:
            if not self.quick:
                res = self.testSourceUri(tester)
                if not res:
                    return res
        elif not self.source.string:
            raise "Must specify a Source String or a source URI"

        res = self.testSourceString(tester)
        if not res or self.quick:
            return res
        res = self.testStylesheets(tester)
        return res


    def testStylesheets(self,tester):
        """Test stylesheets in all forms"""


        res = self.testStylesheetUri(tester)
        if not res:
            return res

        res = self.testStylesheetString(tester)

        if not res:
            return res
        res = self.testStylesheetInstant(tester)

        return res


    def addStylesheetUri(self,processor,curSheet):
        processor.appendStylesheetUri(curSheet.uri)

    def testStylesheetUri(self,tester):
        return self._testSheet(tester,self.addStylesheetUri,"Stylesheet Uri",uri=1)

    def addStylesheetString(self,processor,curSheet):
        st = curSheet.string and curSheet.string or self._readUri(curSheet.uri)
        processor.appendStylesheetString(st)

    def addStylesheetpDomlettePyExpat(self,processor,curSheet):
        src,reader = self._pDomlettePyExpatReader(curSheet)
        try:
            processor.appendStylesheetNode(src)
        finally:
            reader.releaseNode(src)
    def addStylesheetpDomletteSax(self,processor,curSheet):
        src,reader = self._pDomletteSaxReader(curSheet)
        try:
            processor.appendStylesheetNode(src)
        finally:
            reader.releaseNode(src)
    def addStylesheetcDomletteRawExpat(self,processor,curSheet):
        src,reader = self._cDomletteRawExpatReader(curSheet)
        try:
            processor.appendStylesheetNode(src)
        finally:
            reader.releaseNode(src)
    def addStylesheetSax2(self,processor,curSheet):
        src,reader = self._sax2Reader(curSheet)
        try:
            processor.appendStylesheetNode(src)
        finally:
            reader.releaseNode(src)

    def testStylesheetString(self,tester):
        res = self._testSheet(tester,self.addStylesheetString,"Stylesheet String, appendString")
        if not res:
            return res
        res = self._testSheet(tester,self.addStylesheetpDomlettePyExpat,"Stylesheet Node, pDomlette.PyExpatReader")
        if not res:
            return res
        res = self._testSheet(tester,self.addStylesheetpDomletteSax,"Stylesheet Node, pDomlette.SaxReader")
        if not res:
            return res
        res = self._testSheet(tester,self.addStylesheetcDomletteRawExpat,"Stylesheet Node, cDomlette.RawExpatReader")
        if not res:
            return res
        res = self._testSheet(tester,self.addStylesheetSax2,"Stylesheet Node, Sax2.Reader")

        return res

    def testStylesheetInstant(self,tester):
        res = self._testSheet(tester,self.addStylesheetInstant,"Stylesheet Instant, appendInstant")
        if not res:
            return res
        res = self._testSheet(tester,self.addStylesheetpDomlettePyExpatInstant,"Stylesheet Instant, pDomlette.PyExpatReader")
        if not res:
            return res
        res = self._testSheet(tester,self.addStylesheetpDomletteSaxInstant,"Stylesheet Instant, pDomlette.SaxReader")
        if not res:
            return res
        res = self._testSheet(tester,self.addStylesheetcDomletteRawExpatInstant,"Stylesheet Instant, cDomlette.RawExpatReader")
        if not res:
            return res
        res = self._testSheet(tester,self.addStylesheetSax2Instant,"Stylesheet Instant, Sax2.Reader")

        return res

    def addStylesheetInstant(self,processor,curSheet):
        st = curSheet.string and curSheet.string or self._readUri(curSheet.uri)
        r = StylesheetReader.StylesheetReader()
        sty = r.fromString(st)
        styInstant = StylesheetReader.CreateInstantStylesheet(sty)
        r.releaseNode(sty.ownerDocument)
        sty = StylesheetReader.FromInstant(styInstant)
        processor.appendInstantStylesheet(sty)

    def addStylesheetpDomlettePyExpatInstant(self,processor,curSheet):
        src,reader = self._pDomlettePyExpatReader(curSheet)
        try:
            sty = StylesheetReader.FromDocument(src)
        finally:
            reader.releaseNode(src.ownerDocument)
        r = StylesheetReader.StylesheetReader()
        styInstant = StylesheetReader.CreateInstantStylesheet(sty)
        r.releaseNode(sty.ownerDocument)
        sty = StylesheetReader.FromInstant(styInstant)
        processor.appendInstantStylesheet(sty)

    def addStylesheetpDomletteSaxInstant(self,processor,curSheet):
        src,reader = self._pDomletteSaxReader(curSheet)
        try:
            sty = StylesheetReader.FromDocument(src)
        finally:
            reader.releaseNode(src.ownerDocument)
        r = StylesheetReader.StylesheetReader()
        styInstant = StylesheetReader.CreateInstantStylesheet(sty)
        r.releaseNode(sty.ownerDocument)
        sty = StylesheetReader.FromInstant(styInstant)
        processor.appendInstantStylesheet(sty)
        
    def addStylesheetcDomletteRawExpatInstant(self,processor,curSheet):
        src,reader = self._cDomletteRawExpatReader(curSheet)
        try:
            sty = StylesheetReader.FromDocument(src)
        finally:
            reader.releaseNode(src.ownerDocument)
        r = StylesheetReader.StylesheetReader()
        styInstant = StylesheetReader.CreateInstantStylesheet(sty)
        r.releaseNode(sty.ownerDocument)
        sty = StylesheetReader.FromInstant(styInstant)
        processor.appendInstantStylesheet(sty)
        
    def addStylesheetSax2Instant(self,processor,curSheet):
        src,reader = self._sax2Reader(curSheet)
        try:
            sty = StylesheetReader.FromDocument(src)
        finally:
            reader.releaseNode(src.ownerDocument)
        r = StylesheetReader.StylesheetReader()
        styInstant = StylesheetReader.CreateInstantStylesheet(sty)
        r.releaseNode(sty.ownerDocument)
        sty = StylesheetReader.FromInstant(styInstant)
        processor.appendInstantStylesheet(sty)



    def _testSheet(self,tester,addFunc,title,uri=0):
        for sheetIndex in range(len(self.stylesheets)):
            curSheet = self.stylesheets[sheetIndex]

            if uri and not curSheet.uri:
                continue

            tester.startTest(title)
            processor = Processor.Processor()
            processor.registerExtensionModules(self.extensionModules)
            src = None
            if self.stylesheetReader:
                processor.setStylesheetReader(self.stylesheetReader)
            if self.documentReader:
                processor.setStylesheetReader(self.documentReader)
            try:
                try:
                    for tempIndex in range(len(self.stylesheets)):
                        if tempIndex == sheetIndex:
                            addFunc(processor,curSheet)
                        else:
                            tempSheet = self.stylesheets[tempIndex]
                            st = tempSheet.string and tempSheet.string or self._readUri(tempSheet.uri)
                            processor.appendStylesheetString(st)

                    src,reader = self._pDomlettePyExpatReader(self.source)
                    result = processor.runNode(src,ignorePis=self.ignorePis,topLevelParams=self.topLevelParams, writer=self.writer,forceStripElements = 1)
                except self.exception, e:
                    if not self.expectedException:
                        import traceback
                        traceback.print_exc(e)
                        raise e
                    self._verifyExceptionValues(e)
                else:
                    if self.expectedException:
                        raise "Expected Exception %s not raised" % self.exception

                    tester.testResults(self.expected,result,done=0,diff=1,func=self.compareFunc)
            finally:
                if src:
                    reader.releaseNode(src)
                processor.reclaim()
                
            tester.testDone()

        return 1


    def testSourceString(self,tester):
        """Test processor.runString"""
        tester.startTest("Source String, runString")
        st = self.source.string and self.source.string or self._readUri(self.source.uri)
        processor = None
        try:
            processor = self._initProcessor()
            result = processor.runString(st,ignorePis=self.ignorePis,topLevelParams=self.topLevelParams, writer=self.writer)
            processor.reclaim()
        except self.exception, e:
            if not self.expectedException:
                import traceback
                traceback.print_exc(e)
                raise e
            self._verifyExceptionValues(e)
        else:
            if self.expectedException:
                raise "Expected Exception %s not raised" % self.exception
            tester.testResults(self.expected,result,done=0,diff=1,func=self.compareFunc)

        tester.testDone()

        if self.quick:
            return 1

        tester.startTest("Source String, from pDomlette.PyExpatReader")
        try:
            processor = self._initProcessor()
            src,reader = self._pDomlettePyExpatReader(self.source)
            result = processor.runNode(src,ignorePis=self.ignorePis,topLevelParams = self.topLevelParams, writer=self.writer, baseUri = self.baseUri,forceStripElements = 1)
            reader.releaseNode(src)
            processor.reclaim()
        except self.exception, e:
            if not self.expectedException:
                import traceback
                traceback.print_exc(e)
                raise e
            self._verifyExceptionValues(e)
        else:
            if self.expectedException:
                raise "Expected Exception %s not raised" % self.exception        
            tester.testResults(self.expected,result,done=0,diff=1,func=self.compareFunc)
        tester.testDone()

        if not self.source.nsAware:
            tester.startTest("Source String, from pDomlette.SaxReader")
            try:
                processor = self._initProcessor()
                src,reader = self._pDomletteSaxReader(self.source)
                result = processor.runNode(src,ignorePis=self.ignorePis,topLevelParams = self.topLevelParams, writer=self.writer, baseUri = self.baseUri,forceStripElements = 1)
                processor.reclaim()
            except self.exception, e:
                if not self.expectedException:
                    import traceback
                    traceback.print_exc(e)
                    raise e
                self._verifyExceptionValues(e)
            else:
                if self.expectedException:
                    raise "Expected Exception %s not raised" % self.exception              
                tester.testResults(self.expected,result,done=0,diff=1,func=self.compareFunc)
                reader.releaseNode(src)
            tester.testDone()

        if not self.source.validate:
            tester.startTest("Source String, from cDomlette.RawExpatReader")
            try:
                processor = self._initProcessor()
                src,reader = self._cDomletteRawExpatReader(self.source)
                result = processor.runNode(src,ignorePis=self.ignorePis,topLevelParams = self.topLevelParams, writer=self.writer, baseUri = self.baseUri,forceStripElements = 1)
                processor.reclaim()
            except self.exception, e:
                if not self.expectedException:
                    import traceback
                    traceback.print_exc(e)
                    raise e
                self._verifyExceptionValues(e)
            else:
                if self.expectedException:
                    raise "Expected Exception %s not raised" % self.exception                      
                tester.testResults(self.cDomletteExpected and self.cDomletteExpected or self.expected,result,done=0,diff=1,func=self.compareFunc)
                reader.releaseNode(src)
            tester.testDone()

        if not self.source.nsAware:
            tester.startTest("Source String, from Sax.Reader")
            try:
                processor = self._initProcessor()
                src,reader = self._saxReader(self.source)
                result = processor.runNode(src,ignorePis=self.ignorePis,topLevelParams = self.topLevelParams, writer=self.writer, baseUri = self.baseUri,forceStripElements = 1)
                processor.reclaim()
            except self.exception, e:
                if not self.expectedException:
                    import traceback
                    traceback.print_exc(e)
                    raise e
                self._verifyExceptionValues(e)
            else:
                if self.expectedException:
                    raise "Expected Exception %s not raised" % self.exception 
                tester.testResults(self.expected,result,done=0,diff=1,func=self.compareFunc)
                reader.releaseNode(src)
            tester.testDone()

        tester.startTest("Source String, from Sax2.Reader")
        try:
            processor = self._initProcessor()
            src,reader = self._sax2Reader(self.source)
            result = processor.runNode(src,ignorePis=self.ignorePis,topLevelParams = self.topLevelParams, writer=self.writer, baseUri = self.baseUri,forceStripElements = 1)
            processor.reclaim()
        except self.exception, e:
            if not self.expectedException:
                import traceback
                traceback.print_exc(e)
                raise e
            self._verifyExceptionValues(e)
        else:
            if self.expectedException:
                raise "Expected Exception %s not raised" % self.exception 
            tester.testResults(self.expected,result,done=0,diff=1,func=self.compareFunc)
            reader.releaseNode(src)
        tester.testDone()

        return 1


    def testSourceUri(self,tester):
        """Test processor.runUri, and all of the readers"""

        tester.startTest("Source URI, runUri")
        try:
            processor = self._initProcessor()
            result = processor.runUri(self.source.uri,ignorePis=self.ignorePis,topLevelParams=self.topLevelParams, writer=self.writer)
            processor.reclaim()
        except self.exception, e:
            if not self.expectedException:
                import traceback
                traceback.print_exc(e)
                raise e
            self._verifyExceptionValues(e)
        else:
            if self.expectedException:
                raise "Expected Exception %s not raised" % self.exception             
            tester.testResults(self.expected,result,done=0,diff=1,func=self.compareFunc)
        tester.testDone()

        return 1


    def _initProcessor(self):
        """Create a processor with the stylesheets in the simplest format"""
        p = Processor.Processor()
        p.registerExtensionModules(self.extensionModules)
        if self.stylesheetReader:
            p.setStylesheetReader(self.stylesheetReader)
        if self.documentReader:
            p.setStylesheetReader(self.documentReader)

        try:
            for st in self.stylesheets:
                if st.uri:
                    p.appendStylesheetUri(st.uri)
                else:
                    p.appendStylesheetString(st.string)
        except:
            p.reclaim()
            p = None
            raise
        return p

    def _cDomletteRawExpatReader(self,fi):
        r = cDomlette.RawExpatReader()
        if fi.uri:
            src = r.fromUri(fi.uri,baseUri=fi.baseUri,stripElements=fi.stripElements)
        else:
            src = r.fromString(fi.string,baseUri=fi.baseUri,stripElements=fi.stripElements)
        return src,r

    def _pDomlettePyExpatReader(self,fi):
        r = pDomlette.PyExpatReader()
        if fi.uri:
            src = r.fromUri(fi.uri,baseUri=fi.baseUri,stripElements=fi.stripElements)
        else:
            src = r.fromString(fi.string,baseUri=fi.baseUri,stripElements=fi.stripElements)
        return src,r

    def _pDomletteSaxReader(self,fi):
        r = pDomlette.SaxReader(validate=fi.validate)
        if fi.uri:
            src = r.fromUri(fi.uri,baseUri=fi.baseUri,stripElements=fi.stripElements)
        else:
            src = r.fromString(fi.string,baseUri=fi.baseUri,stripElements=fi.stripElements)
        return src,r

    def _saxReader(self,fi):
        r = Sax.Reader(validate=fi.validate)
        if fi.uri:
            src = r.fromUri(fi.uri)
        else:
            src = r.fromString(fi.string)
        return src,r

    def _sax2Reader(self,fi):
        r = Sax2.Reader(validate=fi.validate)
        if fi.uri:
            src = r.fromUri(fi.uri)
        else:
            src = r.fromString(fi.string)
        return src,r

    def _readUri(self,uri):
        stream = Uri.BASIC_RESOLVER.resolve(uri, self.baseUri)
        rt = stream.read()
        stream.close()
        return rt
            
    def _verifyExceptionValues(self,exception):
        for name,value in self.exceptionValues.items():
            if not hasattr(exception,name) or getattr(exception,name) != value:
                import traceback
                traceback.print_exc(exception)
                raise "Expected '%s' to be '%s' on exception %s" % (name,value,exception)
                                 
        return

