import cStringIO, string, time
import test_harness

specs = {'XML' : [], 'HTML' : []}

# XML Output [1] - defaults
sheet = """<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="xml" version="1.0"/>

  <xsl:template match="/">
    <docelem>
    <foo>
      <xsl:text>]]</xsl:text>
      <xsl:text>></xsl:text>
    </foo>
    <bar>
      <xsl:text>]</xsl:text>
      <xsl:text>></xsl:text>
    </bar>
    <foobar>
      <xsl:text>&lt;&amp;]]</xsl:text>
      <xsl:text>!</xsl:text>
    </foobar>
    </docelem>
  </xsl:template>

</xsl:stylesheet>"""

source = """<foo>dummy</foo>"""

expected = """<?xml version='1.0' encoding='UTF-8'?>
<docelem><foo>]]&gt;</foo><bar>]></bar><foobar>&lt;&amp;]]!</foobar></docelem>"""

specs['XML'].append((sheet, source, expected))

# XML Output [2] - standalone no
sheet = """<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="xml" version="1.0" standalone="no"/>
<xsl:output doctype-system="test.dtd"/>

  <xsl:template match="/">
    <foo/>
  </xsl:template>

</xsl:stylesheet>"""

source = """<foo>dummy</foo>"""

expected = """<?xml version='1.0' encoding='UTF-8' standalone='no'?>
<!DOCTYPE foo SYSTEM "test.dtd">
<foo/>"""

specs['XML'].append((sheet, source, expected))

# XML Output [3] - system doctype 
sheet = """<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="xml" version="1.0" standalone="no" doctype-system="test.dtd"/>

  <xsl:template match="/">
    <foo/>
  </xsl:template>

</xsl:stylesheet>"""

source = """<foo>dummy</foo>"""

expected = """<?xml version='1.0' encoding='UTF-8' standalone='no'?>
<!DOCTYPE foo SYSTEM "test.dtd">
<foo/>"""

specs['XML'].append((sheet, source, expected))

# XML Output [4] - cdata elements
sheet = """<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:spam="http://spam.com"
  version="1.0"
>
<xsl:output method="xml" cdata-section-elements="foo spam:bar"/>

  <xsl:template match='/'>
    <docelem>
    <foo>
      <![CDATA[<foo output>]]>
    </foo>
    <spam:bar>
      <![CDATA[<bar output>]]>
    </spam:bar>
    <bar>
      <![CDATA[<plain bar output>]]>
    </bar>
    <xsl:apply-templates/>
    </docelem>
  </xsl:template>

  <xsl:template match='@*|node()' priority='-1'>
    <xsl:copy>
      <xsl:apply-templates select='@*|node()'/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>"""

source = """<out xmlns:spam="http://spam.com"><foo><![CDATA[<source foo output>]]></foo><spam:bar><![CDATA[<source bar output>]]></spam:bar></out>"""

expected = """<?xml version='1.0' encoding='UTF-8'?>
<docelem xmlns:spam='http://spam.com'><foo><![CDATA[
      <foo output>
    ]]></foo><spam:bar><![CDATA[
      <bar output>
    ]]></spam:bar><bar>
      &lt;plain bar output>
    </bar><out xmlns:spam='http://spam.com'><foo><![CDATA[<source foo output>]]></foo><spam:bar><![CDATA[<source bar output>]]></spam:bar></out></docelem>"""

specs['XML'].append((sheet, source, expected))


# HTML Output [1] - defaults
sheet = """<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="html"/>

  <xsl:template match="/">
    <html><body><p>spam &#150;&#160;&#255; eggs</p></body></html>
  </xsl:template>

</xsl:stylesheet>"""


source = """<foo>dummy</foo>"""

expected = """<html>
  <body>
    <p>spam \302\226&nbsp;&yuml; eggs</p>
  </body>
</html>"""

specs['HTML'].append((sheet, source, expected))

# HTML Output [2] - system doctype 
sheet = """<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="html" doctype-system="test.dtd"/>

  <xsl:template match="/">
    <foo/>
  </xsl:template>

</xsl:stylesheet>"""

source = """<foo>dummy</foo>"""

expected = """<!DOCTYPE foo SYSTEM "test.dtd">
<foo></foo>"""

specs['HTML'].append((sheet, source, expected))

# HTML Output [3] - system and public doctype
sheet = """<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="html" doctype-system="test.dtd" doctype-public="test.dtd"/>

  <xsl:template match="/">
    <foo/>
  </xsl:template>

</xsl:stylesheet>"""

source = """<foo>dummy</foo>"""

expected = """<!DOCTYPE foo PUBLIC "test.dtd" "test.dtd">
<foo></foo>"""

specs['HTML'].append((sheet, source, expected))


def Test(tester):
    for (method, tests) in specs.items():
        test = 1
        for (sheet, source, expected) in tests:

            tester.startGroup("Output Case %s test %d" % (method,test))
            source = test_harness.FileInfo(string=source)
            sty = test_harness.FileInfo(string=sheet)
            xtest = test_harness.XsltTester(source,
                                            [sty],
                                            expected,
                                            )
            
            res = xtest.test(tester)
            tester.groupDone()
            test = test + 1
            if not res:
                return res

    return 1

##            processor = Processor()
##            processor.appendStylesheetString(sheet)
##            start = time.time()
##            result = processor.runString(source)
##            end = time.time()
##            print '*** Result %s [%d] (time: %.3f seconds) ***' % (method, test, (end - start))
##            if result != expected:
##                raise Exception('Case %s [%d] failed.  Expected\n"""%s"""\ngot\n"""%s"""'% (method, test, repr(expected), repr(result)))
##            print result


if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    Test(tester)
