#Test basic persistent and access of DOM objects

from Ft.DbDom import Dom
from Ft.Ods import Database
from xml.dom import ext
import sys, os, cStringIO, string
from Ft.Lib import TreeCompare

DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

def test1(tester):

    tester.startGroup("Element Creation")
    db = Database.Database()
    db.open(DBNAME)

    tx = db.new()
    tx.begin()

    tester.startTest("Element Creation")
    doc = Dom.DocumentImp()

    e = doc.createElementNS("http://www.foo.com","foo:bar")

    tester.testResults(Dom.Node.ELEMENT_NODE,e.nodeType,done=0,msg="Wrong NodeType")
    tester.testResults('foo',e.prefix,done=0,msg="Wrong Prefix")
    tester.testResults('bar',e.localName,done=0,msg="Wrong LocalName")
    tester.testResults('http://www.foo.com',e.namespaceURI,done=0,msg="Wrong Namespace")
    tester.testResults('foo:bar',e.tagName,done=0,msg="Wrong Tagname")
    tester.testResults('foo:bar',e.nodeName,done=0,msg="Wrong Nodename")
    tester.testResults(doc,e.ownerDocument,done=0,msg="Wrong Owner Document")

    if db.lookup("Element"):
        db.unbind("Element")
    db.bind(e,"Element")
    if db.lookup("Document"):
        db.unbind("Document")
    db.bind(doc,"Document")

    tx.commit()

    tester.testDone()

    tester.startTest("Read from Database")

    tx = db.new()
    tx.begin()

    e = db.lookup("Element")
    doc = db.lookup("Document")

    tester.testResults(Dom.Node.ELEMENT_NODE,e.nodeType,done=0,msg="Wrong NodeType")
    tester.testResults('foo',e.prefix,done=0,msg="Wrong Prefix")
    tester.testResults('bar',e.localName,done=0,msg="Wrong LocalName")
    tester.testResults('http://www.foo.com',e.namespaceURI,done=0,msg="Wrong Namespace")
    tester.testResults('foo:bar',e.tagName,done=0,msg="Wrong Tagname")
    tester.testResults('foo:bar',e.nodeName,done=0,msg="Wrong Nodename")
    tester.testResults(doc,e.ownerDocument,done=0,msg="Wrong Owner Document")


    tx.abort()
    db.close()
    tester.testDone()
    tester.groupDone()

def test2(tester):
    #Create a small element tree
    tester.startGroup("Element Tree")

    db = Database.Database()
    db.open(DBNAME)

    tx = db.new()
    tx.begin()

    tester.startTest("Tree Creation")

    doc = Dom.DocumentImp()
    par = doc.createElementNS("http://foo.com","foo:par")
    child1 = doc.createElementNS("http://foo.com","foo:child1")
    text1 = doc.createTextNode("Some Text Here")
    child1.appendChild(text1)
    comment1 = doc.createComment("Some Comment Here")
    child1.appendChild(comment1)
    child2 = doc.createElement("child2")
    gchild = doc.createElementNS("http://foo.com","foo:gchild")
    gchild.setAttributeNS("http://foo.com","foo:attr1",'VALUE')
    gchild.setAttribute("attr2",'VALUE2')

    par.appendChild(child1)
    par.appendChild(child2)
    child2.appendChild(gchild)

    
    for v,name in [(par,'parent'),
                   (child1,'child1'),
                   (child2,'child2'),
                   (text1,'text1'),
                   (comment1,'comment1'),
                   (gchild,'gchild'),
                   ]:
        if db.lookup(name):
            db.unbind(name)
        db.bind(v,name)

    tester.testResults(2,len(par.childNodes[:]),done=0,msg="Wrong Number of children")
    tester.testResults(child1,par.childNodes[0],done=0,msg="Wrong First child")
    tester.testResults(child2,par.childNodes[1],done=0,msg="Wrong Second child")
    tester.testResults(2,len(child1.childNodes),done=0,msg="Wrong Number of children")
    tester.testResults(text1,child1.childNodes[0],done=0,msg="Wrong First child")
    tester.testResults(comment1,child1.childNodes[1],done=0,msg="Wrong Second child")
    tester.testResults(1,len(child2.childNodes),done=0,msg="Wrong Number of children")
    tester.testResults(gchild,child2.childNodes[0],done=0,msg="Wrong First child")
    tester.testResults(2,len(gchild.attributes.values()),done=0,msg="Wrong Number of attrs")
    tester.testResults(1,gchild.hasAttributeNS('http://foo.com','attr1'),done=0,msg="No foo:attr1 attribute")
    tester.testResults(1,gchild.hasAttribute('attr2'),done=0,msg="No attr2 attribute")
    tester.testResults('VALUE',gchild.getAttributeNS('http://foo.com','attr1'),done=0,msg="Wrong value for foo:attr1")
    tester.testResults('VALUE2',gchild.getAttribute('attr2'),done=0,msg="Wrong value for attr2")

    
    tx.commit()

    tester.testDone()
    tester.startTest("Read from DB")

    tx = db.new()
    tx.begin()

    par = db.lookup("parent")
    child1 = db.lookup("child1")
    child2 = db.lookup("child2")
    text1 = db.lookup("text1")
    comment1 = db.lookup("comment1")
    gchild = db.lookup("gchild")

    tester.testResults(2,len(par.childNodes),done=0,msg="Wrong Number of children")
    tester.testResults(child1,par.childNodes[0],done=0,msg="Wrong First child")
    tester.testResults(child2,par.childNodes[1],done=0,msg="Wrong Second child")
    tester.testResults(2,len(child1.childNodes),done=0,msg="Wrong Number of children")
    tester.testResults(text1,child1.childNodes[0],done=0,msg="Wrong First child")
    tester.testResults(comment1,child1.childNodes[1],done=0,msg="Wrong Second child")
    tester.testResults(1,len(child2.childNodes),done=0,msg="Wrong Number of children")
    tester.testResults(gchild,child2.childNodes[0],done=0,msg="Wrong First child")
    tester.testResults(2,len(gchild.attributes.values()),done=0,msg="Wrong Number of attrs")
    tester.testResults(1,gchild.hasAttributeNS('http://foo.com','attr1'),done=0,msg="No foo:attr1 attribute")
    tester.testResults(1,gchild.hasAttribute('attr2'),done=0,msg="No attr2 attribute")
    tester.testResults('VALUE',gchild.getAttributeNS('http://foo.com','attr1'),done=0,msg="Wrong value for foo:attr1")
    tester.testResults('VALUE2',gchild.getAttribute('attr2'),done=0,msg="Wrong value for attr2")

    tx.abort()
    db.close()
    tester.testDone()
    tester.groupDone()

    
def test3(tester,data):

    tester.startGroup("Create from data stream")
    db = Database.Database()
    db.open(DBNAME)

    tx = db.new()
    tx.begin()


    tester.startTest("Load from stream")
    from Ft.DbDom import Reader
    r = Reader.DbDomReader()
    doc = r.fromString(data)

    tester.testResults(Dom.DocumentImp.__name__,doc.__class__.__name__,done=0)

    if db.lookup("document"):
        db.unbind("document")

    db.bind(doc,"document")

    st = cStringIO.StringIO()
    ext.Print(doc,stream=st)
    res = st.getvalue()
    st.close()
    tx.commit()

    tester.testResults(string.strip(data),string.strip(res),done=0,msg="Wrong Printed results",func=TreeCompare.TreeCompare)

    tester.testDone()


    tester.startTest("Read from DB")

    tx = db.new()
    tx.begin()

    doc = db.lookup("document")
    st = cStringIO.StringIO()
    ext.PrettyPrint(doc,stream=st)
    res = st.getvalue()
    st.close()

    tx.abort()
    db.close()

    tester.testResults(Dom.DocumentImp.__name__,doc.__class__.__name__,done=0)
    tester.testResults(string.strip(data),string.strip(res),done=0,msg="Wrong Printed results",func=TreeCompare.TreeCompare)

    tester.testDone()

    tester.groupDone()

def test4(tester,data):

    tester.startGroup("Create from data stream (specified document)")
    db = Database.Database()
    db.open(DBNAME)

    tx = db.new()
    tx.begin()


    tester.startTest("Load from stream")
    from Ft.DbDom import Reader
    r = Reader.DbDomReader()
    doc = Dom.DocumentImp()
    df = r.fromString(data,ownerDoc = doc)
    tester.testResults(Dom.DocumentFragmentImp.__name__,df.__class__.__name__,done=0)
    doc.appendChild(df)

    if db.lookup("document"):
        db.unbind("document")

    db.bind(doc,"document")

    st = cStringIO.StringIO()
    ext.PrettyPrint(doc,stream=st)
    res = st.getvalue()
    st.close()
    tx.commit()

    tester.testResults(Dom.DocumentImp.__name__,doc.__class__.__name__,done=0)
    tester.testResults(string.strip(data),string.strip(res),done=0,msg="Wrong Printed results",func=TreeCompare.TreeCompare)

    tester.testDone()
    db.close()
    tester.groupDone()



def test(tester):
    test1(tester)
    test2(tester)
    data = open('addr_book1.xml').read()
    test3(tester,data)
    test4(tester,data)

if __name__ == '__main__':

    #Create a new Element.  Set some values, commit it, reload it, and see if they are there.
    #The first test will only test string attrs, child nodes and attributes are in the second test

    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1, 1)
    test(tester)
