#Test basic persistent and access of DOM objects

from Ft.DbDom import Dom
from Ft.Ods import Database
from xml.dom import ext
import sys, os
from Ft.Ods.StorageManager.Adapters import Constants

DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

def test_childNodes(tester,node,expected):
    tester.testResults(len(expected),len(node.childNodes),done=0,msg="Invalid Length")
    for ctr in range(len(expected)):
        tester.testResults(expected[ctr],node.childNodes[ctr],done=0,msg="Invalid Child")
        if ctr == 0:
            tester.testResults(None,node.childNodes[ctr].previousSibling,done=0,msg="Invalid Previous")
        else:
            tester.testResults(node.childNodes[ctr-1],node.childNodes[ctr].previousSibling,done=0,msg="Invalid Previous")
            
        if ctr == len(expected) - 1:
            tester.testResults(None,node.childNodes[ctr].nextSibling,done=0,msg="Invalid Next")
        else:
            tester.testResults(node.childNodes[ctr+1],node.childNodes[ctr].nextSibling,done=0,msg="Invalid Next")

        tester.testResults(node,node.childNodes[ctr].parentNode,done=0,msg="Invalid parentNode")


    if expected:
        tester.testResults(expected[0],node.firstChild,done=0,msg="Invalid firstChild")
        tester.testResults(expected[-1],node.lastChild,done=0,msg="Invalid lastChild")
    else:
        tester.testResults(None,node.firstChild,done=0,msg="Invalid firstChild")
        tester.testResults(None,node.lastChild,done=0,msg="Invalid lastChild")


def test_appendChild(tester):
    db = Database.Database()
    db.open(DBNAME)


    tester.startGroup("AppendChild")

    tx = db.new()
    tx.begin()

    doc = Dom.DocumentImp()

    tester.startTest("Create Nodes")
    e = doc.createElementNS("http://www.foo.com","foo:bar")
    e2 = doc.createElementNS("http://www.foo.com","foo:child")
    e3 = doc.createElementNS("http://www.foo.com","foo:child2")
    df = doc.createDocumentFragment()

    df.appendChild(e2)
    df.appendChild(e3)

    tester.testDone()

    tester.startTest("test Children")
    test_childNodes(tester,e,[])
    test_childNodes(tester,df,[e2,e3])
    tester.testDone()

    tester.startTest("test append df")
    e.appendChild(df)
    test_childNodes(tester,e,[e2,e3])
    test_childNodes(tester,df,[])
    tester.testDone()

    tx.commit()

    tester.testDone()
    tester.groupDone()
    db.close()

def test_insertBeforeChild(tester):
    db = Database.Database()
    db.open(DBNAME)


    tester.startGroup("InsertBefore")

    tx = db.new()
    tx.begin()

    doc = Dom.DocumentImp()

    tester.startTest("Create Nodes")
    e = doc.createElementNS("http://www.foo.com","foo:bar")
    e2 = doc.createElementNS("http://www.foo.com","foo:child")
    e3 = doc.createElementNS("http://www.foo.com","foo:child2")
    e4 = doc.createElementNS("http://www.foo.com","foo:child3")
    df = doc.createDocumentFragment()
    e.appendChild(e2)
    df.appendChild(e3)
    df.appendChild(e4)

    tester.testDone()

    tester.startTest("test Children")
    test_childNodes(tester,e,[e2])
    test_childNodes(tester,df,[e3,e4])
    tester.testDone()

    tester.startTest("test insertBefore")
    e.insertBefore(df,e2)
    test_childNodes(tester,e,[e3,e4,e2])
    test_childNodes(tester,df,[])
    tester.testDone()

    tx.commit()

    tester.testDone()

    tester.groupDone()
    db.close()



from Ft.DbDom import Reader

DOC="""<?xml version = "1.0"?>
<!DOCTYPE ADDRBOOK SYSTEM "addr_book.dtd">
<ADDRBOOK>
	<ENTRY ID="pa">
	</ENTRY>
</ADDRBOOK>"""


def test_reader(tester):

    db = Database.Database()
    db.open(DBNAME)

    tester.startGroup("Test reading into a DF")

    tx = db.new()
    tx.begin()

    tester.startTest("Read into a DF")

    doc = Dom.implementation.createDocument('','',None)

    r = Reader.DbDomReader()
    df = r.fromString(DOC,ownerDoc = doc)
    
    
    tester.testResults(Dom.Node.DOCUMENT_FRAGMENT_NODE,df.nodeType,done=0,msg="Node Type")
    tester.testResults(doc,df.firstChild.ownerDocument,done=0,msg="Wrong ownerDoc")

    tx.commit()
    r.releaseNode(df)
    r.releaseNode(doc)

    tester.testDone()

    db.close()


def test(tester):
    retVal = test_appendChild(tester)
    retVal = test_insertBeforeChild(tester)
    retVal = test_reader(tester)
    

if __name__ == '__main__':

    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1, 1)
    test(tester)


