#Test basic persistent and access of DOM objects

from Ft.DbDom import Dom
from Ft.Ods import Database
from xml.dom import ext
import sys, os
from Ft.Ods.StorageManager.Adapters import Constants

DBNAME=os.environ.get("ODS_TEST_DB","ods:test")

def test_childNodes(tester,node,expected):
    tester.testResults(len(expected),len(node.childNodes),done=0,msg="Invalid Length")
    for ctr in range(len(expected)):
        tester.testResults(expected[ctr],node.childNodes[ctr],done=0,msg="Invalid Child")
        if ctr == 0:
            tester.testResults(None,node.childNodes[ctr].previousSibling,done=0,msg="Invalid Previous")
        else:
            tester.testResults(node.childNodes[ctr-1],node.childNodes[ctr].previousSibling,done=0,msg="Invalid Previous")
            
        if ctr == len(expected) - 1:
            tester.testResults(None,node.childNodes[ctr].nextSibling,done=0,msg="Invalid Next")
        else:
            tester.testResults(node.childNodes[ctr+1],node.childNodes[ctr].nextSibling,done=0,msg="Invalid Next")

        tester.testResults(node,node.childNodes[ctr].parentNode,done=0,msg="Invalid parentNode")


    if expected:
        tester.testResults(expected[0],node.firstChild,done=0,msg="Invalid firstChild")
        tester.testResults(expected[-1],node.lastChild,done=0,msg="Invalid lastChild")
    else:
        tester.testResults(None,node.firstChild,done=0,msg="Invalid firstChild")
        tester.testResults(None,node.lastChild,done=0,msg="Invalid lastChild")


def test_appendChild(tester):
    db = Database.Database()
    db.open(DBNAME)


    tester.startGroup("AppendChild")

    tx = db.new()
    tx.begin()

    doc = Dom.DocumentImp()

    tester.startTest("Create Nodes")
    e = doc.createElementNS("http://www.foo.com","foo:bar")
    e2 = doc.createElementNS("http://www.foo.com","foo:child")
    e3 = doc.createElementNS("http://www.foo.com","foo:child2")

    e.appendChild(e2)

    if db.lookup("Element"):
        db.unbind("Element")

    db.bind(e,"Element")

    tester.testDone()

    tester.startTest("test Children")
    test_childNodes(tester,e,[e2])
    tester.testDone()

    tester.startTest("test 2nd append")
    e.appendChild(e3)
    test_childNodes(tester,e,[e2,e3])
    tester.testDone()

    tx.commit()

    tester.testDone()

    id2 = e2._4ods_getOid()
    id3 = e3._4ods_getOid()
    

    tester.startTest("Read it from the database")

    tx = db.new()
    tx.begin()

    e = db.lookup("Element")
    e2 = db._4ods_getPersistentObjectById(Constants.ObjectTypes.OBJECT,id2)
    e3 = db._4ods_getPersistentObjectById(Constants.ObjectTypes.OBJECT,id3)

    test_childNodes(tester,e,[e2,e3])
    tester.testDone()

    tester.startTest("Test moving Element")
    e2.appendChild(e3)
    test_childNodes(tester,e,[e2])
    test_childNodes(tester,e2,[e3])
    tester.testDone()

    tester.groupDone()
    tx.abort()
    db.close()

def test_insertBeforeChild(tester):
    db = Database.Database()
    db.open(DBNAME)


    tester.startGroup("InsertBefore")

    tx = db.new()
    tx.begin()

    doc = Dom.DocumentImp()

    tester.startTest("Create Nodes")
    e = doc.createElementNS("http://www.foo.com","foo:bar")
    e2 = doc.createElementNS("http://www.foo.com","foo:child")
    e3 = doc.createElementNS("http://www.foo.com","foo:child2")

    e.insertBefore(e2,None)

    if db.lookup("Element"):
        db.unbind("Element")

    db.bind(e,"Element")

    tester.testDone()

    tester.startTest("test Children")
    test_childNodes(tester,e,[e2])
    tester.testDone()

    tester.startTest("test 2nd append")
    e.insertBefore(e3,e2)
    test_childNodes(tester,e,[e3,e2])
    tester.testDone()

    tx.commit()

    tester.testDone()

    id2 = e2._4ods_getOid()
    id3 = e3._4ods_getOid()
    

    tester.startTest("Read it from the database")

    tx = db.new()
    tx.begin()

    e = db.lookup("Element")
    e2 = db._4ods_getPersistentObjectById(Constants.ObjectTypes.OBJECT,id2)
    e3 = db._4ods_getPersistentObjectById(Constants.ObjectTypes.OBJECT,id3)

    test_childNodes(tester,e,[e3,e2])
    tester.testDone()

    tester.startTest("Test moving node")
    e4 = doc.createElementNS("http://www.foo.com","foo:child3")
    e3.appendChild(e4)
    e.insertBefore(e4,e2)

    test_childNodes(tester,e3,[])
    test_childNodes(tester,e,[e3,e4,e2])
    
    tester.testDone()

    tester.groupDone()
    tx.abort()
    db.close()



from Ft.DbDom import Reader

DELETE_DOC="""<?xml version = "1.0"?>
<!DOCTYPE ADDRBOOK SYSTEM "addr_book.dtd">
<ADDRBOOK>
	<ENTRY ID="pa">
	</ENTRY>
</ADDRBOOK>"""


def test_delete(tester):
    from xml.xslt import Processor

    import time
    db = Database.Database()
    db.open(DBNAME)
    tx = db.new()
    tx.begin()   

    r = Reader.DbDomReader()
    doc = r.fromString(DELETE_DOC)

    if db.lookup('document'):
        db.unbind('document')
    db.bind(doc,'document')

    tx.commit()

    tx = db.new()
    tx.begin()   

    print "Load for delete"

    doc = db.lookup('document')
    print "Delete"
    doc.delete(recurse = 1)

    tx.commit()

    db.close()

def test(tester):
    test_appendChild(tester)
    test_insertBeforeChild(tester)
    test_delete(tester)
    

if __name__ == '__main__':

    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1, 1)

    test(tester)


